! **************************************************
!                   Arrays Module 
!
!                  Stewart K. Reed
!                   4th June 2007
!
! Description:
!
!   This module is simply to store arrays for the 
!   system which don't really fit else where.
!
!   Also included is a module for storing work
!   arrays so that they don't need to be reallocated
!   at every step.
! **************************************************
module arrays
  
  use coherentstatestype
  implicit none

  real(kind=8) :: t ! the current time
  real(kind=8) :: pt ! the time before now!
  integer :: psteps ! number of steps done before this run.

  real(kind=8) :: toteng ! the system energy
  real(kind=8) :: dtoteng ! change in the system energy from removing basis fcts
   
  real(kind=8),dimension(:),allocatable :: dhexpect,hework
 
  ! Coherent States Basis dimension: ndf,nbasis
!  complex(kind=8),dimension(:,:),allocatable,target::zb 
!   real(kind=8),dimension(:),allocatable,target :: S ! The action. dimension nbasis
  ! The Coefficients. dimension nbasis
!  complex(kind=8),dimension(:),allocatable,target :: C
  type(coherentstatebasisset) :: bs

  ! The  other coefficients. dimension nbasis
  complex(kind=8),dimension(:),allocatable :: D 
    
  real(kind=8) :: initnorm ! initial norm

  ! Method for dropping basis functions
  logical,dimension(:),allocatable :: incbf
  ! working array for dropping basis functions
  logical,dimension(:),allocatable :: oldincbf 

  ! A couple of numbers
  real(kind=8),parameter ::pi     = 3.1415926535897932385d0
!  real(kind=8),parameter ::sqrtpi = dsqrt(pi)
!  real(kind=8),parameter ::sqrt2  = dsqrt(2.0d0)
  real(kind=8),parameter ::sqrtpi = 1.7724538509055160273d0 !dsqrt(pi)
  real(kind=8),parameter ::sqrt2  = 1.4142135623730950488d0 !dsqrt(2.0d0)

  ! initial state(s) 
  ! ----------------
  ! number,z(ndf,ninitstates) and widths (cmprss) 
  integer :: ninitstates
  complex(kind=8),dimension(:,:), allocatable :: zinit
  real(kind=8),dimension(:,:),allocatable :: cmpress
  

  ! Arrays for setting up basis
  ! ---------------------------
  real(kind=8), dimension(:,:) , allocatable :: grid  !ndf,4
  integer, dimension(:,:) , allocatable :: ngrid      !ndf,2

  ! Widths for random Gaussian distributions
  real(kind=8) :: enmax ! Max allowable energy for basis fcts
  real(kind=8),dimension(:,:),allocatable :: gsigma 

  ! ----For Harmonic oscillator ----
  ! vibrational state
  integer,dimension(:),allocatable :: harmmethod
  integer,dimension(:),allocatable :: vstate
  real(kind=8) :: probscale
  real(kind=8) :: zmax
  integer:: maxtries ! Maximum number of tries at choosing a point.
  real(kind=8),dimension(:),allocatable :: harmComp    ! sampling compression
  ! offset of of origin for sampling. I.e. produce a distribution about this pt.
  complex(kind=8),dimension(:),allocatable :: zoffset 
  
  ! superposition of harmonic oscillator wave functions
  integer :: nsupermodes
  integer :: nmaxsuperbasis  ! Max number of functions
  integer,dimension(:),allocatable :: nsuperbasis ! number of functions for each superposition
  real(kind=8), dimension(:,:), allocatable :: superphi
  real(kind=8), dimension(:), allocatable :: superqmin,superqmax
  real(kind=8), dimension(:), allocatable :: superpmin,superpmax
  real(kind=8), dimension(:), allocatable :: supermaxprob

  ! Working arrays for the step routine
  ! -----------------------------------
  type(coherentstatebasisset) :: bs1,bs2,bs3,bs4,bsnew
  ! (ndf,nbasis)
  !complex(kind=8),dimension(:,:),allocatable :: zbnew,zk1,zk2,zk3,zk4
  ! S,C dimension(nbasis)
  !real(kind=8),dimension(:),allocatable :: Snew, Sk1,Sk2,Sk3,Sk4
  !complex(kind=8),dimension(:),allocatable :: Cnew , Ck1, Ck2,Ck3,Ck4
  complex(kind=8),dimension(:),allocatable :: Dold

  ! Working arrays for calculating D from C
  ! ---------------------------------------
  ! Overlap Matrix
  complex(kind=8),allocatable,dimension(:,:):: overlpm
  ! nbasis
  real(kind=8),dimension(:),allocatable :: FERR,BERR,RWORK
  integer,dimension(:),allocatable :: ipiv
  ! nbasis x nbasis
  complex(kind=8),dimension(:,:),allocatable::overlpmf

  ! 2*nbasis
  complex(kind=8),dimension(:),allocatable:: WORK
  integer :: LWORK ! = 2*nbasis = size(WORK)
  integer :: LRWORK ! size of RWORK

#ifdef USE_SCALAPACK
  ! ------ Extra stuff for scalapack ------
  ! length of arrays for describing arrays.
  INTEGER,PARAMETER :: SP_DLEN_ = 9
  ! Set some variables so that we can use access parts of the description array
  ! more transparently.
  INTEGER,PARAMETER :: BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DTYPE_ = 1,      &
       CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,     &
       RSRC_ = 7, CSRC_ = 8, LLD_ = 9 
  
  INTEGER,DIMENSION(SP_DLEN_) :: SP_DESC_OLAP,SP_DESC_C,SP_DESC_D

  ! Block sizes.
  INTEGER :: SP_MB,SP_NB 
  INTEGER :: SP_MXLLD_OLAP,SP_MXLLD_CD
  INTEGER :: SP_Col_OLAP, SP_Row_OLAP,SP_LLD_OLAP
  
  integer,parameter :: RSRC = 0, CSRC = 0 ! ?????????

  ! Details of local overlap array.
  ! Number of complete column blocks and row blocks
  integer :: SP_nbc,SP_nbr
  ! Number of parital blocks
  integer :: SP_ceb,SP_reb

  ! scalapack extra arrays
  complex(kind=8),dimension(:,:),allocatable :: localC, localD

  ! Stuff for general complex solver, don't think these are actually used so I don't think
  ! we actually need to allocate them. Still need to define them though
  real(kind=8),dimension(:),allocatable :: SCA_R,SCA_C

  ! Number of right hand sides
  integer :: NRHS
  ! ------ End of extra stuff for scalapack ------
#endif  
  
  ! some extra temporary arrays and variables to use when we have to use MPI1 and 
  ! therefore cannot use MPI_IN_PLACE
#ifdef USE_MPI1
  complex(kind=8), dimension(:),allocatable :: mpi_tmp_Z_array
  complex(kind=8) ::  mpi_tmp_Z
  real(kind=8), dimension(:),allocatable :: mpi_tmp_D_array
  real(kind=8) ::  mpi_tmp_D
#endif

  contains
    
    subroutine allocatearrays(ndf,nbasis,namps)
#ifdef USE_SCALAPACK
      use scalapackstuff
#endif
      implicit none
      integer, intent(in) :: ndf
      integer, intent(in) :: nbasis
      integer, intent(in) :: namps ! number of Ehrenfest trajectories
      
      ! initial states
      call  createbasisset(ndf,nbasis,namps,bs)
      !allocate(zb(ndf,nbasis))
      !zb(:,:) = 0.0
    
      allocate(incbf(nbasis),oldincbf(nbasis))
      incbf = .true.

      !allocate(S(nbasis))
      !S(:) = (0.0,0.0)

      !allocate(C(nbasis),D(nbasis))
      allocate(D(nbasis))
      !C(:) = (0.0,0.0)
      D(:) = (0.0,0.0)
      t = 0.0 ! set the time to zero
      pt = 0.0 ! previous time
      psteps = 0 ! number of steps done previously.
      
      dtoteng = 0.0 ! Total change in energy due to dropping basis functions
      
      ! Working arrays for step
      call createbasisset(ndf,nbasis,namps,bsnew)
      call createbasisset(ndf,nbasis,namps,bs1)
      call createbasisset(ndf,nbasis,namps,bs2)
      call createbasisset(ndf,nbasis,namps,bs3)
      call createbasisset(ndf,nbasis,namps,bs4)
!       allocate(zbnew(ndf,nbasis))
!       allocate(zk1(ndf,nbasis))
!       allocate(zk2(ndf,nbasis))
!       allocate(zk3(ndf,nbasis))
!       allocate(zk4(ndf,nbasis))
       
!       allocate(Snew(nbasis))
!       allocate(Sk1(nbasis))
!       allocate(Sk2(nbasis))
!       allocate(Sk3(nbasis))
!       allocate(Sk4(nbasis))

!       allocate(Cnew(nbasis))
!       allocate(Ck1(nbasis))
!       allocate(Ck2(nbasis))
!       allocate(Ck3(nbasis))
!       allocate(Ck4(nbasis))

       allocate(Dold(nbasis))

       
       ! Some temporary arrays for doing MPI_REDUCE
#ifdef USE_MPI1
       allocate(mpi_tmp_Z_array(nbasis))
       allocate(mpi_tmp_D_array(nbasis))
       ! mpi_tmp_D = 0.0d0
       ! mpi_tmp_Z = (0.0d0,0.0d0)
#endif
    end subroutine allocatearrays


    subroutine allocatelapackarrays(ndf,nbasis)
      implicit none
      integer, intent(in) :: ndf
      integer, intent(in) :: nbasis

      ! Work arrays for calculating D from C
      allocate(overlpm(nbasis,nbasis))
      overlpm(:,:) = (0.0,0.0)
      allocate(overlpmf(nbasis,nbasis))
      overlpmf(:,:) = (0.0,0.0)
      
      allocate(FERR(nbasis),BERR(nbasis),RWORK(nbasis))
      allocate(WORK(2*nbasis))
       LWORK = size(WORK)
       allocate(ipiv(nbasis))
       
     end subroutine allocatelapackarrays
     

     subroutine allocatescalapackarrays(ndf,nbasis)
#ifndef USE_SCALAPACK
       implicit none
       integer, intent(in) :: ndf
       integer, intent(in) :: nbasis
#else
       use scalapackstuff
       implicit none

       integer, intent(in) :: ndf
       integer, intent(in) :: nbasis
       
      integer :: info    ! error code from scalapack routines
      real(kind=8) :: rcond
      character :: EQUED ! For calling solver
      integer,external :: numroc ! function to work out array size.

       ! Calculate maximum size of overlap matrix
       !  SP_MXLLD_OLAP = floor (dble(M)/(dble(min(nprow,npcol))))+1
       ! The leading dimension i.e.
       !serial test
       SP_Col_OLAP = NUMROC( nbasis, SP_NB, MYCOL, 0, NPCOL ) 
       SP_Col_OLAP = max(SP_Col_OLAP,1)
       
       SP_Row_OLAP =  NUMROC( nbasis, SP_MB, MYROW, 0, NPROW )
       SP_LLD_OLAP = max(SP_Row_OLAP,1) 
       
       CALL DESCINIT( SP_DESC_OLAP, nbasis, nbasis, SP_MB, SP_NB, RSRC, CSRC, ICTXT, SP_LLD_OLAP,INFO )

       allocate(overlpm(SP_Row_OLAP,SP_Col_OLAP))
       overlpm(:,:) = (0.0,0.0)
       allocate(overlpmf(SP_Row_OLAP,SP_Col_OLAP))
       overlpmf(:,:) = (0.0,0.0)

         ! Calculate maximum size of C and D
       SP_MXLLD_CD = floor (dble(nbasis)/(dble(nprow)))+1
       CALL DESCINIT( SP_DESC_C, nbasis, 1, SP_MB, SP_NB, RSRC, CSRC, ICTXT, SP_LLD_OLAP,INFO )
       if(info .ne. 0) then
          write(*,*) "descC proc",myrank," info ",info
          write(*,*) SP_DESC_C(-info), "was ",SP_Col_OLAP
       end if
       CALL DESCINIT( SP_DESC_D, nbasis, 1, SP_MB, SP_NB, RSRC, CSRC, ICTXT, SP_LLD_OLAP,INFO )
       if(info .ne. 0) then
          write(*,*) "descD proc",myrank," info ",info
          write(*,*) SP_DESC_D(-info), "was ",SP_Col_OLAP
       end if
       allocate(localC(SP_Row_OLAP,1))
       allocate(localD(SP_Row_OLAP,1))

       NRHS = 1 
       allocate(FERR(NRHS),BERR(NRHS))

       allocate(IPIV(SP_Row_OLAP+SP_MB))

       ! Want to find out how big to make the arrays
       allocate(WORK(1))
       allocate(RWORK(1))
       LWORK = -1
       LRWORK = -1
       EQUED = 'N'
       call pzgesvx('N','N',nbasis, NRHS,overlpm ,1,1,SP_DESC_OLAP, &
            overlpmf, 1,1,SP_DESC_OLAP, &
            IPIV,EQUED,SCA_R,SCA_C,localC, 1,1,SP_DESC_C, localD, 1,1,SP_DESC_D,&
            RCOND, FERR, BERR, WORK, LWORK, &
            RWORK, LRWORK,INFO )
       LWORK = REAL(WORK(1))
       LRWORK = RWORK(1)
       deallocate(WORK,RWORK)
       allocate(WORK(LWORK))
       allocate(RWORK(LRWORK))
       

         ! Calculate number of complete blocks
       SP_nbc = floor(dble(SP_Col_OLAP)/dble(SP_MB))
       SP_nbr = floor(dble(SP_Row_OLAP)/dble(SP_NB))
       ! Now calculate extra bits
       SP_ceb = mod(SP_Col_OLAP,SP_MB)
       SP_reb = mod(SP_Row_OLAP,SP_NB)

#endif
     end subroutine allocatescalapackarrays


    subroutine deallocatearrays
      implicit none

      if (allocated(zinit)) deallocate(zinit)
!      if(allocated(zb)) deallocate(zb)
      if(allocated(overlpm)) deallocate(overlpm)
!      if(allocated(S)) deallocate(S)
!      if(allocated(C)) deallocate(C)
      if(allocated(D)) deallocate(D)

      ! Working arrays for step
!!$      if(allocated(zbnew)) deallocate(zbnew)
!!$      if(allocated(zk1)) deallocate(zk1) 
!!$      if(allocated(zk2)) deallocate(zk2)
!!$      if(allocated(zk3)) deallocate(zk3)
!!$      if(allocated(zk4)) deallocate(zk4)
!!$
!!$      if(allocated(Snew)) deallocate(Snew)
!!$      if(allocated(Sk1))  deallocate(Sk1)
!!$      if(allocated(Sk2))  deallocate(Sk2)
!!$      if(allocated(Sk3))  deallocate(Sk3)
!!$      if(allocated(Sk4))  deallocate(Sk4)
!!$
!!$      if(allocated(Cnew))  deallocate(Cnew)
!!$      if(allocated(Ck1))  deallocate(Ck1)
!!$      if(allocated(Ck2))  deallocate(Ck2)
!!$      if(allocated(Ck3))  deallocate(Ck3)
!!$      if(allocated(Ck4))  deallocate(Ck4)

      if(allocated(Dold))  deallocate(Dold)

      if (allocated(grid)) deallocate(grid)
      if (allocated(ngrid)) deallocate(ngrid)
      
      if(allocated(gsigma)) deallocate(gsigma)


       if(allocated(FERR)) deallocate(FERR)
       if(allocated(BERR)) deallocate(BERR)
       if(allocated(RWORK)) deallocate(RWORK)
       if(allocated(WORK)) deallocate(WORK)
       if (allocated(ipiv)) deallocate(ipiv)
       if (allocated(overlpmf)) deallocate(overlpmf)

    end subroutine deallocatearrays
  
end module arrays

module scalapackarrays
  implicit none
  
end module scalapackarrays
