module hamiltoniandata

  ! mass
  real(kind=8),dimension(:),allocatable ::m

  ! Tunnel + Harmonic Bath Hamiltonial parameters
  real(kind=8) :: eta
  ! omega and omega^2
  real(kind=8),dimension(:),allocatable :: omega
  real(kind=8),dimension(:),allocatable :: omega2
  ! Coupling between degrees of freedom
  real(kind=8),dimension(:),allocatable :: coup

  ! Henon-Heiles  Hamiltonial parameters
  real(kind=8) :: lambda

  ! ---- polynomial and Gaussian potentials ----
  integer,dimension(:),allocatable :: polydegree
  logical,dimension(:),allocatable :: morse
  real(kind=8),dimension(:,:),allocatable :: pcoeff
  real(kind=8),dimension(:),allocatable :: morseA
  real(kind=8),dimension(:),allocatable :: morseq0
  real(kind=8),dimension(:),allocatable :: morsealpha
  real(kind=8),dimension(:),allocatable :: morseC
  logical,dimension(:),allocatable :: gaussian
  real(kind=8),dimension(:),allocatable :: gaussA
  real(kind=8),dimension(:),allocatable :: gaussalpha
  real(kind=8),dimension(:),allocatable :: gaussq0
  
  ! coupling
  real(kind=8),dimension(:),allocatable :: harmcouplea12,harmcouplea21,harmcouplea22

  ! smoothly truncate couplings
  integer,dimension(:),allocatable :: trunc   !0 - no truncation, -1 for x<0 and 1 x> 0. Other options now available.
  integer,dimension(:),allocatable :: truncm  ! which mode to truncate on
  real(kind=8),dimension(:),allocatable :: truncpos,truncpos2
  real(kind=8),dimension(:),allocatable :: truncwidth

  ! switch couplings on and off through another mode.
  integer,dimension(:),allocatable :: coupswitch
  integer,dimension(:),allocatable :: coupswitchmode
  integer,dimension(:),allocatable :: coupswitchdir
  real(kind=8),dimension(:),allocatable ::coupswitchpos
  real(kind=8),dimension(:),allocatable ::coupswitchgamma


  ! ---- Cartesian hamiltonian ----
  integer :: cart_nints ! number of interactions
  integer :: cart_maxsgterms
  ! number of terms for each interaction
  integer,dimension(:),allocatable :: cart_ntint
  ! amplitudes for terms in gaussian sum  i,j - j=interaction; i = sum term
  real(kind=8),dimension(:,:),allocatable :: cart_gsamp 
  ! sigmas for terms in gaussian sum
  real(kind=8),dimension(:,:),allocatable :: cart_gstwosigmasq,cart_gssigma
  ! which particles are interacting particle 1 and particle 2
  integer,dimension(:),allocatable :: cart_part1,cart_part2
  
  ! ---- Which Hamiltonian ----
  integer :: hamtype    ! Numerical type
  ! define types
  integer,parameter :: polymham = 3
  integer,parameter :: classpolymham = 5
  integer,parameter :: cartham = 4
  integer,parameter :: clascartham = 6

  character(len=50) :: hamname ! name as given in hamiltonian input file

  ! ---- Time spent in functions ----
  real(kind=8) :: totengtime
  real(kind=8) :: DsolveTime
  real(kind=8) :: StepTime
  real(kind=8) :: dC_dtTime

  ! ---- Loop parallelisation -----
  integer :: ibeg_nbasis,iend_nbasis

contains
  ! ==================================================
  !          Allocate common arrays
  ! ================================================== 
  subroutine allocatehamarrays
    use coherentstates, ONLY: ndf

    ! Masses
    allocate(m(ndf))
    m(:) = 1.0

  end subroutine allocatehamarrays

  ! ==================================================
  !           Deallocate all arrays
  ! ==================================================
  subroutine deallocatehamarrays
    implicit none
    if(allocated(omega)) deallocate(omega)
    if(allocated(omega2)) deallocate(omega2)

    if(allocated(coup)) deallocate(coup)

    if(allocated(m)) deallocate(m)

    if(allocated(trunc)) deallocate(trunc)
    if(allocated(truncpos)) deallocate(truncpos)
    if(allocated(truncwidth)) deallocate(truncwidth)

  end subroutine deallocatehamarrays

end module hamiltoniandata
