 ! ==================================================
  !           Henon-Heiles Hamiltomian 
  ! ==================================================
  function hamhenonheiles(zic,zj)
    use coherentstates, ONLY: ndf,gamma,s2gammarec
    ! include subroutines too.
    use coherentstates, ONLY: pos,mom
    implicit none

    ! two states <z_i| and |z_j>
    complex(kind=8),dimension(ndf),intent(in) :: zic,zj
    ! hamiltonian
    complex(kind=8):: hamhenonheiles

    ! position and momentum vectors
    complex(kind=8),dimension(ndf) :: q,p

    ! Calculate position and momentum vectors
    call pos(zj,zic,q)
    call mom(zj,zic,p)


    hamhenonheiles = sum(p(:)**2 + q(:)**2+gamma(:)/2.0 +0.5/gamma(:))/2.0 &
         + lambda * sum( &
         (q(1:ndf-1)**2+0.5/gamma(1:ndf-1))*q(2:ndf) &
         - q(2:ndf)**3/3.0-2*s2gammarec(2:ndf)**3 )   
  end function hamhenonheiles


  function dhamhenonheilesdz(zic,zj)
    use coherentstates, ONLY: ndf,gamma,s2gammarec,shgamma
    ! include subroutines too.
    use coherentstates, ONLY: pos,mom
    implicit none

    ! two states <z_i| and |z_j>
    complex(kind=8),dimension(ndf),intent(in) :: zic,zj
    ! hamiltonian
    complex(kind=8):: dhamhenonheilesdz(ndf)

    ! position and momentum vectors
    complex(kind=8),dimension(ndf) :: q,p

    ! Calculate position and momentum vectors
    call pos(zj,zic,q)
    call mom(zj,zic,p)


    dhamhenonheilesdz(:)  = (0,-1)*p(:)*shgamma(:) + q(:)*s2gammarec(:)

    dhamhenonheilesdz(1:ndf-1) =  dhamhenonheilesdz(1:ndf-1) &
         + 2*lambda*q(1:ndf-1)*q(2:ndf)*s2gammarec(1:ndf-1)

    dhamhenonheilesdz(2:ndf) =  dhamhenonheilesdz(2:ndf) &
         + lambda*(q(1:ndf-1)**2+0.5/gamma(1:ndf-1)-q(2:ndf)**2)*s2gammarec(2:ndf)

  end function dhamhenonheilesdz


  ! ==================================================
  !            Read Hamiltomian parameters
  !     specific to the Henon-Heiles system
  !  allocates arrays too.
  ! ==================================================
  subroutine readhamhh
    use files, ONLY: fug,fham
    use errors

    character(len=50) :: line
    integer :: ierr

    ! Read in eta 
    read(fham,*,iostat=ierr) line
    if  (line .ne. "lambda" .or. ierr .ne. 0  ) then
       write(fug,*) "Failed to read keyword lambda in hamiltonian.inpt"
       stop
    end if
    backspace(fham)
    read(fham,*,iostat=ierr) line, lambda
    if(ierr.ne.0) call filereaderror(ierr,"hamiltonian.inpt","Error reading lambda")      
    ! Summaraise
    write(fug,*) "Hamiltonian parameters"
    write(fug,*) "----------------------"
    write(fug,*) "  Studying Henon-Heiles system"
    write(fug,*) "  lambda =",lambda
    write(fug,*)

  end subroutine readhamhh
