! **************************************************
!               Coherent States 
!
!                  Stewart K. Reed
!                   4th June 2007
!
!  Fully working version as of 13th July 2007
!
! Description:
!
! **************************************************

  program ccs
    use coherentstatestype
    use coherentstates
    use hamiltonian, ONLY: readhamparams,deallocatehamarrays,totalenergy,h_ord,&
                           totengtime,DsolveTime,StepTime,dC_dtTime
    use arrays, ONLY: zinit,bs,D,allocatearrays,allocatelapackarrays,allocatescalapackarrays,deallocatearrays,toteng,vstate,&
                      initnorm,psteps,t,pt,incbf
    use options
    use files, ONLY:fug,facf,fccf,fhcf
    ! functions
    use hamiltonian, ONLY: step,determineloopbounds
    use makebasis

    use scalapackstuff, ONLY: isroot, nprocs, comm, MPI_INTEGER8, initialsetupscalapack,setupscalapack,&
                              stopscalapack,myrank,nprow,npcol
    implicit none

    integer :: ts,i ! current step number
    real(kind=8) :: n

    real(kind=8),allocatable,dimension(:,:) :: basisenergies

    !Times
    real(kind=8) :: starttime,endtime,startlooptime,endlooptime
    real(kind=8), external :: MPI_Wtime

    !For when started
    integer*4 :: today(3), now(3)
    integer,dimension(8) :: datevalues
    character(len=100) ::fmt

    integer :: mpierr,info

    call initialsetupscalapack

    ! Open output file if required
    if (fug .ne. 6 .and. isroot) then
       open(fug,file = "output")
    end if
    
    !call CPU_TIME(starttime)
    starttime = MPI_Wtime()

    DsolveTime = 0.0d0 ! time taken solving for D
    totengtime = 0.0
    StepTime = 0.0d0   ! time taken doing the time steps.
    dC_dtTime = 0.0d0  ! time calculating dC_dtTime
    overlapTime = 0.0d0
    
    if(isroot) then
       write(fug,*) "         ccs with Ehrenfest dynamics"
       write(fug,*) "                Stewart Reed"
!       write(fug,*) "                7th November 2008"
!       write(fug,*) "             15th December 2008"
!       write(fug,*) "              12th March 2009"
!       write(fug,*) "               21st April 2009"
!       write(fug,*) "               17th June 2009"
!       write(fug,*) "               24th August 2009"
       include "built.f90"
       write(fug,*) 

       
!       call idate(today)   ! today(1)=day, (2)=month, (3)=year
!       call itime(now)     ! now(1)=hour, (2)=minute, (3)=second
       call date_and_time(values=datevalues)
       fmt = "('    Run started at ' i2.2, ':', i2.2, ':', i2.2 ' on the '  i2.2, '/', i2.2, '/', i4.4 )"
       ! The portland group compiler gives a number in the range [0:99]
       if (today(3) < 1000) then
          today(3) = 2000+today(3)
       end if
!       write (fug, fmt )  now, today(2), today(1), today(3)
       write (fug, fmt )  datevalues(5:7),datevalues(3),datevalues(2),datevalues(1)
       write(fug,*)
       if (nprocs .eq. 1 ) then
          write(fug,*) "   Running on",nprocs,"processor"
       else
          write(fug,*) "   Running on",nprocs,"processors"
       end if
    end if

    ! Read options from run.inpt file.
    call readoptions

    ! Set up processor grid
    call setupscalapack(info)

    if (isroot) then
       if (nprocs .gt. 1 ) then
          write(fug,*) "   arranged in a ",nprow,"x",npcol,"grid"
       end if
    end if

    if (isroot) then
       call writeoptions
    end if

    ! Seed the random number generator with the current date/time?
    if (randseed .and. .not. restartsim) then
!       call ZBQLINI(randgenseed,fug)
       if(isroot) then
          call ZBQLINI(randgenseed,fug)
       else
          call ZBQLINI(randgenseed,0)
       end if
!       call random_seed()
       ! Make sure all processors have the same random number
!       call MPI_BCAST(randgenseed,1,MPI_INTEGER8,0,comm)

       ! if we paralellise create basis we may want each processor to 
       ! have a different random number in which case we could add say
       ! 10*rank  to randgenseed.
   end if

    ! Create arrays 
    call allocatearrays(ndf,nbasis,nehernfest)
    if(solvemethod==1) then
       call allocatelapackarrays(ndf,nbasis)
    else if (solvemethod == 3) then
       call allocatescalapackarrays(ndf,nbasis)
    end if

    ! set loop bounds depending on numbers of processors.
    call determineloopbounds

    ! Read in hamiltonian parameters
    call readhamparams
    
    ! Need to set up initial state
    call readinitstates

    if(.not. restartsim) then
       call createbasis(basismethod)
    end if
    call openoutputfiles

!    toteng = totalenergy(zb,D,S,incbf)
!    call output(0)
    
    if (restartsim) then
       if (isroot) write(fug,*) "Restarting a simulation"
       call restart
    end if

    allocate(basisenergies(nbasis,2))
    do i = 1,nbasis
       basisenergies(i,1) = h_ord (dconjg(bs%zb(:,i)),bs%zb(:,i))*eu
    end do

    if(.not.restartsim) then
       initnorm = norm(bs%C,D)
       ! Check to make sure the initial norm is positive
       if (initnorm < 0) then
          write(fug,*) "Basis set problem: the initial norm is negative"
          stop
       end if
       bs%C(:) = bs%C(:) /dsqrt(initnorm)
       D(:) = D(:) / dsqrt(initnorm)
    end if
    if (isroot) write(fug,*) "Initial norm ",initnorm

     n = norm(bs%C,D)
     if (isroot) write(fug,*) "New norm ",n

    toteng = totalenergy(bs,D,incbf)
    call output(0)

    if (writeacffreq .gt. 0 ) then  
       call calccorfn(0.0d0,bs,zinit(:,1),D,facf)
    end if

    if (ccorlog > 0 ) then
       call calccorfn(0.0d0,bs,zinit(:,2),D,fccf)
    end if
    
    if (hcorfreq > 0 ) then
       call harmcorfn(0.0d0,bs,vstate,D,fhcf)
    end if

    call CPU_TIME(startlooptime)
    ! Main Loop
    do ts = 1,nsteps
       t = dt*ts*tu+pt
       if (ts .eq. 1 .or. (mod(float(ts),float(printfreq)).eq.0) .or. ts.eq. nsteps) then
          if (isroot) write(fug,*) "Step ",ts+psteps, "  time ",t
       end if
!       if (isroot) write(fug,*) "Energy",totalenergy(zb,D,S,incbf)
       call step
!n = norm(zb,C,D)
!  C(:) = C(:) /dsqrt(n)
!       D(:) = D(:) / dsqrt(n)
      
       
       if (ts .eq. 1 .or. (mod(float(ts),float(printfreq)).eq.0) .or. ts.eq. nsteps &
            .or. mod(float(ts),float(writeEfreq)).eq.0 ) then
          toteng = totalenergy(bs,D,incbf)
          if (ts .eq. 1 .or. (mod(float(ts),float(printfreq)).eq.0) .or. ts.eq. nsteps) then
             if (isroot)  write(fug,*) "Energy",toteng
          end if
       end if
       
       ! Get the root processor to output stuff.
       call output(ts)
       
       if (writeacffreq .gt. 0 .and. mod(float(ts),float(writeacffreq)).eq.0) then
          call calccorfn(dt*ts*tu,bs,zinit(:,1),D,facf)
       end if
       if (ccorlog > 0 .and. mod(float(ts),float(ccorlog)).eq.0) then
          if(mod(float(ts),float(ccorlog)).eq.0) then  
             call calccorfn(dt*ts*tu,bs,zinit(:,2),D,fccf)
          end if
       end if
       
       if (hcorfreq > 0 .and. mod(float(ts),float(hcorfreq)).eq.0) then
          if(mod(float(ts),float(hcorfreq)).eq.0) then  
              call harmcorfn(dt*ts*tu,bs,vstate,D,fhcf)
          end if
       end if
             
!       write(fug,*) "Norm ",n
!       C(:) = C(:) /dsqrt(n)
!       D(:) = D(:) / dsqrt(n)
       
    end do
    call CPU_TIME(endlooptime)
!    do i = 1,nbasis
!       basisenergies(i,2) = h_ord (dconjg(zb(:,i)),zb(:,i))*eu
!      write (57,*) basisenergies(i,1),basisenergies(i,2)
!   end do
!    write (57,*) sum(basisenergies(:,1)),sum(basisenergies(:,2))

    if (isroot) call  dump(nsteps,"systate.out")
    ! Clean up - deallocate arrays
    call deallocatecsarrays
    call deallocatehamarrays
    call deallocatearrays

    ! Stop scalapack
    call stopscalapack

    ! Get final time
!    call CPU_TIME(endtime)
    endtime = MPI_Wtime()

    ! Root processor should write out timings
    if(isroot) then
       write(fug,*) "Timings"
       write(fug,*) "Total time",endtime-starttime
       write(fug,*) "Main loop",endlooptime-startlooptime
       write(fug,*) "Energy calculation",totengtime
       write(fug,*) "Time solving for D",DsolveTime
       write(fug,*) "Time stepping",StepTime,"includes solving time"
       write(fug,*) " of this ",dC_dtTime," was spent calculating dC_dt"
       write(fug,*) "Time calculating overlap",overlapTime
       write(fug,*) "Bye Bye"
    end if

  end program ccs
