! **************************************************
!                      output
!
!                  Stewart K. Reed
!
! Description:
!
! This file contains routines to open files for
! output and also to write data to those files. 
! **************************************************

! ===============================================
! openoutputfiles
! ===============================================
subroutine openoutputfiles
  use files, ONLY:fbas,fact,facf,feng,fceng,fpos,famp,fccf,fnorm,fC,fD,fdCdT,&
       fdSdT,fdDdT,fovlp,fhcf,fzpos,fexpect,fnormbasis
  use options, ONLY: ccorlog,writeCDfreq,writeampfreq,hcorfreq,writeposfreq,&
       writeSfreq,writeEfreq,writenormfreq,writebasisnormfreq,writeacffreq,&
       writezposfreq,writeexpect,energyunits,writeovlpfreq
  use scalapackstuff, ONLY: isroot
  implicit none

  if (isroot) then
!     open(fbas,file="basisfns.out")
     
     ! open the following files iff the user has requested a +ve 
     ! interval for writing out the data.
     
     if (writeacffreq .gt. 0 ) then
        open(facf,file="acf.out")
        write(facf,*) "# Auto correlation function"
        write(facf,*) "# t       Re       Img      abs      norm"
     end if
     
     if (writeEfreq .gt. 0 ) then
        open(feng,file="energy.out");
        write(feng,*) "# sum(hord(basis fcts)). Units ",trim(energyunits)
        
        open(fceng,file="consenergy.out");
        write(fceng,*) "# Conserved energy. Units ",trim(energyunits)
     
     end if
     
     if (writeposfreq > 0 ) then
        open(fpos,file="pos.out");
        write(fpos,*) "# q1  p1    q2  p2 ...."
     end if
     
     if (writezposfreq > 0 ) then
        open(fzpos,file="zpos.out");
        write(fzpos,*) "# Re(z1)  Im(z1)    Re(z2)  Im(z2) ...."
     end if
     
     if (writeampfreq .gt. 0) then
        open(famp,file="amplitudes.out");
        write(famp,*) "# wave packet amplitudes"
        write(famp,*) "# Version 2.0"
     end	if
     
     ! wavefunction amplitudes
     if(writeCDfreq .gt. 0) then
        open(fC,file="C.out")
        open(fD,file="D.out")
        open(fdCdT,file="fdCdT.out")
        open(fdDdT,file="fdDdT.out")
     end if
     
     ! Action
     if(writeSfreq .gt. 0) then
        open(fact,file="action.out")
        write(fact,*) "# Classical action"
        
        open(fdSdT,file="fdSdT.out")
        write(fdSdT,*) "# Classical action derivative dSdT"
     end if
     
     ! Cross correlation function
     if ( ccorlog  > 0 ) then
        open(fccf,file="ccf.out")
        write(fccf,*) "# Cross correlation function"
        write(fccf,*) "# t (fs)     Re(ccf)     Img(ccf)   abs(ccf)    norm"
     end if
     
     ! Cross correlation function
     if ( hcorfreq  > 0 ) then
        open(fhcf,file="hcf.out")
        write(fhcf,*) "# Cross correlation function with harmonic oscillator"
        write(fhcf,*) "# t (fs)     Re(ccf)     Img(ccf)   abs(ccf)    norm"
     end if
     
     ! Energy expectation value
     if (writeexpect > 0 ) then
        open(fexpect,file="hexpect.out")
        write(fexpect,*) "# Energy expectation value. Units ",trim(energyunits)
        write(Fexpect,*) "# <H> = sum_i Ci* Di H(zi*,zi)" 
     end if
     
     ! Wavefunction norm
     if(writenormfreq > 0) then
        open(fnorm,file="norm.out")
        write(fnorm,*) "# wavefunction normal"
        write(fnorm,*) "#t     norm"
     end if
     ! contribution of each basis function
     if (writebasisnormfreq > 0 ) then
        open(fnormbasis,file="basisnorm.out")
        write(fnormbasis,*) "# wavefunction normal"
        write(fnormbasis,*) "#t     norm"
     end if
     
     if (writeovlpfreq > 0 ) then
        open(fovlp,file="overlapmatrix.out")
        write(fovlp,*) "#Overlap matrix"
     end if
  end if ! isroot

end subroutine openoutputfiles



! ===============================================
! subroutine to write data to files
! ===============================================
subroutine output(stepnum)
  use coherentstatestype
  use files, ONLY: fbas,fact,feng,fceng,fpos,famp,fnorm,fC,fD,fovlp,&
       fzpos,fexpect,fnormbasis
  use coherentstates, ONLY: ndf,nbasis,overlap_bs
  use arrays, ONLY: bs,D,toteng,dtoteng,t,psteps,incbf,dhexpect
  use options, ONLY: writeampfreq, writeCDfreq,writeposfreq,writeSfreq,&
       writeEfreq,writenormfreq,writebasisnormfreq, writeacffreq,dumpfreq,&
       writezposfreq,writeexpect,writeovlpfreq
  use scalapackstuff, ONLY: isroot

  ! functions
  use coherentstates, ONLY:pos,mom,norm
  use options, ONLY:dt,tu
  use hamiltonian, ONLY: conservedenergy,hamtype, polymodeenergy,&
       expectationvalue,modeexpectationvalue
  implicit none
  integer,intent(in) :: stepnum
  real(kind=8) :: vals(3*ndf+2) ! vals(2*ndf+2)
  integer :: n,i,j
  character*20 :: fmt  ! for dynamic format string   
  complex(kind=8) :: amp,ampC
  complex(kind=8),dimension(ndf) :: q,p
  !  real(kind=8) :: t ! Current time - in arrays.

  real(kind=8),allocatable,dimension(:),save :: modeenergies
  real(kind=8),allocatable,dimension(:),save :: modepureenergies
  real(kind=8),allocatable,dimension(:),save :: modecoupleenergies

  real(kind=8) ::  hexpect
  complex(kind=8)::ovlp

  if(isroot) then
     ! ---- Calculate current time ----
     ! Calculated in main
     !  t = dt*stepnum*tu

     ! ---- Write out basis functions -----
     !  write(fbas,*) "#  Step", stepnum
     !  n = 2*ndf+2
     !  do i = 1,nbasis
     !     do j = 1,ndf
     !        vals(2*j-1) = real(zb(j,i))
     !        vals(2*j) = aimag(zb(j,i))
     !     end do
     !     amp = D(i)*cdexp((0,1.0d0)*S(i))
     !     vals(2*ndf+1) = real(amp) !real(bs%C(i))
     !     vals(2*ndf+2) = aimag(amp) !aimag(bs%C(i))
     !     write(fmt,'("(",i4,"es18.8)")') n
     !     write(fbas,fmt) (vals(j),j=1,n)
     !  end do
     !  write(fbas,*) 


     ! ---- Write out positions/momenta -----

     if(writeposfreq > 0) then
        if (mod(float(stepnum),float(writeposfreq)).eq.0) then  

           write(fpos,*) "#  Step", stepnum
           n = 2*ndf+2
           do i = 1,nbasis
              call pos(bs%zb(:,i),dconjg(bs%zb(:,i)),q)
              call mom(bs%zb(:,i),dconjg(bs%zb(:,i)), p)

              do j = 1,ndf
                 vals(2*j-1) = real(q(j))
                 vals(2*j) = real(p(j))
              end do
           end do
           write(fpos,"(100f18.8)") t,(vals(j),j=1,2*ndf)
           write(fpos,*) 
        end if
     end if


     ! ---- Write out z-positions ----
     if(writezposfreq > 0) then
        if (mod(float(stepnum),float(writezposfreq)).eq.0) then  
           write(fzpos,*) "#  Step", stepnum
           n = 2*ndf+2
           do i = 1,nbasis
              do j = 1,ndf
                 vals(2*j-1) = real(bs%zb(j,i))
                 vals(2*j) = dimag(bs%zb(j,i))
              end do
              write(fzpos,"(100f18.8)") t,(vals(j),j=1,2*ndf)
           end do
           write(fzpos,*)
        end if
     end if


     ! ---- write out the action ----
     if(writeSfreq > 0) then
        if (mod(float(stepnum),float(writeSfreq)).eq.0) then  
           write(fact,*)  "#  Step", stepnum+psteps
           do i = 1,nbasis
              write(fact,*) t, bs%S(1,i)
           end do
           write(fact,*)
        end if
     end if
  end if
  ! ---- total system energy ----
  if(writeEfreq > 0) then
     if (mod(float(stepnum),float(writeEfreq)).eq.0) then 
        if(hamtype .eq. 3 .or. hamtype .eq. 5) then
           ! allocate an array to store energies in
           if(.not.allocated(modeenergies)) then
              allocate(modeenergies(ndf))
              allocate(modepureenergies(ndf))
              allocate(modecoupleenergies(ndf))
           end if

           do i = 1,ndf
              modeenergies(i) = &
                   polymodeenergy(bs,D,i,modepureenergies(i),modecoupleenergies(i),incbf)
           end do
           if(isroot) write(feng,"(50f18.10)")  t,toteng,dtoteng,(modepureenergies(i),i=1,ndf),&
                (modecoupleenergies(i),i=1,ndf)
        else
           !     write(feng,*) stepnum, toteng
           if(isroot) write(feng,*) t, toteng,dtoteng
        end if

        ! ---- Conserved  energy ----
        if(isroot) write(fceng,*) t, conservedenergy(bs%zb)

     end if
  end if

  ! ---- expectation value (total system energy) ----
  if(writeexpect > 0) then
     if (mod(float(stepnum),float(writeexpect)).eq.0) then 

        ! Calculate expectation value
        Hexpect = expectationvalue(bs%zb,bs%C,D,incbf)

        if(hamtype .eq. 3 .or. hamtype .eq. 5) then
           ! allocate an array to store energies in
           if(.not.allocated(modeenergies)) then
              allocate(modeenergies(ndf))
              allocate(modepureenergies(ndf))
              allocate(modecoupleenergies(ndf))
           end if

           ! do i = 1,ndf
           !    call modeexpectationvalue(bs%zb,C,D,incbf,i,modepureenergies(i),modecoupleenergies(i))
           ! end do

           do j = 1,ndf
              call modeexpectationvalue(bs%zb,bs%C,D,incbf,j,vals(2*j-1),vals(j+2*ndf))
              vals(2*j) = dhexpect(1+j)
           end do
           if (isroot) write(fexpect,"(50es18.10)")  t,Hexpect,dhexpect(1),(vals(i),i=1,3*ndf)

           !           (modepureenergies(i),i=1,ndf),(modecoupleenergies(i),i=1,ndf)


        else
           !     write(feng,*) stepnum, toteng
           if (isroot) then
              if (allocated(dhexpect)) then ! should really make sure it is allocated for all hamiltonians.
                 write(fexpect,*) t, Hexpect,dhexpect(1)
              else
                 write(fexpect,*) t, Hexpect,dhexpect(1)
              end if
           end if
        end if

     end if
  end if

  if (isroot) then
     ! ---- write out the amplitudes ----
     if (writeampfreq > 0) then
        write(fmt,"(i3,'es18.8')")
        if (mod(float(stepnum),float(writeampfreq)).eq.0) then  
           write(famp,*) "#  Step  Re(D) Im(D) Re(C) Im(C) q1,p1,.....qn,pn"
           do i = 1,nbasis
              call pos(bs%zb(:,i),dconjg(bs%zb(:,i)),q)
              call mom(bs%zb(:,i),dconjg(bs%zb(:,i)), p)
              amp = D(i)*cdexp((0,1.0d0)*bs%S(1,i))
              ampC = bs%C(i)*cdexp((0,1.0d0)*bs%S(1,i))
              !           write(famp,"(6f18.10)") t, real(q(j)), real(p(j)), &
              !                real(amp), dimag(amp), abs(amp) 
              do j = 1,ndf
                 vals(2*j-1) = real(q(j))
                 vals(2*j) = real(p(j))
              end do
              write(famp,"(100es18.8)") t, &
                   real(amp), dimag(amp), real(ampC),dimag(ampC),&
                   (vals(j),j=1,2*ndf)
           end do
           write(famp,*) 
        end if
     end if

     ! ---- Coefficients D and C ----
     if (writeCDfreq > 0) then
        if (mod(float(stepnum),float(writeCDfreq)).eq.0) then
           write(fC,*) "#  Step", stepnum
           write(fD,*) "#  Step", stepnum

           do i = 1,nbasis
              write(fC,"(4es18.10)") t,real(bs%C(i)),dimag(bs%C(i)),abs(bs%C(i))
              write(fD,"(4es18.10)") t,real(D(i)),dimag(D(i)),abs(D(i))
           end do
           write(fC,*) 
           write(fD,*)     
        end if
     end if

     ! ---- wavefunction norm ----
     if (writenormfreq > 0) then
        if (mod(float(stepnum),float(writenormfreq)).eq.0) then  
           write(fnorm,*) t,norm(bs%C,D)
           !        do i = 1,nbasis
           !           write(66,*)   real(dconjg(bs%C(i))*D(i)), dimag(dconjg(bs%C(i))*D(i))
           !        end do
           !(dconjg(bs%C(i))*D(i),i=1,nbasis)
        end if
     end if

     ! ---- ind basis fn contribution to wavefunction norm ----
     if (writebasisnormfreq > 0) then
        if (mod(float(stepnum),float(writenormfreq)).eq.0) then  
           write(fnormbasis,*) "#  Step ",stepnum," time ",t
           do i = 1,nbasis
              write(fnormbasis,*)  t,real(dconjg(bs%C(i))*D(i)), dimag(dconjg(bs%C(i))*D(i))
           end do
        end if
        write(fnormbasis,*)
     end if

     ! ---- overlap matrix ----
     if (writeovlpfreq > 0) then
        write(fovlp,*) "#  Step ",stepnum," time ",t
        do i = 1,nbasis
           do j = 1,nbasis
              ovlp = overlap_bs(bs,i,j)
              if  (abs(ovlp) > 1e-10) then
                 write(fovlp,"(f10.5,3es18.8)") t,real(ovlp),dimag(ovlp), &
                      abs(ovlp)
              else
                 write(fovlp,*) t,0.0,0.0,0.0
              end if
           end do
        end do
     end if

     if (dumpfreq > 0 ) then
        if (mod(float(stepnum),float(dumpfreq)).eq.0) then
           call  dump(stepnum,"systate.out")
        end if
     end if
  end if
end subroutine output
