! **************************************************
!                     Parallel
!
!                  Stewart K. Reed
!                 7th November 2008
!
! Description:
!   This file contains a module called scalpackstuff.
!  It contains the variables associated with 
!  parallel execution. It also contains routines 
!  for setting up scalalapack and MPI and for 
!  clearing up at the end.
! **************************************************
module scalapackstuff
  implicit none

! include MPI definitions too
  include "mpif.h"

  ! ---- Parameters for scalapck ----
  ! MPI communicator
  integer,parameter :: comm=MPI_COMM_WORLD
  ! scalapack communication context equivalent to MPI's communicator
  integer :: ICTXT

  integer :: NPROW, NPCOL ! Number of rows and columns for processor grid.
  integer :: MYROW, MYCOL ! where this processor is in the grid.
  
  integer :: MYRANK ! which processor am I.
  integer :: NPROCS ! number of processors being used

  logical :: isroot
contains

  ! **************************************************
  !    Subroutine to set up scalapack
  ! **************************************************
  subroutine initialsetupscalapack
    implicit none

    ! ---- Get the number of processors from BLACS. ----
    CALL BLACS_PINFO( myrank, NPROCS )

    if(myrank .eq. 0) then
       isroot =.true.
    else
       isroot = .false.
    end if
  end subroutine initialsetupscalapack


!  subroutine setupscalapack(unprow,unpcol,info)
  subroutine setupscalapack(info)
    implicit none

!    INTEGER,intent(inout),optional :: unprow,unpcol
    integer,intent(out) :: info
    INTEGER ::i,unprocs
    
    info = 0

    ! Processor grid may already have been set somewhere. 
    ! Therefore check that the number of processors in the
    ! grid is the same as the actual number of processors.
    ! If it is, use the suggested grid. If not, determine 
    ! new grid.
    unprocs = nprow* npcol

    if (unprocs .eq. nprocs) then
       info = 1
    end if

    if (info .ne. 1 ) then
       ! ------ determine processor grid size ------
       ! Make it as close to square as possible
       ! should maybe include an option to read it in from file
       NPCOL = 1 !nprocs
       NPROW = nprocs !1
       i = 0
       do while  ( i .le.floor( sqrt(dble(nprocs))) )
          i = i+1
          if (mod(nprocs,i) .eq. 0 ) then
             NPCOL = i !floor(dble(nprocs)/dble(i))
             NPROW = floor(dble(nprocs)/dble(i)) ! i
          end if
       end do

    end if

    ! ---- initialise grid ----
    CALL SL_INIT( ICTXT, NPROW, NPCOL )

    ! ---- Find out where we are on the grid. ----
    CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )

  end subroutine setupscalapack

  ! **************************************************
  !    Subroutine to stop scalapack
  ! **************************************************
  subroutine stopscalapack
    implicit none

    !
    !     RELEASE THE PROCESS GRID
    !     Free the BLACS context
    if (MYROW .ge. 0) then           
       CALL BLACS_GRIDEXIT( ICTXT )
    end if
    !
    !     Exit the BLACS
    !
    CALL BLACS_EXIT( 0 )
    
  end subroutine stopscalapack

end module scalapackstuff
