
module classicpolymorsehammod
  use hamiltoniandata
  private

  ! public interface
  public :: champolymorse, champolymorse1mode, dchampolymorsedz
  public :: readchampm
contains
! ==================================================
  !          Polynomial + Morse Hamiltomian 
  ! ==================================================
  
function champolymorse(zic,zj) 
  use coherentstates, ONLY: ndf,gamma,sgamma
  ! include subroutines too.
  use coherentstates, ONLY: pos,mom
  use arrays, ONLY: sqrtpi
  implicit none

  ! two states <z_i| and |z_j>
  complex(kind=8),dimension(ndf),intent(in) :: zic,zj
  ! hamiltonian
  complex(kind=8)::champolymorse
  complex(kind=8)::Ucoup ! coupling energy
  complex(kind=8)   ::  coups! ! switch coupling with another mode 
  ! position and momentum vectors
  complex(kind=8),dimension(ndf) :: q,p
  integer :: i,j,index

  !    real(kind=8),external :: derf ! derf isn't actually an intrinsic function 
  !                                  ! but this is needed for the PG compiler.

  interface 
     include "MCERROR.interface"
  end interface

  ! Calculate position and momentum vectors
  call pos(zj,zic,q)
  call mom(zj,zic,p)

  ! Now calculate the classical Hamiltonian
  ! p_i^2/2m_i
  champolymorse = 0.5*sum( (p(:)**2)/m(:))

  do i = 1,ndf
     ! Polynomial for ith degree of freedom
     if (polydegree(i) > 0) then
        champolymorse = champolymorse + polyeng(i,q(i),polydegree(i))
     end if
     ! Morse potential for the ith degree of freedom
     if (morse(i)) then

        champolymorse = champolymorse &
             + morseeng(i,morsealpha(i),q(i),morseq0(i),morseA(i),morseC(i))
     end if

     if(gaussian(i)) then
        champolymorse = champolymorse &
             + gaussA(i)*gausseng(i,zic(i),zj(i),gaussalpha(i),gaussq0(i))
     end if
  end do

  ! ---- Coupling ----
  index = 1

  do i = 1,ndf-1
     do j = i+1,ndf
        ! ---- Calculate switch mode if applicable ----
        if (coupswitch(index) .eq. 1) then
           coups = 0.5d0*(1  &
           +coupswitchdir(index)*cderror(sgamma(coupswitchmode(index))*(coupswitchpos(index)-q(coupswitchmode(index)))))

           if (abs(coups) .gt. 1d-9) then
              ! If 'on' calculate optionally truncated coupling
              champolymorse = champolymorse + coups*hampolymorsecoup(i,j,index,q,p)
           end if
        else if (coupswitch(index) .eq. 2) then
           coups = dsqrt(gamma(coupswitchmode(index))/(gamma(coupswitchmode(index))+coupswitchgamma(index)))&
                *cdexp(-gamma(coupswitchmode(index))*coupswitchgamma(index)*q(coupswitchmode(index))**2&
                       /(gamma(coupswitchmode(index))+coupswitchgamma(index)))
           if (abs(coups) .gt. 1d-9) then
              ! If 'on' calculate optionally truncated coupling
              champolymorse = champolymorse + coups*hampolymorsecoup(i,j,index,q,p)
           end if

        else
           ! No switch, calculate  optionally truncated coupling
           champolymorse = champolymorse + hampolymorsecoup(i,j,index,q,p)
        end if

        index = index + 1
     end do
  end do
end function champolymorse

! ==================================================
!          Coupling Hamiltomian 
! 
!  A subroutine to calculate the coupling between
!  two modes. The modes are given by i,j.
! ==================================================
function hampolymorsecoup(i,j,index,q,p)
    use coherentstates, ONLY: gamma,sgamma
  use arrays, ONLY: sqrtpi
  use files, ONLY: fug
  implicit none
  integer,intent(in) :: i,j,index
  complex(kind=8),dimension(*) :: q,p
  
  complex(kind=8)::hampolymorsecoup ! output coupling energy

  complex(kind=8) :: errfn,errfn2,expfn2,expfn ! errorfunction
  complex(kind=8) :: C1i,C1j,C2i,C2j

  ! For maple code
  real(kind=8) :: t6,t9,t52,t71
  complex(kind=8) :: t1,t2,t4,t11,t12,t13,t14,t16,t17,t18,t19,t22,t23,t33,t34,t43,t44,t54,t58,t67,t69

  interface 
     include "MCERROR.interface"
     include "rerfunc.interface"
  end interface

  if (trunc(index) .eq. 0) then
     ! ---- untruncated function ----
     !              hampolymorse = hampolymorse & 
     hampolymorsecoup =  harmcouplea12(index) * q(i)*q(j)**2 &
          + harmcouplea21(index) * q(i)**2*q(j) & 
          + harmcouplea22(index) * q(i)**2*q(j)**2
    
  else if (trunc(index) .eq. 1 .or. trunc(index) .eq. -1 ) then
     ! truncate the coupling potential smoothly.
     hampolymorsecoup =  ( &
          harmcouplea12(index) * q(i)*q(j)**2 &
          + harmcouplea21(index) * q(i)**2*q(j) & 
          + harmcouplea22(index) * q(i)**2*q(j)**2 &
          ) &
          * 0.5*( rerfunc(-trunc(index)*&
             (real(q(truncm(index)))-truncpos(index))/truncwidth(index)) +1)
!              *0.5*( rerfunc((-1.0*trunc(index)*&
!          real(q(truncm(index)))-truncpos(index))/truncwidth(index)) +1) ! switching fct

  else if (trunc(index) .eq. 2 .or. trunc(index) .eq. -2 ) then
     ! Truncate using a heaviside. The factor of 2 is because trunc(index) = +/- 2 rather than +/- 1
     ! Note only the real portion of the position is considered. The position is only complex if we 
     ! are considering q = A(zi+zj*) for i .ne. j.
     if ( 0.5*trunc(index)*real(q(truncm(index))) <  0.5*trunc(index)*truncpos(index))then
        hampolymorsecoup = &
             harmcouplea12(index) * q(i)*q(j)**2 &
             + harmcouplea21(index) * q(i)**2*q(j) & 
             + harmcouplea22(index) * q(i)**2*q(j)**2
     end if

  else if (trunc(index) .eq. 3) then ! truncate in both directions for one mode
     ! Truncate using two heavisides. Need |q| < cutoff for non-zero coupling
     !The factor of 2 is because trunc(index) = +/- 2 rather than +/- 1
     if ( abs(q(truncm(index))) < truncpos(index))then
        hampolymorsecoup = &
             harmcouplea12(index) * q(i)*q(j)**2 &
             + harmcouplea21(index) * q(i)**2*q(j) & 
             + harmcouplea22(index) * q(i)**2*q(j)**2
     end if

  else if (trunc(index) .eq. 4) then 
     ! Truncate using two heavisides. Need |q| < cutoff for non-zero coupling
     !The factor of 2 is because trunc(index) = +/- 2 rather than +/- 1
     if ( (abs(q(i)) < truncpos(index)) .and.  (abs(q(j)) < truncpos2(index)))then
        hampolymorsecoup = &
             harmcouplea12(index) * q(i)*q(j)**2 &
             + harmcouplea21(index) * q(i)**2*q(j) & 
             + harmcouplea22(index) * q(i)**2*q(j)**2
     end if
 
  else if (trunc(index) .eq. 5) then
          ! truncate the coupling potential smoothly.
     hampolymorsecoup =  ( &
          harmcouplea12(index) * q(i)*q(j)**2 &
          + harmcouplea21(index) * q(i)**2*q(j) & 
          + harmcouplea22(index) * q(i)**2*q(j)**2 &
          ) &
          * 0.5*( &
          rerfunc(-(real(q(truncm(index)))-truncpos(index))/truncwidth(index)) &
          + rerfunc((real(q(truncm(index)))+truncpos(index))/truncwidth(index)))

  else
     write(*,*) "I do not know how to calculate the energy using trunction type ",trunc(index)
     stop
  end if

end function hampolymorsecoup
   ! ==================================================
   !          Polynomial + Morse Hamiltomian
   !  Calculate the energy of only one mode.
   ! ==================================================
  
subroutine champolymorse1mode(zic,zj,mode,Umode,Ucoup) 
  use coherentstates, ONLY: ndf,gamma,sgamma
  ! include subroutines too.
  use coherentstates, ONLY: pos,mom
  use arrays, ONLY: pi,sqrtpi,sqrt2

  implicit none

  ! two states <z_i| and |z_j>
  complex(kind=8),dimension(ndf),intent(in) :: zic,zj
  integer,intent(in) :: mode
  complex(kind=8),intent(out)::Umode,Ucoup

  ! hamiltonian
  !    complex(kind=8)::hampolymorse1mode

  ! position and momentum vectors
  complex(kind=8),dimension(ndf) :: q,p
  integer :: i,j,index,index2

  complex(kind=8) :: errfn,errfn2,expfn2,expfn ! errorfunction
  complex(kind=8) :: C1i,C1j,C2i,C2j

  complex(kind=8) :: Ucouppart  ! a contribution to the coupling energy
  complex(kind=8) :: coups ! coupling switching factor

  !    real(kind=8),external :: derf ! derf isn't actually an intrinsic function
  !                                  ! but this is needed for the PG compiler.

  interface 
     include "MCERROR.interface"
     include "rerfunc.interface"
  end interface

  ! Calculate position and momentum vectors
  call pos(zj,zic,q)
  call mom(zj,zic,p)

  ! Now calculate the classical Hamiltonian
  ! p_i^2/2m_i
  Umode = 0.5* p(mode)**2


  ! Polynomial for ith degree of freedom
  if (polydegree(mode) > 0) then
     Umode = Umode &
          + polyeng(mode,q(mode),polydegree(mode))
  end if
  ! Morse potential for the ith degree of freedom
  if (morse(mode)) then

     Umode = Umode &
          + morseeng(mode,morsealpha(mode),q(mode),morseq0(mode),morseA(mode),morseC(mode))
  end if

  if(gaussian(mode)) then
     Umode = Umode &
          + gaussA(mode)*gausseng(mode,zic(mode),zj(mode),gaussalpha(mode),gaussq0(mode))
  end if

  ! ---- Coupling ----
  Ucoup = (0.0,0.0)
 ! write(113,*) "# Next step"
  index = 1
  index2 = 0                  
  do i = 1,ndf-1
     do j = i+1,ndf
        if (i .eq. mode .or. j .eq. mode) then
           
           ! ---- Calculate switch mode if applicable ----
           if (coupswitch(index) .eq. 1) then
              coups = 0.5d0*(1  &
                   +coupswitchdir(index)*cderror(sgamma(coupswitchmode(index))*(coupswitchpos(index)-q(coupswitchmode(index)))))

!              write(113,*)index,real(coups),aimag(coups)
              if (abs(coups) .gt. 1d-9) then
                 ! If 'on' calculate optionally truncated coupling
                            ! Add the (switched) coupling energy
              Ucoup = Ucoup + coups*hampolymorsecoup(i,j,index,q,p)
              end if

           else if (coupswitch(index) .eq. 2) then
              coups = dsqrt(gamma(coupswitchmode(index))/(gamma(coupswitchmode(index))+coupswitchgamma(index)))&
                   *cdexp(-gamma(coupswitchmode(index))*coupswitchgamma(index)*q(coupswitchmode(index))**2&
                   /(gamma(coupswitchmode(index))+coupswitchgamma(index)))
              if (abs(coups) .gt. 1d-9) then
                 ! If 'on' calculate optionally truncated coupling
                 Ucoup = Ucoup + coups*hampolymorsecoup(i,j,index,q,p)
              end if
              
           else
              ! No switch, calculate  optionally truncated coupling
               Ucoup = Ucoup + hampolymorsecoup(i,j,index,q,p)
            end if

         end if
        index = index + 1
     end do
  end do

end subroutine champolymorse1mode

  ! ==================================================
  !                 Derivatives of 
  !        the Polynomial + Morse Hamiltomian 
  ! ==================================================
function dchampolymorsedz (zic,zj)
  use coherentstates, ONLY: ndf,gamma,s2gammarec,shgamma,sgamma
  ! include subroutines too.
  use coherentstates, ONLY: pos,mom
  use arrays, ONLY: pi,sqrtpi,sqrt2
  use files, ONLY: fug

  implicit none

  ! two states <z_i| and |z_j>
  complex(kind=8),dimension(ndf),intent(in) :: zic,zj
  ! hamiltonian
  complex(kind=8)::dchampolymorsedz(ndf)

  integer :: i,j,index
  ! position and momentum vectors
  complex(kind=8),dimension(ndf) :: q,p

  complex(kind=8) :: errfn,errfn2,expfn2,expfn ! errorfunction
  complex(kind=8) :: C1i,C1j,C2i,C2j,dC1di,dC2di,dC1dj,dC2dj

  ! For maple code
  real(kind=8) :: t6,t9,t52,t71
  complex(kind=8) :: t1,t2,t4,t11,t12,t13,t14,t16,t17,t18,t19,t22,t23,t33,t34,t43,t44,t54,t58,t67,t69


  complex(kind=8) :: coups  ! Coupling switching function.
  complex(kind=8):: dhampolymorsedzi,dhampolymorsedzj,dhampolymorsedzt ! contributions to the derivative

  real(kind=8) :: swf1,swf2
  !    real(kind=8),external :: derf ! derf isn't actually an intrinsic function
  !                                  ! but this is needed for the PG compiler.

  ! include interface for the complex error function
  interface 
     include "MCERROR.interface"
     include "rerfunc.interface"
  end interface

  ! Calculate position and momentum vectors
  call pos(zj,zic,q)
  call mom(zj,zic,p)

  ! Derivatives w.r.t. z_i
  ! dq_i/dz_i = 1/(sqrt( 2gamma(i))) = s2gammarec(i)
  ! dp_i/dz_i = -ihbar*sqrt(gamma(i)/2) = (0,-1)*shgamma(i) as hbar = 1

  ! Kinetic part
  dchampolymorsedz(:) = p(:)*(0,-1)*shgamma(:)/m(:)

  do i = 1,ndf
     ! Polynomial for ith degree of freedom
     if (polydegree(i) > 0) then
        dchampolymorsedz(i) = dchampolymorsedz(i) &
             + dpolyengdq(i,q(i),polydegree(i))*s2gammarec(i)
     end if
     ! Morse potential for the ith degree of freedom
     if (morse(i)) then
        dchampolymorsedz(i) = dchampolymorsedz(i) &
             + dmorseengdq(i,morsealpha(i),q(i),morseq0(i),morseA(i))*s2gammarec(i)
     end if
     if (gaussian(i)) then
        ! don't need the s2gammarec as dgaussengdz does derivative wrt z rather than q.
        dchampolymorsedz(i) = dchampolymorsedz(i) &
             + gaussA(i)*dgaussengdz(i,zic(i),zj(i),gaussalpha(i),gaussq0(i))
     end if
  end do

  ! ---- Coupling ----
  index = 1
  do i = 1,ndf-1
     do j = i+1,ndf

        ! ---- optionally apply a truncation function ----
        ! Apply a switching function too.
        if (coupswitch(index) .eq. 1) then
           coups = 0.5d0*(1  &
                +coupswitchdir(index)*cderror(sgamma(coupswitchmode(index))*(coupswitchpos(index)-q(coupswitchmode(index)))))
        else if (coupswitch(index) .eq. 2) then
           coups = dsqrt(gamma(coupswitchmode(index))/(gamma(coupswitchmode(index))+coupswitchgamma(index)))&
                *cdexp(-gamma(coupswitchmode(index))*coupswitchgamma(index)*q(coupswitchmode(index))**2&
                /(gamma(coupswitchmode(index))+coupswitchgamma(index)))
        else
           coups = 1.0d0
        end if

        if (abs(coups) .gt. 1d-9) then
           ! Set the derivatives to zero incase cutoffs mean they don't get assigned below.
           dhampolymorsedzi = (0.0d0,0.0d0)
           dhampolymorsedzj = (0.0d0,0.0d0)

           if (trunc(index) .eq. 0) then 
              ! Default
              ! Don't apply any truncation
              dhampolymorsedzi = (harmcouplea12(index) * (q(j)**2) &
                   + 2*harmcouplea21(index) * q(i)*q(j) &
                   + harmcouplea22(index) *2*q(i)*q(j)**2) & 
                   * s2gammarec(i)
              
              !             dhampolymorsedz(j) = dhampolymorsedz(j) &
              dhampolymorsedzj = (harmcouplea21(index) * (q(i)**2) &
                   + 2*harmcouplea12(index) * q(i)*q(j) &
                   + harmcouplea22(index) *2*q(j)*q(i)**2) &
                   * s2gammarec(j)

              
           else if (trunc(index) .eq. 1 .or. trunc(index) .eq. -1 ) then
              ! truncate the coupling potential smoothly.
              ! The switching function
              swf1 = 0.5*( rerfunc(-trunc(index)*&
                   (real(q(truncm(index)))-truncpos(index))/truncwidth(index)) +1)

              dhampolymorsedzi = (harmcouplea12(index) * (q(j)**2) &
                   + 2*harmcouplea21(index) * q(i)*q(j) &
                   + harmcouplea22(index) *2*q(i)*q(j)**2) & 
                   * s2gammarec(i) *swf1

              dhampolymorsedzj = (harmcouplea21(index) * (q(i)**2) &
                   + 2*harmcouplea12(index) * q(i)*q(j) &
                   + harmcouplea22(index) *2*q(j)*q(i)**2) &
                   * s2gammarec(j) *swf1
              ! extra term for the truncated mode
              dhampolymorsedzt = &
                   -trunc(index)*exp(- ((real(q(truncm(index)))-truncpos(index))/truncwidth(index))**2)/sqrtpi/truncwidth(index)&
                   * s2gammarec(truncm(index)) &
                   * ( harmcouplea12(index) * (q(i)*q(j)**2) &
                   + harmcouplea21(index) * (q(i)**2*q(j) ) & 
                   + harmcouplea22(index) * (q(i)**2*q(j)**2 ))

              if (truncm(index) .eq. i) then
                 dhampolymorsedzi = dhampolymorsedzi + dhampolymorsedzt
              else
                 dhampolymorsedzj = dhampolymorsedzj + dhampolymorsedzt
              end if

           else if (trunc(index) .eq. 2 .or. trunc(index) .eq. -2 ) then
              ! Truncate using a heaviside. The factor of 2 is because trunc(index) = +/- 2 rather than +/- 1
              ! Note only the real portion of the position is considered. The position is only complex if we 
              ! are considering q = A(zi+zj*) for i .ne. j.
              if ( 0.5*trunc(index)*real(q(truncm(index))) <  0.5*trunc(index)*truncpos(index))then

                 dhampolymorsedzi = (harmcouplea12(index) * (q(j)**2) &
                      + 2*harmcouplea21(index) * q(i)*q(j) &
                      + harmcouplea22(index) * 2 *q(i)*q(j)**2) & 
                      * s2gammarec(i)

                 dhampolymorsedzj = (harmcouplea21(index) * (q(i)**2) &
                      + 2*harmcouplea12(index) * q(i)*q(j) &
                      + harmcouplea22(index) *2*q(j)*q(i)**2) &
                      * s2gammarec(j)
              end if

           else if (trunc(index) .eq. 3) then ! truncate in both directions for one mode
              ! Truncate using two heavisides. Need |q| < cutoff for non-zero coupling
              !The factor of 2 is because trunc(index) = +/- 2 rather than +/- 1
              if ( abs(q(truncm(index))) < truncpos(index))then
                 dhampolymorsedzi = (harmcouplea12(index) * (q(j)**2) &
                      + 2*harmcouplea21(index) * q(i)*q(j) &
                      + harmcouplea22(index) * 2 *q(i)*q(j)**2) & 
                      * s2gammarec(i)

                 dhampolymorsedzj = (harmcouplea21(index) * (q(i)**2) &
                      + 2*harmcouplea12(index) * q(i)*q(j) &
                      + harmcouplea22(index) *2*q(j)*q(i)**2) &
                      * s2gammarec(j)
              end if

           else if (trunc(index) .eq. 4) then 
              ! Truncate using two heavisides. Need |q| < cutoff for non-zero coupling
              !The factor of 2 is because trunc(index) = +/- 2 rather than +/- 1
              if ( (abs(q(i)) < truncpos(index)) .and.  (abs(q(j)) < truncpos2(index)))then
                 dhampolymorsedzi = (harmcouplea12(index) * (q(j)**2) &
                      + 2*harmcouplea21(index) * q(i)*q(j) &
                      + harmcouplea22(index) * 2 *q(i)*q(j)**2) & 
                      * s2gammarec(i)

                 dhampolymorsedzj = (harmcouplea21(index) * (q(i)**2) &
                      + 2*harmcouplea12(index) * q(i)*q(j) &
                      + harmcouplea22(index) *2*q(j)*q(i)**2) &
                      * s2gammarec(j)
              end if
           else if (trunc(index) .eq. 5 ) then
              ! truncate the coupling potential smoothly in both directions
              ! The switching function
              ! Truncation for +ve q
              swf1 = 0.5*( rerfunc(&
                   -(real(q(truncm(index)))-truncpos(index))/truncwidth(index)) +1)
              ! Truncation for -ve q
              swf2 = 0.5*( rerfunc(&
                   (real(q(truncm(index)))+truncpos(index))/truncwidth(index)) +1)

              dhampolymorsedzi = (harmcouplea12(index) * (q(j)**2) &
                   + 2*harmcouplea21(index) * q(i)*q(j) &
                   + harmcouplea22(index) *2*q(i)*q(j)**2) & 
                   * s2gammarec(i) *swf1*swf2

              dhampolymorsedzj = (harmcouplea21(index) * (q(i)**2) &
                   + 2*harmcouplea12(index) * q(i)*q(j) &
                   + harmcouplea22(index) *2*q(j)*q(i)**2) &
                   * s2gammarec(j) *swf1*swf2
              ! extra term for the truncated mode
!!$              dhampolymorsedzt = &
!!$                   (swf1*exp(- ((real(q(truncm(index)))+truncpos(index))/truncwidth(index))**2)/sqrtpi/truncwidth(index) &
!!$                   -swf2*exp(- ((real(q(truncm(index)))-truncpos(index))/truncwidth(index))**2)/sqrtpi/truncwidth(index))&
!!$                   * s2gammarec(truncm(index)) &
!!$                   * ( harmcouplea12(index) * (q(i)*q(j)**2) &
!!$                   + harmcouplea21(index) * (q(i)**2*q(j) ) & 
!!$                   + harmcouplea22(index) * (q(i)**2*q(j)**2 ))

               ! extra term for the truncated mode
              dhampolymorsedzt = &
                   ( exp(- ((real(q(truncm(index)))+truncpos(index))/truncwidth(index))**2)&
                   -exp(- ((real(q(truncm(index)))-truncpos(index))/truncwidth(index))**2))/sqrtpi/truncwidth(index)&
                   * s2gammarec(truncm(index)) &
                   * ( harmcouplea12(index) * (q(i)*q(j)**2) &
                   + harmcouplea21(index) * (q(i)**2*q(j) ) & 
                   + harmcouplea22(index) * (q(i)**2*q(j)**2 ))

              if (truncm(index) .eq. i) then
                 dhampolymorsedzi = dhampolymorsedzi + dhampolymorsedzt
              else
                 dhampolymorsedzj = dhampolymorsedzj + dhampolymorsedzt
              end if
           else ! Default - fail
              write(fug,*) "I don't know how to calculate the derivative of trunction type ",trunc(index)
              stop
           end if

!!$           dchampolymorsedz(i) = dchampolymorsedz(i) + coups*dhampolymorsedzi
!!$           dchampolymorsedz(j) = dchampolymorsedz(j) + coups*dhampolymorsedzj
           dchampolymorsedz(i) = dchampolymorsedz(i) + dhampolymorsedzi
           dchampolymorsedz(j) = dchampolymorsedz(j) + dhampolymorsedzj
           
!!$           if (coupswitch(index) .eq. 1) then
!!$              dchampolymorsedz(coupswitchmode(index)) = dchampolymorsedz(coupswitchmode(index)) &
!!$                   - 1/sqrtpi/sqrt2*coupswitchdir(index) &
!!$                   *cdexp(-gamma(coupswitchmode(index))*(coupswitchpos(index)-q(coupswitchmode(index)))**2) &
!!$                   * hampolymorsecoup(i,j,index,q,p)
!!$              !s2gammarec(coupswitchmode(index))*sgamma(coupswitchmode(index)) =  1/sqrt2
!!$           else if (coupswitch(index) .eq. 2) then
!!$              dchampolymorsedz(coupswitchmode(index)) =  dchampolymorsedz(coupswitchmode(index)) &
!!$                   - 2*s2gammarec(coupswitchmode(index)) * coupswitchgamma(index)*q(coupswitchmode(index))&
!!$                   * (dsqrt(gamma(coupswitchmode(index))/(gamma(coupswitchmode(index))+coupswitchgamma(index))))**3 &
!!$                   * cdexp(-gamma(coupswitchmode(index))*coupswitchgamma(index)*q(coupswitchmode(index))**2&
!!$                   /(gamma(coupswitchmode(index))+coupswitchgamma(index))) &
!!$                   * hampolymorsecoup(i,j,index,q,p)
!!$           end if
        end if ! coups > 1e-9 ! ***** This may not be in the correct place for future switches. ****
        index = index + 1
     end do
  end do
end function dchampolymorsedz

  
  ! ==================================================
  !                Reordered Polynomial
  ! ==================================================
   function polyeng(i,qi,pd) 
    use coherentstates, ONLY: gamma
    implicit none

    integer, intent(in) :: i
    ! degree of poynomial
    integer, intent(in) :: pd 
    !Position of ith degree of freedom
    complex(kind=8), intent(in) :: qi 
    ! energy
    complex(kind=8) :: polyeng

    integer :: p ! for loop - power 
    ! 
    polyeng = pcoeff(i,0)+ pcoeff(i,1)*qi
    if (pd .eq. 1) return

    ! Calculate higher terms in a loop
    do p = 2,pd
       polyeng = polyeng &
            +  pcoeff(i,p) * qi**p
    end do
    
  end function polyeng


  ! ==================================================
  !    Derivative w.r.t. q of Reordered Polynomial
  ! ==================================================  
   function dpolyengdq(i,qi,pd) 
    use coherentstates, ONLY: gamma
    implicit none
    integer, intent(in) :: i
    ! degree of poynomial
    integer, intent(in) :: pd 
    !Position of ith degree of freedom
    complex(kind=8), intent(in) :: qi 
    ! energy
    complex(kind=8) :: dpolyengdq
    integer :: p ! for loop - power 
!    dpolyengdq = 0.0

    ! linear term
    dpolyengdq = pcoeff(i,1)*(1.0,0.0)
    if (pd .eq. 1) return

    ! Calculate higher terms in a loop
    do p = 2,pd
       dpolyengdq = dpolyengdq &
            +  p*pcoeff(i,p) * qi**(p-1)
    end do
  end function dpolyengdq


  ! ==================================================
  !                Reordered Morse fct
  ! ==================================================
  function morseeng(i,alpha,qi,q0,A,mc)
    use coherentstates, ONLY: ndf,gamma
    implicit none
    integer,intent(in) :: i
    real(kind=8),intent(in) :: alpha
    complex(kind=8), intent(in) :: qi
    real(kind=8), intent(in) :: q0
    real(kind=8), intent(in) :: A
    real(kind=8), intent(in) :: mc

    complex(kind=8) :: morseeng


    morseeng = A*(1.0d0-exp(-alpha*(qi-q0)))**2 + mc
  end function morseeng

  ! ==================================================
  !   Derivative wrt q of th reordered Morse fct
  ! ==================================================
  function dmorseengdq(i,alpha,qi,q0,A)
    use coherentstates, ONLY: gamma
    implicit none
    integer,intent(in) :: i
    real(kind=8),intent(in) :: alpha
    complex(kind=8), intent(in) :: qi
    real(kind=8), intent(in) :: q0
    real(kind=8), intent(in) :: A

    complex(kind=8) :: dmorseengdq
    
    dmorseengdq = A*(2.0d0,0.0d0)*alpha*(1.0d0-exp(-alpha*(qi-q0)))*exp(-alpha*(qi-q0))
    
  end function dmorseengdq

  ! ==================================================
  !                Reordered Gaussian fct
  ! ==================================================
  function gausseng(i,zic,zj,alpha,q0)
    use coherentstates, ONLY: gamma
    implicit none
    integer,intent(in) :: i
    complex(kind=8),intent(in) :: zic,zj
        
    real(kind=8),intent(in) :: alpha
    real(kind=8), intent(in) :: q0

    complex(kind=8) :: gausseng

    gausseng = exp(-0.5*(zic+zj)*(zic+zj)*alpha/gamma(i))

  end function gausseng

  ! ==================================================
  !   Derivative wrt q of th reordered Gaussian fct
  ! ==================================================
  function dgaussengdz(i,zic,zj,alpha,q0)
    use coherentstates, ONLY: ndf,gamma
    implicit none
    integer,intent(in) :: i
    complex(kind=8),intent(in) :: zic,zj
        
    real(kind=8),intent(in) :: alpha
    real(kind=8), intent(in) :: q0

    complex(kind=8) :: dgaussengdz

    dgaussengdz = -alpha/gamma(i) *(zic+zj)* exp(-0.5*(zic+zj)*(zic+zj)*alpha/gamma(i))
   
  end function dgaussengdz



  ! ==================================================
  !            Read Hamiltomian parameters
  !     specific to this Hamiltonian
  !  allocates arrays too.
  ! ==================================================
  subroutine readchampm 
    use options, ONLY: verbosity,writeexpect,eu
    use  coherentstates, ONLY: ndf
    use files, ONLY: fug,fham
    use arrays, ONLY: dhexpect,hework
    use errors
    use scalapackstuff, ONLY: isroot

    character(len=250) ::line
    character(len=50) :: line2 ! for formatted output
    character(len=5) :: line3 ! for formatted output
    integer :: ierr ! For errors reading from the file
    integer :: i,j    ! For loops, what else?
    integer :: index,coupling
    integer :: numcouplings ! How many couplings are read in?
    real(kind=8) a12,a21,a22 ! For reading in coupling.
    integer :: numtruncs,tt,tm ! number of truncations in file and truncation type and mode
    real(kind=8) :: tp,tp2,tw ! truncation position and width


    integer :: numswitches,csw,st,sm,sd
    real(kind=8) :: sp,sg

    ! Allocate more general arrays
    call allocatehamarrays

    ! Allocate Hamiltonian specific arrays
    allocate(polydegree(ndf))
    allocate(pcoeff(ndf,0:10))
    allocate(morse(ndf))
    allocate(morseA(ndf))
    allocate(morsealpha(ndf))
    allocate(morseq0(ndf))
    allocate(morseC(ndf))
    allocate(gaussian(ndf))
    allocate(gaussA(ndf))
    allocate(gaussalpha(ndf))
    allocate(gaussq0(ndf))

    allocate(harmcouplea12(ndf*(ndf-1)/2))
    allocate(harmcouplea21(ndf*(ndf-1)/2))
    allocate(harmcouplea22(ndf*(ndf-1)/2))
    
    ! For coupling truncation
    allocate(trunc(ndf*(ndf-1)/2))
    allocate(truncm(ndf*(ndf-1)/2))
    allocate(truncpos(ndf*(ndf-1)/2))
    allocate(truncpos2(ndf*(ndf-1)/2))
    allocate(truncwidth(ndf*(ndf-1)/2))

    ! For coupling switching
    allocate(coupswitch(ndf*(ndf-1)/2))
    allocate(coupswitchmode(ndf*(ndf-1)/2))
    allocate(coupswitchdir(ndf*(ndf-1)/2))
    allocate(coupswitchpos(ndf*(ndf-1)/2))
    allocate(coupswitchgamma(ndf*(ndf-1)/2))


    ! For dropping basis functions
    if (writeexpect > 0 ) then
       allocate(dhexpect(ndf+1))
       allocate(hework(ndf+1))
       dhexpect = 0.0d0
    end if

    ! Empty arrays
    polydegree(:) = 0
    pcoeff(:,:) = 0.0
    morse(:) = .false.
    morseA(:) = 0.0
    morsealpha(:) = 0.0
    morseq0(:) = 0.0
    gaussian(:) = .false.
    gaussA(:) = 0.0
    gaussalpha(:) = 0.0
    gaussq0(:) = 0.0

    harmcouplea12(:)= 0.0
    harmcouplea21(:)= 0.0
    harmcouplea22(:)= 0.0

    trunc(:) = 0
    truncm(:) = 0
    truncpos(:) = 0.0
    truncpos2(:) = 0.0
    truncwidth(:) = 0.0

    coupswitch(:) = 0
    coupswitchmode(:) = 0
    coupswitchdir(:) = 0
    coupswitchpos(:) = 0.0d0
    coupswitchgamma(:) = 0.0d0

    ! ---- Read in masses -----
    read(fham,*,iostat=ierr) line
    if  (line .ne. "mass" .or. ierr .ne. 0  ) then
       if (isroot) write(fug,*) "Failed to read keyword mass in hamiltonian.inpt"
       stop
    end if
    do i =1,ndf
       read(fham,*,iostat=ierr) m(i)
       if(ierr.ne.0) call filereaderror(ierr,"hamiltonian.inpt","Error reading masses")
    end do


    ! ---- Now read in mode energy fct ----
    ! Read first key word mode
   read(fham,*,iostat=ierr) line

    do i = 1,ndf
       ! Check that the keyword is mode
       if  (line .ne. "mode" ) then
          if (isroot) write(fug,*) "Failed to read keyword mode in hamiltonian.inpt"
          stop
       else if ( ierr .ne. 0 ) then 
          call filereaderror(ierr,"hamiltonian.inpt","Error reading mode")
       end if

       ! Now read interaction type
       read(fham,*,iostat=ierr) line

       do while (line .ne. "mode" .and. ierr .eq.0 .and. line .ne. "end")

          ! ---- polynomial ----
          if (line .eq. "poly") then

             backspace(fham)
             read(fham,*,iostat=ierr) line,polydegree(i)
             
             if  ( ierr .ne. 0 .or. polydegree(i) .le. 0 ) then
                if (isroot) write(fug,*) "Failed to read suitable degree of polynomial in hamiltonian.inpt for mode ",i
                stop
             end if
             do j = 0,polydegree(i)
                read(fham,*,iostat=ierr) line,pcoeff(i,j)
                if  ( ierr .ne. 0 .or. polydegree(i) .le. 0 ) then
                   if(isroot) write(fug,*) "Failed to read coefficient for  degree",j," for mode ",i
                   stop
                end if
                ! Convert to internal energy units
                pcoeff(i,j) = pcoeff(i,j)/eu
             end do

             ! ---- morse ----
          else if (line .eq. "morse" ) then
             morse(i) = .true.
             read(fham,*,iostat=ierr) line,morseA(i)
             if  ( line .ne. "A" .or.  ierr .ne. 0  ) then
                if (isroot) write(fug,*) "Failed to read coefficient A for morse potential of mode ",i
                stop
             end if
              ! Convert to internal energy units
             morseA(i) = morseA(i)/eu

             read(fham,*,iostat=ierr) line,morsealpha(i)
             if  ( line .ne. "alpha" .or. ierr .ne. 0  ) then
                if (isroot) write(fug,*) "Failed to read coefficient alpha for morse potential of mode ",i
                stop
             end if
             read(fham,*,iostat=ierr) line,morseq0(i)
             if  ( line .ne. "q0" .or. ierr .ne. 0  ) then
                if (isroot) write(fug,*) "Failed to read q0 for morse potential of mode ",i
                stop
             end if
             read(fham,*,iostat=ierr) line,morseC(i)
             if  ( line .ne. "C" .or. ierr .ne. 0  ) then
                if (isroot) write(fug,*) "Failed to read C for morse potential of mode ",i
                stop
             end if
              ! Convert to internal energy units
             morseC(i) = morseC(i) /eu

             ! ---- Gaussian ----
          else if (line .eq. "gaussian" ) then
             gaussian(i) = .true.
             read(fham,*,iostat=ierr) line,gaussA(i)
             if  ( line .ne. "A" .or.  ierr .ne. 0  ) then
                if(isroot) write(fug,*) "Failed to read coefficient A for Gaussian potential of mode ",i
                stop
             end if
              ! Convert to internal energy units
             gaussA(i) = gaussA(i) /eu

             read(fham,*,iostat=ierr) line,gaussalpha(i)
             if  ( line .ne. "alpha" .or. ierr .ne. 0  ) then
                if(isroot) write(fug,*) "Failed to read coefficient alpha for morse potential of mode ",i
                stop
             end if
             read(fham,*,iostat=ierr) line,gaussq0(i)
             if  ( line .ne. "q0" .or. ierr .ne. 0  ) then
                if (isroot) write(fug,*) "Failed to read q0 for morse potential of mode ",i
                stop
             end if
          else 
             if (isroot) write(fug,*)  "Found unknown potential type in hamiltonian.inpt", line
          end if
          ! Read in next line
          read(fham,*,iostat=ierr) line
       end do ! do while
    end do

    ! ---- Mode Coupling ----
    read(fham,*,iostat=ierr) line

    numcouplings = 0

    if (line .eq. "coupling") then

       backspace(fham)
       read(fham,*,iostat=ierr) line,numcouplings
       if ( ierr.ne. 0 )then
          if (isroot) write(fug,*) "Error reading number of couplings from hamiltonian.inpt"
          if (isroot) write(fug,*) line,numcouplings
          stop
       end if

       if (verbosity .ge. 1 .and. isroot) then
          write(fug,*) "Reading ",numcouplings," couplings"
       end if

       do coupling = 1,numcouplings

          read(fham,*,iostat = ierr) i,j, a12,a21,a22
          if ( ierr.ne. 0 )then
             if (isroot) write(fug,*) "Error reading coupling ",coupling," from hamiltonian.inpt"
             stop
          end if
          if ( i .eq. j) then
             if (isroot) write(fug,*) "Can't couple a mode to itself in hamiltonian.inpt"
             stop
          else if ( i .gt.j) then
             if (isroot) write(fug,*) " In coupling potential first index has to be less than the second index."
             stop
          end if
          
          index = j-i + ndf*(ndf-1)/2 - (ndf-i+1)*(ndf-i)/2
          
          harmcouplea12(index)=a12/eu
          harmcouplea21(index)=a21/eu
          harmcouplea22(index)=a22/eu
       end do
       
       ! Check to see if the is a coupling for each pair of degrees of freedom
       if (numcouplings .lt. ndf*(ndf-1)/2) then
          if (isroot) write(fug,*) "  Warning: Only read ",numcouplings, " sets of couplings."
          if (isroot) write(fug,*) "  Setting other couplings to zero" 	
          if (isroot) write(fug,*) 
       end if


       ! ---- Mode Coupling truncation ----
       read(fham,*,iostat=ierr) line

       numtruncs = 0
       
       if (line .eq. "couptrunc") then

          backspace(fham)
          read(fham,*,iostat=ierr) line,numtruncs
          if ( ierr.ne. 0 )then
             if (isroot) then
                write(fug,*) "Error reading number of couplings from hamiltonian.inpt"
                write(fug,*) line,numtruncs
             end if
             stop
          end if
          
          if (verbosity .ge. 1 .and. isroot) then
             write(fug,*) "Reading ",numtruncs," truncations"
          end if
          
          do coupling = 1,numtruncs
             
             read(fham,*,iostat = ierr) i,j, tt
             !,tm,tp,tw

             if ( ierr.ne. 0 )then
                if (isroot) write(fug,*) "Error reading truncation ",coupling," from hamiltonian.inpt"
                stop
             end if
             if ( i .eq. j) then
                if (isroot) write(fug,*) "Can't couple a mode to itself in hamiltonian.inpt"
                stop
             else if ( i .gt.j) then
                if (isroot) write(fug,*) " In coupling potential truncation first index has to be less than the second index."
                stop
             end if

             tw = 0.0
             tp2 = 0.0
             backspace(fham)
             if (tt .eq. 0) then ! No truncation
                read(fham,*,iostat = ierr) i,j, tt
             else if ( tt.eq. -1 .or. tt.eq.1) then
                read(fham,*,iostat = ierr) i,j, tt,tm,tp,tw
             else if  ( tt.eq. -2 .or. tt.eq.2) then
                read(fham,*,iostat = ierr) i,j, tt,tm,tp
             else if  ( tt.eq.3) then
                read(fham,*,iostat = ierr) i,j, tt,tm,tp
             else if  ( tt.eq.4) then
                read(fham,*,iostat = ierr) i,j, tt,tp,tp2
             else if  ( tt.eq.5) then
                read(fham,*,iostat = ierr) i,j, tt,tm,tp,tw
             else
                if (isroot) then
                   write(fug,*) " Error reading coupling trunctation:"
                   write(fug,*) "     Unknown truncation method",tt
                end if
                stop
             end if

             index = j-i + ndf*(ndf-1)/2 - (ndf-i+1)*(ndf-i)/2
             
             ! save details of this truncation
             trunc(index)=tt
             truncm(index)=tm
             truncpos(index)=tp
             truncpos2(index)=tp2
             truncwidth(index)=tw
          end do
       end if
    end if
    
    ! ---- Mode Coupling switching ----
    read(fham,*,iostat=ierr) line
    
    numswitches = 0
    
    if (line .eq. "coupswitch") then
       
       backspace(fham)
       read(fham,*,iostat=ierr) line,numswitches
       if ( ierr.ne. 0 )then
          if (isroot) then
             write(fug,*) "Error reading number of coupling switches from hamiltonian.inpt"
             write(fug,*) line,numswitches
          end if
          stop
       end if
       
       if (verbosity .ge. 1 .and. isroot) then
          write(fug,*) "Reading ",numswitches," switches"
       end if
       
       do csw = 1,numswitches
          ! find out what type of switch is required
          read(fham,*,iostat = ierr) i,j,st
          backspace(fham)
          
          ! Set temporary variables to 0.
          sd = 0
          sp = 0.0d0
          sg = 0.0d0

          ! Read in details of switch 
          if( st .eq. 1) then
             read(fham,*,iostat = ierr) i,j,st,sm,sd,sp

          else if( st .eq. 2) then
             read(fham,*,iostat = ierr) i,j,st,sm,sg
          else
             if (isroot) then
                write(fug,*) "I don't know about switch type ",st," in hamiltonian.inpt"   
                write(fug,*) "i = ",i," j = ",j
             end if
             stop
          end if
          if (ierr.ne. 0 ) then
             if (isroot) write(fug,*) "Problem reading coupling switch ", csw," from hamiltonian file"
             stop
          end if
          ! Calculate the index and check it is valid.
          index = j-i + ndf*(ndf-1)/2 - (ndf-i+1)*(ndf-i)/2
          if (index <1 .or. index > (ndf*(ndf-1)/2) ) then
             if (isroot) then
                write(fug,*) "No such mode combination in  coupling switch ", csw," from hamiltonian file"
                write(fug,*) "i=",i,", j=",j,", index=",index
             end if
             stop
          end if

          ! save details of this switch
          coupswitch(index) = st
          coupswitchmode(index) = sm
          coupswitchdir(index) = sd
          coupswitchpos(index) = sp
          coupswitchgamma(index) = sg
       end do
    end if
    
    if (isroot) then
       write(fug,*) "Finished reading modes"
       write(fug,*) 
    end if

    ! Summarise
    if (isroot) then
       write(fug,*) "Classical Hamiltonian parameters"
       write(fug,*) "--------------------------------"
       write(fug,*) "  Studying a system with polynomial and morse potentials"
       write(fug,*) "  Masses:               i     m_i"
       do i = 1,ndf
          write(fug,"(a21,i5,f8.4)") "                     ",i,m(i)
       end do
       
       do i = 1,ndf
          write(fug,"(a7,i4)") "  Mode",i
          if(polydegree(i) > 0) then
             
             write(line,"(f12.6)") pcoeff(i,0)
             
             do j = 1,polydegree(i)
                ! Write power to string variable line3
                if (j .lt. 10) then
                   write(line3,"(i1)") j
                else if ( j .lt. 100) then
                   write(line3,"(i2)") j
                else
                   write(line3,"(i3)") j
                end if
                
                ! write coefficient etc.
                if( j .eq. 1) then
                   write(line2,"(a3,f8.4,a2,a4)")  " + ",pcoeff(i,j)*eu,"q"
                else
                   write(line2,"(a3,f8.4,a2,a4)")  " + ",pcoeff(i,j)*eu,"q^",line3
                end if
                line = trim(line)//trim(line2)
             end do
             write(fug,*) "     V =",trim(line)
          end if
          
          if(morse(i)) then
             if (polydegree(i) > 0) then
                write(fug,*) "        + ",morseA(i)*eu," exp(",&
                     -1*morsealpha(i),".(q-",morseq0(i),")) + " ,morseC(i)
             else
                write(fug,*) morseA(i)*eu," exp(",&
                     -1 * morsealpha(i),".(q-",morseq0(i),")) + " ,morseC(i)
             end if
          end if
          
          if(gaussian(i)) then
             if (polydegree(i) > 0 .or. morse(i)) then
                write(fug,*) "        + ",gaussA(i)*eu," exp(",&
                     -1*gaussalpha(i),".(q-",gaussq0(i),")^2)"
             else
                write(fug,*) gaussA(i)*eu," exp(",&
                     -1*gaussalpha(i),".(q-",gaussq0(i),"))"
             end if
          end if
          
       end do

       ! ---- summarise coupling and truncation ----
       if(numcouplings .gt. 0) then
          write(fug,*) 
          write(fug,*) "  Coupling parameters"
          write(fug,*) "       mode 1  mode 2   a12           a21           a22"
          index = 0
          do i = 1,ndf -1
             do j = i+1,ndf
                index = index + 1
                write (fug,"('    ',2I8,3es14.4)") i,j,harmcouplea12(index)*eu,harmcouplea21(index)*eu,harmcouplea22(index)*eu
             end do
          end do
          ! --- truncation ---
          if(numtruncs .gt. 0) then
             write(fug,*) 
             write(fug,*) "  Coupling truncation"
             write(fug,*) "       mode 1  mode 2   type   tcmode     position      width"
             index = 0
             do i = 1,ndf -1
                do j = i+1,ndf
                   index = index + 1
                   if(trunc(index) .ne. 0) then
                      write (fug,"('    ',4I8,'  ',2es14.4)") i,j,trunc(index), truncm(index),  truncpos(index), truncwidth(index)
                   end if
                end do
             end do
          end if
       end if ! if num couplings > 0
       ! --- switching function ---
       if (numswitches .gt. 0) then
          write(fug,*) 
          write(fug,*) "  Coupling switching "
          write(fug,*) "        Coupled modes  switch   type     dir    pos           width"
          index = 0
          do i = 1,ndf -1
             do j = i+1,ndf
                index = index + 1
                if(coupswitch(index) .ne. 0) then
                   write (fug,"('    ',5i8,2es14.4)") i,j,coupswitch(index), &
                        coupswitchmode(index),  coupswitchdir(index), &
                        coupswitchpos(index), coupswitchgamma(index)
                end if
             end do
          end do
       end if ! if numswitches > 0
    end if
  end subroutine readchampm
  
end module classicpolymorsehammod
