module arrays

  implicit none

  ! System type
  integer :: systemtype
  
  ! Number of cartesian dimensions
  integer :: ncartdim

  ! Number of particles
  integer :: nparticles

  ! Number of degrees of freedom
  integer :: ndf

  ! number of modes to be sampled from. Only really useful for non normal mode systems.
  integer :: nmodes
  
  real(kind=8) :: totalmass
  real(kind=8),dimension(:),allocatable :: mass

  logical :: reproject
  logical :: displace ! displace the basis functions away from 0.
  real(kind=8) :: omega0=1d-15 ! zero for overlap matrix
  real(kind=8) :: minbfprob ! minimum basis function probability.
  integer :: bfsepmode ! Should the phase space separation be considered when choosing basis functions?
                       ! < 0 No;  0 Total separation; > 0  for one mode only.
  real(kind=8) :: bfsep2 ! Min separation squared

  integer :: bfengrange ! 0 don't enforce energy range selection. 1 Do enforce it.
  real(kind=8) :: minbfeng, maxbfeng

  ! ---- The basis functions ----
  ! number of basis functions
  integer :: nbasis  
  ! the basis functions
  complex(kind=8),dimension(:,:),allocatable :: zb 
  ! amplitudes
  complex(kind=8),dimension(:),allocatable :: C
  ! The width parameters for the basis functions
  real(kind=8),dimension(:),allocatable :: gamma
  real(kind=8),dimension(:,:),allocatable :: x,y,z,px,py,pz
  ! For doing normal mode systems
  real(kind=8),dimension(:,:),allocatable :: qb,pb 
  real(kind=8),dimension(:),allocatable :: qb0,pb0 
  ! method for producing basis functions for each mode
  ! 1 - single, 2 - sum, 3 - file
  integer,dimension(:),allocatable :: method 

  ! ---- Ground state wave function ----
  ! For pure states. i.e. method - 1
  integer,dimension(:),allocatable:: mode 
  ! expanded in terms of harmonic oscillator functions
  ! max number of terms in expansion
  integer :: maxnterms 
  ! number of terms in expansion for each degree of freedom
  integer,dimension(:),allocatable :: nterms
  ! expansion coefficients
  real(kind=8),dimension(:,:),allocatable :: ec

  ! The width parameters for the harmonic oscillator functions
  real(kind=8),dimension(:),allocatable :: xi

  real(kind=8),dimension(:),allocatable :: q0,p0
  ! ---- For sampling ----
  real(kind=8),dimension(:),allocatable :: comp
  integer :: maxtries

  ! ---- For reading in basis functions fromfile ----
  integer :: nfiles ! number of modes from files.
  character(len=81),dimension(:),allocatable :: filenames
  integer :: nfbf ! number of basis functions in files
  ! make one large array which would cope if all degrees of freedom had points in files. 
  real(kind=8),dimension(:,:),allocatable :: qf,pf 
   ! this allows orders of these basisfunctions to be randomi
  integer,dimension(:,:),allocatable :: order
  ! Does the file contain a wavefn - 1 or just points 0
  integer,dimension(:),allocatable :: cswavefnfile
  ! store amplitudes could be C or D depending on usage.
  complex(kind=8),dimension(:,:),allocatable:: CDfile
  ! should the points be used as is or redistributed?
  integer,dimension(:),allocatable :: distpts
  ! for reading in trajectories for pairs of particles.
  integer :: filep1, filep2

  ! ---- File units ----
  integer :: fug = 6
  integer :: finputs = 10
  integer :: fham = 11  
  integer :: fbasis  = 20
  integer :: fqp = 21
  integer :: famps = 22
  integer :: fbamp = 23
  integer :: fwfn = 24
  integer(kind=8) :: randseed

end module arrays

