subroutine choosebasisfnnormalmodes
  use arrays
  use harmonicoscillator, ONLY: pickharmrnd
  implicit none

  complex(kind=8) :: hz(ndf),amps(ndf)
  integer :: dof,i,j,bf ! for loops
  
  ! For choosing which term in the expansion to sample
  integer :: term
  real(kind=8) ::pr,pl
  real(kind=8) :: zmax
  real(kind=8),external :: zbqlu01 ! random number generator

  integer,dimension(ndf) :: toorder
  integer :: thispt
  complex(kind=8) :: zj
  interface
     include 'reorder.inc'
     include 'readbasisfns.inc'
  end interface
  !createrandomlists(n,ndf,toorder,order)

  zmax = 50d0

  ! ---- Get basis fns from files if appropriate ----
  if (nfiles > 0) then
     call readbasisfns(ndf,nbasis,filenames,qf,pf,cswavefnfile,CDfile,nterms)
     ! do we want to reorder? Only if we have more than one file
     ! and then only reorder which are not being resampled.
     toorder(:) = 0
     if ( nfiles > 1) then
        do i = 1, ndf
           if (method(i) == 3 .and. distpts(i) .eq. 0) toorder(i) = 1
        end do
     end if
     ! Entries for which tooorder = 0 will get a sequence starting at 1.
     call createrandomlists(ndf,nbasis,toorder,order)

     write(*,*) "Created random lists"
     !Now calculate Ds rather than Cs for any modes which are being resampled
     call calcfileDs
     write(*,*) "Rediagonalised"
  end if

  ! ---- create basis fns ----
  do bf = 1,nbasis

     do dof = 1,ndf
        ! Choose which term of the expansion we should use
        if(method(dof) == 1) then

           ! Choose all from ground state for now.
           call pickharmrnd(zmax,zb(dof,bf),mode(dof),comp(dof),amps(dof))
           qb(dof,bf) =  sqrt(2/xi(dof))*real(zb(dof,bf))+q0(dof)
           pb(dof,bf) =  sqrt(2*xi(dof))*dimag(zb(dof,bf))+p0(dof)

        else if (method(dof) == 2) then
           if (nterms(dof) > 1) then
              pr = ZBQLU01(0.0)
              pl = ec(dof,1)**2
              term = 0
              pl = 0.0d0
              do while ((pr  > pl) .and. (term .lt.nterms(dof)))
                 term = term+1
                 pr = pr + ec(dof,term)**2
              end do
              ! The first term should be the ground state
              write(fug,*) "Choosing from term ",term-1
           else 
              write(fug,*) "Only one term in expansion"
              term = 1
           end if
           ! Choose form selected state
           call pickharmrnd(zmax,zb(dof,bf),term-1,comp(dof),amps(dof))

           qb(dof,bf) =  sqrt(2/xi(dof))*real(zb(dof,bf))+q0(dof)
           pb(dof,bf) =  sqrt(2*xi(dof))*dimag(zb(dof,bf))+p0(dof)
           ! scale overlap contribution
           if (nterms(dof) > 1) then
              amps(dof) = ec(dof,term)**2 * amps(dof)
           end if

        else if (method(dof) == 3) then
           ! points come from file
!           write(fug,*) "Order",(order(dof,bf))
!           write(fug,*) "qf,pf", qf(dof,(order(dof,bf))), pf(dof,(order(dof,bf)))
           if(distpts(dof) .ne. 1 ) then
              qb(dof,bf) = qf(dof,(order(dof,bf)))
              pb(dof,bf) = pf(dof,(order(dof,bf)))
              ! Calculate z for later.
              zb(dof,bf) = dsqrt(xi(dof)/2.0d0)*qb(dof,order(dof,bf)) &
                   +(0.0d0,1.0d0)* pf(dof,(order(dof,bf)))/dsqrt(2*xi(dof))
              ! assume an overlap contribution of 1.
              amps(dof) = CDfile(dof,bf) !1.0d0 
           else
              ! determine central point to use
              ! assume nbasis/nterms in an integer
              
              ! This bit doesn't quite work.
!              thispt = floor(dble(bf)/dble(nbasis/nterms(dof)))
              thispt = floor(dble(bf-mod(bf,nbasis/nterms(dof)))/dble(nbasis/nterms(dof)))+1
              if (thispt == nterms(dof)+1) thispt = 1

!              if ( (nbasis/nterms(dof))> 1 ) then
 !                thispt = thispt + 1
 !             end if
              write(*,*) "This point",thispt
              ! Now choose a z from the harmonic oscillator ground state
              !- i.e. a coherent states
              call pickharmrnd(zmax,zb(dof,bf),0,0.2d0,amps(dof))
              ! displace this z,
              zb(dof,bf) =  zb(dof,bf) + &
                   dsqrt(xi(dof)/2.0d0)*qf(dof,thispt) &
                   +(0.0d0,1.0d0)* pf(dof,thispt)/dsqrt(2*xi(dof))
              ! Now calculate amplitude
              amps(dof) = (0.0d0,0.0d0)
              do j = 1,nterms(dof)
                 zj = dsqrt(xi(dof)/2.0d0)*qb(dof,j) &
                   +(0.0d0,1.0d0)* pf(dof,j)/dsqrt(2*xi(dof))
                 amps(dof) = amps(dof) +  &
                      CDfile(dof,j)*cdexp(dconjg( zb(dof,bf))*zj - cdabs(zb(dof,bf))**2/2 -cdabs(zj)**2/2)
              end do
              
              ! Now need to calculate q and p
              qb(dof,bf) = real(zb(dof,bf))*sqrt(2.0d0/xi(dof))
              pb(dof,bf) = dimag(zb(dof,bf))*sqrt(2.0d0*xi(dof))
              write(112,*) qb(dof,bf),pb(dof,bf),real(amps(dof)),dimag(amps(dof))
           end if
        else
           write(fug,*) "Sorry, method ",method(dof)," is currently unsupported"
           stop
        end if

        ! Now need to store these
     end do
  
     ! Store amplitude
     C(bf) = (1.0d0,0.0d0)
     do dof = 1,ndf
        write(fug,*) amps(dof)
        C(bf) = C(bf)* amps(dof)
     end do
     write(fug,*) "Amplitude",C(bf)
  end do


end subroutine choosebasisfnnormalmodes


! ************************************************************************
!                        calcfileDs
! 
!  Calculate Ds rather than Cs for file modes which are being 
!  redistributed
! ************************************************************************
subroutine calcfileDs
  use arrays
  implicit none

  real(kind=8),dimension(:),allocatable :: S
  complex(kind=8),dimension(:,:),allocatable :: modez
  complex(kind=8),dimension(:),allocatable :: modeD,modeC
  real(kind=8),dimension(1) :: xi1
  integer :: i,j
  real(kind=8):: norm
  character(len=20)::modenum
  character(len=50):: filename

  ! interface to other routines
  interface
     include "reproject.inc"
  end interface

  allocate(modez(1,nbasis))
  
  allocate(S(nbasis))
  S(:) = 0.0d0 ! set the action 0 as this should be time=0.

  allocate(modeD(nbasis),modeC(nbasis))
  do i = 1,ndf
     if (method(i) == 3 .and. distpts(i) .eq. 1) then
        modez(:,:) = (0.0d0,0.0d0)
        modeD(:) = 0.0d0
        modeC(:) = 0.0d0

        do j = 1,nterms(i)
           modez(1,j) = dsqrt(xi(i)/2.0d0)*qf(i,j)+(0.0d0,1.0d0)*pf(i,j)/(dsqrt(2*xi(i)))
           ! write out zs.
        end do
        write(*,*) "C dof",i
        do j=1,nterms(i)
           write(*,*) CDfile(i,j)
        end do
        xi1(1) = xi(i)
         modeC(:) = CDfile(i,:)
!         CDfile(i,1:nterms(i))
        call calcD(1,nterms(i),modeC(1:nterms(i)),modez(:,1:nterms(i)),S(1:nterms(i)),xi1,modeD(1:nterms(i)))
        write(*,*) "D dof",i
        do j=1,nterms(i)
           write(*,*) modeD(j)
        end do
!        calculate norm
        norm = sum(dconjg(CDfile(i,1:nterms(i)))*modeD(1:nterms(i)))
        write(*,*) "Norm",norm,sum(dconjg(CDfile(i,1:nterms(i)))*modeD(1:nterms(i)))
        cdfile(i,:) = modeD(:)/dsqrt(norm)
        write(modenum,*) i
        write(filename,*) "wavefnmode",trim(adjustl(modenum))
        open(204,file=trim(adjustl(filename)))
        write(204,*) "#NormalisedD"
        do j=1,nterms(i)
           write(204,*) real(CDfile(i,j)),dimag(CDfile(i,j))
        end do
        close(204)
     end if
  end do
end subroutine calcfileDs
