!!% ========================================================================
!!%  Subroutine: choosebasisfntetra
!!%  Author: Stewart Reed
!!%  Date: 21st April 2009
!!%
!!%  This routine chooses basis functions for a tetra atomic molecule. It is
!!%  based on the routine for triatomic molecules.
!!% ========================================================================

subroutine choosebasisfntetra()
  use arrays
  use harmonicoscillator, ONLY: pickharmrnd,overlapzho
  use morsesampling

  implicit none

  complex(kind=8) :: amps(ndf)
  real(kind=8):: q(ndf,nbasis),p(ndf,nbasis)
  real(kind=8) :: xcom,ycom,zcom
  real(kind=8) :: A,B,zmax,magL2
  integer :: dof,i
  real(kind=8),dimension(3) :: L ! Angular momentum
  ! For choosing which term in the expansion to sample
  integer :: term
  real(kind=8) ::pr,pl
  real(kind=8),external :: zbqlu01 ! random number generator
  integer :: bf
  integer:: ntries
  real(kind=8) :: qpdp

  ! For the fourth atom
!  real(kind=8),dimension(3) :: R4isq
  real(kind=8) :: x21,y21,z21, x32,y32,z32, x13,y13,z13
  real(kind=8) :: quad,qxy,pxy,pzz
  logical :: tetrapos
  
  ! Variables for extra restrictions
  logical :: chosen ! Have we successfully chosen a basis function?
  complex(kind=8) :: totamp
  real(kind=8) :: prob ! Basis function probability = |totamp|^2
  real(kind=8) :: sep2 ! Phase space separation of basis functions
  integer :: j
  real(kind=8) :: cseng

  interface
     include 'readbasisfns.inc'
  end interface

  ! check that the system specifications are appropriate
  if ( nparticles.ne.4 ) then
     write(fug,*) "Error, system specification is wrong"
     write(fug,*) " Needs ndf=3, npartilces=4, ncartdim=2 and ndf=3"
     stop
  end if


  ! ---- Get basis fns from files if appropriate ----
  if (nfiles > 0) then
     call readbasisfns(ndf,nbasis,filenames,qf,pf,cswavefnfile,CDfile,nterms)
  end if

  open(unit=222,file="basisfunctions-morsemodes")
  zmax = 50d0

  do bf = 1,nbasis
     write(fug,*) "Choosing basis fn ",bf
     A = -1.0d0
     B = -1.0d0
     ntries = 0
     tetrapos = .false.
     do while ((A< 0.0d0 .or. A > 1.0d0 .or.  B < 0.0d0 ).and. ntries .le.  maxtries .and. .not. tetrapos) 
        ntries = ntries+1
        do dof = 1,nmodes
           write(fug,*) "Method ",dof,method(dof),mode(dof)
           ! Choose which term of the expansion we should use
           if(method(dof) == 1) then
              term = mode(dof)+1 

              ! pick position and momentum from the appropriate harmonic oscillator state
              call pickharmrnd(zmax,zb(dof,bf),term-1,comp(dof),amps(dof))

              q(dof,bf) =  sqrt(2/xi(dof))*real(zb(dof,bf))+q0(dof)
              p(dof,bf) =  sqrt(2*xi(dof))*dimag(zb(dof,bf))
           else if (method(dof) == 2) then
              if (nterms(dof) > 1) then
                 pr = ZBQLU01(0.0)
                 pl = ec(dof,1)**2
                 term = 0
                 pl = 0.0d0
                 do while ((pr  > pl) .and. (term .lt.nterms(dof)))
                    term = term+1
                    pr = pr + ec(dof,term)**2
                 end do
                 ! The first term should be the ground state
                 write(fug,*) "Choosing from term ",term-1
              else 
                 write(fug,*) "Only one term in expansion"
                 term = 1
              end if

              ! pick position and momentum from the appropriate harmonic oscillator state
              call pickharmrnd(zmax,zb(dof,bf),term-1,comp(dof),amps(dof))

              q(dof,bf) =  sqrt(2/xi(dof))*real(zb(dof,bf))+q0(dof)
              p(dof,bf) =  sqrt(2*xi(dof))*dimag(zb(dof,bf))
              if (nterms(dof) > 1) then
                 !             amps(dof) = ec(dof,term)**2 * amps(dof)
                 amps(dof) = (0.0d0,0.0d0)
                 do i = 1,nterms(dof)
                    amps(dof) =  amps(dof)+ec(dof,i)*overlapzho(zb(dof,bf),i)
                 end do
              end if
           else if (method(dof)==3) then 
              ! ---- q,p pairs from a file ----
              amps(dof) = (1.0d0,0.0d0)
              q(dof,bf) = 0.0d0
              p(dof,bf) = 0.0d0
              ! calculate the distance between the two particles in position and momentum
              do i = 1,ncartdim
                 q(dof,bf) = q(dof,bf) +  (qf((filep1-1)*ncartdim+i,bf) - qf((filep2-1)*ncartdim+i,bf))**2
                 p(dof,bf) = p(dof,bf) +  (pf((filep1-1)*ncartdim+i,bf) - pf((filep2-1)*ncartdim+i,bf))**2
              end do
              q(dof,bf) = sqrt(q(dof,bf)) ! This will be halved later
              p(dof,bf) = sqrt(p(dof,bf))/2.0d0 ! The momentum of each particle will be set to +/- p(dof) so divide by 2 here
              !Find sign of momentum
              qpdp = 0.0d0 ! Calculate the dot production between position and momentum vectors.
              do i = 1,ncartdim
                 qpdp =  qpdp+ &
                       (qf((filep1-1)*ncartdim+i,bf) - qf((filep2-1)*ncartdim+i,bf))&
                      *(pf((filep1-1)*ncartdim+i,bf) - pf((filep2-1)*ncartdim+i,bf))
              end do
              if (qpdp < 0) then
                 p(dof,bf) = -1.0d0 * p(dof,bf)
              end if

              ! Now randomise the displacement slightly if required
              if (distpts(dof) .eq. 1) then
                 call randomisez(q(dof,bf),p(dof,bf),xi(dof),comp(dof),amps(dof))
              end if
           else if (method(dof)==4) then 
              ! ---- Choose q,p from the morse wave function ----
              call pickmorse(mode(dof),q(dof,bf),p(dof,bf),amps(dof),ntries)
           else
              write(fug,*) "Sorry, method ",method(dof)," is currently unsupported"
              stop
           end if

        end do

        A = (q(1,bf)**2+q(2,bf)**2-q(3,bf)**2)/(2*q(1,bf)**2)
        B = q(2,bf)**2-A**2*q(1,bf)**2

        write(fug,*) "q",q(1:nmodes,bf)
        write(fug,*) "p",p(1:nmodes,bf)
        write(fug,*) "A ",A,"B",B

        ! Now calculate the position of the 4th atom but only if we have valid coordinates for the other 3 atoms
        if (A> 0.0d0 .and. A < 1.0d0 .and.  B > 0.0d0)then
           ! ---- First we calculate the positions of the first 3 atoms. ----
           ! set all z components to 0.
           z(:,bf) = 0.0d0
           
           x(1,bf) = -0.5d0*q(1,bf)
           y(1,bf) = 0.0d0
           
           x(2,bf) = 0.5d0*q(1,bf)
           y(2,bf) = 0.0d0
           

           x(3,bf) = x(1,bf) + A*q(1,bf)
           y(3,bf) = dsqrt(q(2,bf)**2-A**2*q(1,bf)**2)

           ! ---- Now calculate the position of the 4th ----
           ! R4isq = r4i^2-xi^2+yi^2+
!!$           do i = 1,3
!!$              R4isq(i) = q(3+i)**2 - (x(i,bf)**2+y(i,bf)**2+z(i,bf)**2)
!!$           end do
!!$           x21 = x(2,bf) - x(1,bf)
!!$           y21 = y(2,bf) - y(1,bf)
!!$           z21 = z(2,bf) - z(1,bf)
!!$
!!$           x32 = x(3,bf) - x(2,bf)
!!$           y32 = y(3,bf) - y(2,bf)
!!$           z32 = z(3,bf) - z(2,bf)
!!$
!!$           x13 = x(1,bf) - x(3,bf)
!!$           y13 = y(1,bf) - y(3,bf)
!!$           z13 = z(1,bf) - z(3,bf)

           x(4,bf) =  -0.5*(-x(1,bf)**2+x(2,bf)**2-q(5,bf)**2+q(4,bf)**2)/(x(1,bf)-x(2,bf))
           
          y(4,bf) = 0.5*(-x(3,bf)*x(1,bf)**2+x(3,bf)*x(2,bf)**2-x(3,bf)*q(5,bf)**2+x(3,bf)*q(1,bf)**2 &
               +x(3,bf)**2*x(1,bf)-x(3,bf)**2*x(2,bf)+y(3,bf)**2*x(1,bf)-y(3,bf)**2*x(2,bf)&
               +x(2,bf)*x(1,bf)**2-x(2,bf)*q(4,bf)**2-x(2,bf)**2*x(1,bf)+q(5,bf)**2*x(1,bf) &
               -q(6,bf)**2*x(1,bf)+x(2,bf)*q(6,bf)**2)/(y(3,bf)*(x(1,bf)-x(2,bf)))

          quad = -x(4,bf)**2+2*x(4,bf)*x(1,bf)-x(1,bf)**2-y(4,bf)**2+q(4,bf)**2
          write(fug,*) "quad", quad,(-x(4,bf)**2+2*x(4,bf)*x(2,bf)-x(2,bf)**2-y(4,bf)**2+q(5,bf)**2)
          if (quad > 0) then
             z(4,bf) = sqrt(quad)

!!$
!!$          denom = y13*x32*z21+x13*y21*z32+x21*y32*z13-y32*x13*z21-x32*y21*z13-x21*y13*z32
!!$          if (abs(denom) .gt. 1e-7) then
!!$              x(4,bf) = 2*(-R4isq(2)*y32*z13+R4isq(2)*y13*z32-y21*z32*R4isq(1)+y21*z32*R4isq(3) &
!!$                   +y21*R4isq(3)*z13-y21*R4isq(2)*z13+z21*y13*R4isq(2)+z21*y32*R4isq(1) &
!!$                   -z21*y13*R4isq(3)+R4isq(1)*y32*z13-R4isq(1)*y13*z32-R4isq(3)*y32*z21)/denom
!!$
!!$              y(4,bf) = -2*(-x21*z32*R4isq(1)+x21*z32*R4isq(3)+x21*R4isq(3)*z13-x21*R4isq(2)*z13 &
!!$                   -x32*z21*R4isq(3)+x32*z21*R4isq(1)-z32*x13*R4isq(1)+R4isq(2)*x13*z21 &
!!$                   +z32*x13*R4isq(2)+x32*R4isq(1)*z13-x32*R4isq(2)*z13-R4isq(3)*x13*z21)/denom
!!$
!!$             z(4,bf) =  2*(-x32*y21*R4isq(3)-x21*y13*R4isq(2)+x21*y32*R4isq(3)-x21*y32*R4isq(1) &
!!$                  +x21*y13*R4isq(3)+y32*x13*R4isq(2)+x32*y21*R4isq(1)+y13*x32*R4isq(1) &
!!$                  +x13*y21*R4isq(2)-x13*y21*R4isq(3)-y32*x13*R4isq(1)-y13*x32*R4isq(2))/denom



             tetrapos =.true.
          else
             tetrapos = .false.
          end if !  (abs(denom) .gt. 1e-7)


          ! ==== Apply some extra restrictions ====
          ! Test to see if the amplitude is sufficiently large. 
           ! Setting minbfprob to zero should essentially turn off this check.
          if (tetrapos) then
             !assume that we will keep it and then other tests can simply reject it
             ! This makes adding extra tests easier.
             chosen = .true.

             totamp = PRODUCT(amps( 1:nmodes))
             prob = dble(totamp*conjg(totamp)) ! Should be real anyway but need to do the type conversion
             if (prob < minbfprob) then
                chosen = .false.
                write(*,*) "Probability condition not met", prob ," < ", minbfprob
             end if
             ! Now test separation of basis functions, check to make sure it hasn't already been rejected though.
             if (chosen .and. bfsepmode .ge. 0)then
                ! Calculate separation in one mode only.
                if (bfsepmode > 0 ) then
                   j = 1
                   do while (chosen .and. j < bf)
                      sep2 = 0.5*( xi(bfsepmode) * (q(bfsepmode,bf)-q(bfsepmode,j)) **2 &
                           + (p(bfsepmode,bf)-p(bfsepmode,j))**2/xi(bfsepmode))
                      if (sep2 < bfsep2) then
                         chosen = .false.
                         write(*,*) "Dropped basis fn as too close to fn ",j," mode ",bfsepmode, " sep ",sqrt(sep2)
                      else
                         write(*,*) "Separation",sqrt(sep2)
                      end if
                      j = j + 1
                   end do
                else ! Calculate total phase space separation
                   j = 0
                   do while (chosen .and. j < bf)
                      sep2 = sum(0.5*( xi(1:nmodes) * (q(1:nmodes,bf)-q(1:nmodes,j)) **2 &
                           + (p(1:nmodes,bf)-p(1:nmodes,j))/xi(1:nmodes)))
                      if (sep2 < bfsep2) then
                         chosen = .false.
                      end if
                      j = j + 1
                   end do
                end if
             end if

             if (chosen .and. bfengrange > 0) then
                cseng = 0.0d0
                do dof = 1,nmodes
                   cseng = cseng + morseenergy(mode(dof),q(dof,bf),p(dof,bf))
                end do
                if (cseng < minbfeng .or. cseng > maxbfeng) then
                   chosen = .false.
                   write(*,*) "Dropped basis fn as wrong energy ",cseng
                end if
             end if
          end if  ! ==== end of extra restrictions ====
          
       end if
    end do
     if (ntries == maxtries) then
        write(fug,*) "Reached maximum number of tries to choose basis fn number",bf
        write(fug,*) "Giving up"
        stop
     end if
     write(fug,*) "A",A,"B",B
     ! set all z components to 0.
     write(fug,*) "qs",(q(i,bf),i=1,3)
     !check 
     write(*,*) x(3,bf),y(3,bf),dsqrt(q(3,bf)**2-(1-A)**2*q(1,bf)**2)

     ! Calculate centre of mass
     xcom= sum(x(:,bf)*mass(:))/totalmass
     ycom= sum(y(:,bf)*mass(:))/totalmass
     zcom= sum(z(:,bf)*mass(:))/totalmass

     ! move centre of mass to the origin
     x(:,bf) = x(:,bf) - xcom
     y(:,bf) = y(:,bf) - ycom
     z(:,bf) = z(:,bf) - zcom

     write(fug,*) "Positions"
     do i = 1,nparticles
        write(fug,*) i,x(i,bf),y(i,bf),z(i,bf)
     end do
     write(222,"(i5,12f18.8)") bf, q(1,bf),p(1,bf),q(2,bf),p(2,bf),q(3,bf),p(3,bf),q(4,bf),p(4,bf), q(5,bf),p(5,bf),q(6,bf),p(6,bf)


     ! Store amplitude
     C(bf) = (1.0d0,0.0d0)
     do dof = 1,nmodes
        !      write(fug,*) amps(dof)
        C(bf) = C(bf)* amps(dof)
     end do
     write(fug,*) "Amplitude",C(bf)

     ! ********************************
     ! Now we need to calculate the momentum
     px(:,bf) = 0.0d0
     py(:,bf) = 0.0d0
     pz(:,bf) = 0.0d0

     ! Due to first mode
     px(1,bf) = -p(1,bf)  ! other components are zero
     px(2,bf) =  p(1,bf)  ! other components are zero
     ! Due to second mode
     px(1,bf) = px(1,bf) +((x(3,bf)-x(1,bf))/q(2,bf))*p(2,bf)
     py(1,bf) = py(1,bf) +((y(3,bf)-y(1,bf))/q(2,bf))*p(2,bf)
     px(3,bf) = px(3,bf) -((x(3,bf)-x(1,bf))/q(2,bf))*p(2,bf)
     py(3,bf) = py(3,bf) -((y(3,bf)-y(1,bf))/q(2,bf))*p(2,bf)

     ! Due to third mode
     px(2,bf) = px(2,bf) -((x(3,bf)-x(2,bf))/q(3,bf))*p(3,bf)
     py(2,bf) = py(2,bf) -((y(3,bf)-y(2,bf))/q(3,bf))*p(3,bf)
     px(3,bf) = px(3,bf) +((x(3,bf)-x(2,bf))/q(3,bf))*p(3,bf)
     py(3,bf) = py(3,bf) +((y(3,bf)-y(2,bf))/q(3,bf))*p(3,bf)

     ! Due to fourth, fith and sixth modes
     do i = 1,3
        qxy = (q(3+i,bf)**2-(z(4,bf)-z(i,bf))**2)
        if (qxy > 0) then
           qxy = sqrt(qxy)
        else
           write(fug,*) "***** qxy^2 is negative ****"
        end if
        pzz = (z(4,bf)-z(i,bf))/q(3+i,bf) *p(3+i,bf)
        pxy = (p(3+i,bf)**2-pzz**2)
        if (pxy > 0) then
           if (p(3+i,bf) > 0) then
              pxy = sqrt(pxy)
           else
              pxy = -sqrt(pxy)
           end if
        else
           write(fug,*) "***** pxy^2 is negative ****"
        end if
        px(i,bf) = px(i,bf) -((x(4,bf)-x(i,bf))/qxy)*pxy
        py(i,bf) = py(i,bf) -((y(4,bf)-y(i,bf))/qxy)*pxy
        pz(i,bf) = pz(i,bf) -((z(4,bf)-z(i,bf))/q(3+i,bf))*p(3+i,bf) 
        px(4,bf) = px(4,bf) +((x(4,bf)-x(i,bf))/qxy)*pxy
        py(4,bf) = py(4,bf) +((y(4,bf)-y(i,bf))/qxy)*pxy
        pz(4,bf) = pz(4,bf) +((z(4,bf)-z(i,bf))/q(3+i,bf))*p(3+i,bf)
     end do
     

     write(fug,*) "Momenta"
     do i = 1,nparticles
        write(fug,*) i,px(i,bf),py(i,bf),pz(i,bf)
     end do

     write(fug,*) "Total Momentum"
     write(fug,*) sum(px(:,bf)),sum(py(:,bf)),sum(pz(:,bf))

     write(fug,*) "Angular momentum"
     L(:)=0.0d0
     do i = 1,nparticles
        L(1) =  L(1) + y(i,bf)*pz(i,bf) - z(i,bf)*py(i,bf)
        L(2) =  L(2) + z(i,bf)*px(i,bf) - x(i,bf)*pz(i,bf)
        L(3) =  L(3) + x(i,bf)*py(i,bf) - y(i,bf)*px(i,bf)
     end do
     write(fug,*) L(1),L(2),L(3)

  end do
  close(222)
end subroutine choosebasisfntetra



