! ************************************************************************
!                        createrandomlists
!                        12 December 2008
!                          Stewart Reed
!
!  This routing is essentially the same as used in the recombine basis
!  program except it is generalised to create a random orders for several 
!  sets of data.
!  
!   inputs n             - number of items to reorder
!          ndf           - number of degrees of freedom each of which 
!                          have n items
!          toorder(ndf)  - integer array saying which degrees of freedom 
!                          should be reordered
!   outputs order(ndf,n) - reordered lists. Those degrees of freedom 
!                          which are not reordered with contain the 
!                          sequential order starting at 1.
!                          i.e. 
!                          order(df,item) = rand (1,n) if toorder(ndf) > 0
!                                         = item if toorder(ndf) = 0
! ************************************************************************

subroutine createrandomlists(ndf,n,toorder,order)
  implicit none

  integer,intent(in) :: ndf,n
  integer,intent(in),dimension(ndf) :: toorder
  integer,dimension(ndf,n),intent(out) :: order

  integer :: i,j,df,rc,choice
  real(kind=8) :: r ! random number

  ! Logical array to keep track of which items have been chosen.
  logical,dimension(n) :: chosen

  ! ---- Loop over degrees of freedom ----
  do df = 1,ndf
     ! Test to see if want to reorder this degree of freedom
     if (toorder(df) .eq. 0 )  then
        ! ---- no reordering ----
        do i = 1,n
           order(df,i) = i
        end do
     else
        ! ---- reordering ----
        chosen(:) = .false.
        do i = 1,n
           call random_number(r)
           r = r*dble(n+1-i)

           rc = floor(r)+1


           choice = 0
           j = 0
           do while(choice .lt. rc .and. j .lt. n)
              j = j+1
              if (.not. chosen(j)) choice = choice + 1
           end do

           order(df,i) = j
           chosen(j) = .true.
           !debug         write(*,*) n+1-i, rc,choice
        end do

        !debug     write(*,*) order(:)
     end if
  end do
end subroutine createrandomlists

