subroutine reprojectwavefunction
  use arrays
  implicit none
  complex(kind=8),dimension(:),allocatable :: newC,D
!  complex(kind=8),dimension(:,:),allocatable ::zi ! Use zb in arrays
  complex(kind=8),dimension(:,:),allocatable ::zbf
  real(kind=8),dimension(:),allocatable :: S ! action dim(nbasis)
  integer :: bf,i
  real(kind=8) :: norm
  interface
     function overlapg1g2(ndf,zi,zj,gammas1,gammas2)
       ! ---- interface ----
       integer,intent(in) :: ndf
       complex(kind=8),dimension(ndf), intent(in) :: zi,zj
       real(kind=8),dimension(ndf) :: gammas1,gammas2
       complex(kind=8) :: overlapg1g2
     end function overlapg1g2

     subroutine calcD(ndf,nbasis,C,zb,S,gammas,D)
       ! ---- interface ---- 
       integer, intent(in) :: ndf,nbasis
       complex(kind=8),dimension(nbasis),intent(in) :: C
       real(kind=8),dimension(nbasis),intent(in) :: S ! action
       complex(kind=8),dimension(ndf,nbasis),intent(in) :: zb ! basis states
       real(kind=8),dimension(ndf),intent(in) :: gammas
       complex(kind=8),dimension(nbasis),intent(out) :: D
     end subroutine calcD

  end interface


  ! keep same positions but the widths of new basis functions are gamma rather than xi
  if (.not.allocated(gamma)) then
     write(fug,*) "You haven't specified the gammas for the basis functions"
     write(fug,*) "Not reprojecting"
     return
  end if

  allocate(newC(nbasis),D(nbasis))
  allocate(zbf(ndf,nbasis))
!  allocate(zi(ndf,nbasis))

  !---- calculate zis ----
  do i = 1,nbasis
     ! Should already be defined.
 !    zi(:,i) =  dsqrt(xi(:)/2.0d0)*qb(:,i)+(0.0d0,1.0d0)*pb(:,i)/(dsqrt(2*xi(:)))

     zbf(:,i) = dsqrt(gamma(:)/2.0d0)*qb(:,i)+(0.0d0,1.0d0)*pb(:,i)/(dsqrt(2*gamma(:)))
  end do

  ! ---- calculate Di ----
  allocate(S(nbasis))
  S(:) = 0.0d0 ! set the action 0 as this should be time=0.
  call calcD(ndf,nbasis,C,zb,S,xi,D)
  ! write out amplitudes
  open (unit=famps,file="Dinitial")
  do i = 1,nbasis
     write(famps,*) i,real(D(i)),dimag(D(i))
  end do
  close(famps)
  open (unit=famps,file="Cinitial")
  do i = 1,nbasis
     write(famps,*) i,real(C(i)),dimag(C(i))
  end do
  close(famps)

  ! ---- calculate norm ----
  norm =  sum(dconjg(C(:))*D(:))
  write(fug,*) "norm",norm

  D(:) = D(:)/dsqrt(norm)
  ! ---- now calculate the new Cs  ----
  ! Cj = sum_i <zj|zi'>Di
  do bf = 1,nbasis
!     zbf(:) = dsqrt(gamma(:)/2.0d0)*qb(:,bf)+(0.0d0,1.0d0)*pb(:,bf)/(dsqrt(2*gamma(:)))
!     write(fug,*) "zbf",zbf
     newC(bf) = (0.0d0,0.00)
     do i = 1,nbasis
!        zi(:) = dsqrt(xi(:)/2.0d0)*qb(:,i)+(0.0d0,1.0d0)*pb(:,i)/(dsqrt(2*xi(:)))
!        write(*,*) "zi",zi
!        newC(bf) = newC(bf) + C(i) *  overlapg1g2(ndf,zbf,zi(:,i),gamma,xi)
        newC(bf) = newC(bf) + D(i) *  overlapg1g2(ndf,zbf(:,bf),zb(:,i),gamma,xi)
     end do
  end do

  ! copy newC to old C
  C(:) = newC(:)
  deallocate(newC)
  ! Copy new z to old z
  zb(:,:) = zbf(:,:)

  ! ---- calculate new norm ----
  call calcD(ndf,nbasis,C,zbf,S,xi,D)
  norm =  sum(dconjg(C(:))*D(:))
  C(:)=C(:)/dsqrt(norm)
  write(fug,*) "new norm",norm

  ! ---- write out new C ----
  open (unit=famps,file="Creprojected")
  do i = 1,nbasis
     write(famps,*) i,real(C(i)),dimag(C(i))
  end do
  close(famps)
  
end subroutine reprojectwavefunction

!!% ========================================================================
!!%  Subroutine: calcnorm
!!%  Description: A wee subroutine to calculate the norm of the wave 
!!%  wave function and check the calculated Ds by recalculating C
!!% ========================================================================
subroutine calcnorm
  use arrays
  implicit none
  complex(kind=8),dimension(:),allocatable :: newC,D
  real(kind=8),dimension(:),allocatable :: S ! action dim(nbasis)
  integer :: bf,i
  real(kind=8) :: norm
  interface
     subroutine calcD(ndf,nbasis,C,zb,S,gammas,D)
       ! ---- interface ---- 
       integer, intent(in) :: ndf,nbasis
       complex(kind=8),dimension(nbasis),intent(in) :: C
       real(kind=8),dimension(nbasis),intent(in) :: S ! action
       complex(kind=8),dimension(ndf,nbasis),intent(in) :: zb ! basis states
       real(kind=8),dimension(ndf),intent(in) :: gammas
       complex(kind=8),dimension(nbasis),intent(out) :: D
     end subroutine calcD

  end interface
  

  ! ---- calculate Di ----
  allocate(S(nbasis))
  S(:) = 0.0d0 ! set the action 0 as this should be time=0.
  allocate(D(nbasis))
  allocate(newC(nbasis))

  call calcD(ndf,nbasis,C,zb,S,xi,D)
  ! write out amplitudes
  open (unit=famps,file="Dinitial")
  do i = 1,nbasis
     write(famps,*) i,real(D(i)),dimag(D(i))
  end do
  close(famps)
  open (unit=famps,file="Cs")
  write(famps,*) "bf Re(Cinit) Im(Cinit) Re(Cnew)  Im(Cnew) "
  do i = 1,nbasis
     write(famps,*) i,real(C(i)),dimag(C(i)),real(newC(i)),dimag(newC(i))
  end do
  close(famps)

  ! ---- calculate norm ----
  norm =  sum(dconjg(C(:))*D(:))
  write(fug,*) "norm",norm

  ! ---- Check D by calculate C again ----
  

end subroutine calcnorm
! ==================================================
!              Overlap
! ================================================== 
! Calculate the overlap of two coherent states zi and zj
! The overlap matrix is given be
! <zi|zj>=int(<zi|x><x|zj> dx = exp(zi^* zj - |zi|^2/2 - |zj|^2/2)
! where |zi> is given by the product of the states in each degree
! of freedom.
function overlap(ndf,zi,zj,gammas)
  implicit none
  integer,intent(in) :: ndf
  complex(kind=8),dimension(ndf), intent(in) :: zi,zj
  real(kind=8),dimension(ndf) :: gammas
  complex(kind=8) :: overlap

  integer :: df  ! degree of freedom for loop
  complex(kind=8) :: exponent

  exponent = (0.0d0,0.0d0)
  do df = 1,ndf
     exponent = exponent + dconjg(zi(df))*zj(df) - cdabs(zi(df))**2/2 -cdabs(zj(df))**2/2
  end do
  overlap = cdexp(exponent)
end function overlap

function overlapg1g2(ndf,zi,zj,gammas1,gammas2)
  implicit none
  ! ---- interface ----
  integer,intent(in) :: ndf
  complex(kind=8),dimension(ndf), intent(in) :: zi,zj
  real(kind=8),dimension(ndf) :: gammas1,gammas2
  complex(kind=8) :: overlapg1g2
  
  ! ---- internal ----
  integer :: df  ! degree of freedom for loop
  complex(kind=8) :: exponent
    
    exponent = (0.0d0,0.0d0)
    do df = 1,ndf
       !       overlap = overlap * &
       !            cdexp(dconjg(zi(df))*zj(df) &
       !            - cdabs(zi(df))**2/2 -cdabs(zj(df))**2/2)
       !       exponent = exponent + dconjg(zi(df))*zj(df) - cdabs(zi(df))**2/2 -cdabs(zj(df))**2/2
       
       exponent = exponent -0.5/(gammas1(df)+gammas2(df))*(&
            (gammas1(df)+gammas2(df))*dconjg(zi(df))*zi(df) &
            +(gammas1(df)+gammas2(df))*dconjg(zj(df))*zj(df) &
            - 4*dsqrt(gammas1(df)*gammas2(df))*dconjg(zj(df))*zj(df) &
            +(gammas2(df)-gammas1(df))*(dconjg(zi(df)))**2 &
            +(gammas1(df)-gammas2(df))*(zj(df))**2)
    end do
    overlapg1g2 = cdexp(exponent)
    
  end function overlapg1g2
  
  
  ! ===============================================
  !  serial lapack version of calcD.
  ! ===============================================
  subroutine calcD(ndf,nbasis,C,zb,S,gammas,D)
    use arrays, ONLY: fug,omega0
    implicit none
    ! ---- interface ---- 
    integer, intent(in) :: ndf,nbasis
    complex(kind=8),dimension(nbasis),intent(inout) :: C
    real(kind=8),dimension(nbasis),intent(in) :: S ! action
    complex(kind=8),dimension(ndf,nbasis),intent(in) :: zb ! basis states
    real(kind=8),dimension(ndf),intent(in)::gammas
    complex(kind=8),dimension(nbasis),intent(out) :: D
    
    ! ---- internal ----
    !Variables for LAPACK routines + arrays from arrays module
    integer:: info
    real(kind=8) :: rcond
    
    integer :: i,j ! for loops
    integer :: verbosity = 1
    
    ! Working arrays for calculating D from C
    ! ---------------------------------------
    ! Overlap Matrix
    complex(kind=8),allocatable,dimension(:,:):: overlpm
    ! nbasis
    real(kind=8),dimension(:),allocatable :: FERR,BERR,RWORK
    integer,dimension(:),allocatable :: ipiv
    ! nbasis x nbasis
    complex(kind=8),dimension(:,:),allocatable::overlpmf
    
    ! 2*nbasis
    complex(kind=8),dimension(:),allocatable:: WORK
    integer :: LWORK ! = 2*nbasis = size(WORK)
    integer :: LRWORK ! size of RWORK
    
    ! --- interface for overlap function
    interface
       function overlap(ndf,zi,zj,gammas)
         integer,intent(in) :: ndf
         complex(kind=8),dimension(ndf), intent(in) :: zi,zj
         real(kind=8),dimension(ndf) :: gammas
         complex(kind=8) :: overlap
       end function overlap

       SUBROUTINE ZHECON( UPLO, N, A, LDA, IPIV, ANORM, RCOND, WORK, INFO )
         CHARACTER          UPLO
         INTEGER            INFO, LDA, N
         DOUBLE PRECISION,intent(in)::   ANORM
         DOUBLE PRECISION,intent(out)::RCOND
         INTEGER,dimension(N) ::IPIV
         complex(kind=8),dimension(2*N)  ::WORK
       END SUBROUTINE ZHECON
    end interface

    ! allocate arrays
    LWORK = 2*nbasis
    LRWORK = nbasis
    allocate(FERR(nbasis),BERR(nbasis),RWORK(LRWORK))
    allocate(WORK(LWORK))
    allocate(ipiv(nbasis))
    allocate(overlpm(nbasis,nbasis))
    allocate(overlpmf(nbasis,nbasis))
        
    ! Calculate overlap matrix*exp(i(Sj-Si))
    overlpm(:,:) = (0.0,0.0)
    do i = 1,nbasis
       overlpm(i,i) = (1.0d0,0.0d0)
    end do

    write(202,*)"#C"
    
!    c(1) = 55.0d0
!    c(2) = 107.0d0
!    c(3) = 157.0d0
!    c(4) = 205.0d0
!    c(5) = 251.0d0
!    c(6) = 295.0d0
!    c(7) = 337.0d0
!    c(8) = 377.0d0
!    c(9) = 415.0d0
!    c(10) = 451.0d0
    D(:) = 0.0d0
    do i = 1,nbasis
!       C(i) = 0.0001
       write(202,*) real(C(i)),dimag(C(i))
    end do
    do i = 1,nbasis-1
       do j = i+1,nbasis
          !          write(61,*) overlap(zb(:,i),zb(:,j)),(S(j)-S(i)),cdexp((0.0,1.0)*(S(j)-S(i))),&
          !               overlap(zb(:,i),zb(:,j))*cdexp((0.0,1.0)*(S(j)-S(i)))

          if (abs(overlap(ndf,zb(:,i),zb(:,j),gammas)) .ge. omega0) then
             overlpm(i,j) = overlap(ndf,zb(:,i),zb(:,j),gammas)*cdexp((0.0,1.0)*(S(j)-S(i)))
             overlpm(j,i) = dconjg(overlpm(i,j))
          else
             overlpm(i,j) = 0.0d0
             overlpm(j,i) = 0.0
             !             write(61,*) "Not including",i,j
          end if
          !!         overlpm(i,j) = i+(0.0d0,1.0d0)*j
          !!        overlpm(j,i) = dconjg(overlpm(i,j))
          write(200,*) i,j,real(overlpm(i,j)),dimag(overlpm(i,j))
       end do
    end do

    ! Call solver for Hermitian system
    call zhesvx('N','L',nbasis, 1, overlpm, nbasis, overlpmf, nbasis, &
         IPIV, C, nbasis, D, nbasis, RCOND, FERR, BERR, WORK, LWORK, &
         RWORK, INFO )
    ! Call solver for general complex system
!    call zgesvx('N','N',nbasis, 1, overlpm, nbasis, overlpmf, nbasis, &
!         IPIV,'N',GenR,GenC,C, nbasis, D, nbasis, RCOND, FERR, BERR, WORK, &
!         RWORK, INFO )
    
    
    write(203,*)"#D calculated",INFO
    do i = 1,nbasis
       write(203,*) real(D(i)),dimag(D(i))
    end do

    ! overlap matrix is now destroyed!
    if(INFO .ne. 0) then
       if (verbosity .gt. 1) then
          write(fug,*)
          write(fug,*) "overlap matrix written out"
          open(81,file = "overlapmatrix")
          write(81,*) "# i,j,Re(overlap),Im(overlap)"
          do i = 1,nbasis
             do j = 1,nbasis
                write(81,*) i,j,real(overlpm(j,i)),dimag(overlpm(j,i))
             end do
             write(81,*) 
          end do
          close(81)

          open(81,file = "Dfailed")
          write(81,*)
          do i = 1,nbasis
             write(81,*) D(i)
          end do
          close(81)
       end if

       if (INFO .lt. 0) then
          write(fug,*)  "Error solving omega.exp(i(Sj-Si))D = C."
          write(fug,*) "Element ",-INFO," has an illegal value."
          stop
       else if (INFO .ge. 1 .and. INFO.le.nbasis) then
          write(fug,*)  "Error solving omega.exp(i(Sj-Si))D = C."
          write(fug,*) "D(",INFO,",",INFO,") = 0 so matrix is singular."
          stop
       else if (INFO .eq. nbasis+1) then
          write(fug,*)  "Problem solving omega.exp(i(Sj-Si))D = C."
          write(fug,*) " RCOND is less than machine precision.",RCOND
          write(fug,*) " RCOND = 0 indicates that the matrix is singular"
          stop
       else
          write(fug,*) "Error solving omega.exp(i(Sj-Si))D = C. Error number is",info
          stop
       end if

    end if
  end subroutine calcD
