!!% ========================================================================
!!%  File: utilities
!!%
!!%  This file containes several small subroutines
!!% Namely:
!!%       randomisez
!!%       zfromqp
!!%       qpfromz
!!% ========================================================================


!!% ************************************************************************
!!% Subroutine: randomisez
!!%
!!% Subroutine to take in a complex number z and displace it by an amount
!!% chosen at random from a gaussian distribution.
!!%
!!% ************************************************************************
subroutine randomisez(q,p,gamma,comp,amp)
  use harmonicoscillator, ONLY: pickharmrnd
  implicit none
  ! ---- interface ----
  real(kind=8),intent(inout)  :: q,p
  real(kind=8), intent(in) :: gamma
  real(kind=8), intent(in) :: comp
  complex(kind=8), intent(out) :: amp
  ! ---- local variables ----
  real(kind=8) :: zmax = 50.0d0
  complex(kind=8) :: zdisp    ! displacement
  real(kind=8) :: qdisp,pdisp

  ! Pick displacement from a gaussian distribution
  call pickharmrnd(zmax,zdisp,0,comp,amp)
  
  ! convert the displacement z to q,p
  call qpfromz(zdisp,gamma,qdisp,pdisp)

  ! displace q and p
  q = q+qdisp
  p = p+pdisp

end subroutine randomisez

!!% ************************************************************************
!!% Subroutine: zfromqp
!!% 
!!% A wee subroutine to calculate z from a q,p pair with a given gamma
!!% ************************************************************************
subroutine zfromqp ( q, p, gamma, z)
  implicit none
  ! ---- interface ----
  real(kind=8),intent(in) :: q,p,gamma
  complex(kind=8), intent(out) :: z

  z = complex(sqrt(gamma/2.0d0)*q,p/sqrt(2.0d0*gamma))

end subroutine zfromqp

!!% ************************************************************************
!!% Subroutine: qpfromz
!!% 
!!% A wee subroutine to calculate a q,p pair from z with a given gamma
!!% ************************************************************************
subroutine qpfromz(z,gamma,q,p)
  implicit none
  ! ---- interface ----
  complex(kind=8), intent(in) :: z
  real(kind=8),intent(in) :: gamma
  real(kind=8),intent(out) :: q,p
  
  q = sqrt(2.0d0/gamma)*real(z)
  p = sqrt(2.0d0*gamma)*dimag(z)

end subroutine qpfromz
