! ************************************************************************
!                           writebasis
!                        12 December 2008
!                          Stewart Reed
!
!  Subroutines to write basis fns to file.
!  There are two ways of doing this depending on system type.
!  I.e. normal modes or particulate.
!  Requires no options, only data from arrays.
! ************************************************************************

subroutine writebasis
  use arrays, ONLY: nparticles,ncartdim
  implicit none
  
  if (nparticles > 0) then
     call writebasisparticles
  else
     call writebasisnormalmodes
  end if
end subroutine writebasis
  

subroutine writebasisparticles
  use arrays
  implicit none

  integer :: bf,i,j
  character(len=100) :: fmt
  real(kind=8),dimension(:),allocatable,save :: vals

  if(.not.allocated(vals)) allocate(vals(2*ndf))

  open(unit=fbasis,file="basis")
  write(fbasis,*) "nbasis",nbasis
  write(fbasis,*) "amplitudes" !"unitamp"
  
  open(unit=fbamp,file="basisamplitudes")
  write(fbamp,*) "# Initial basis set"
  write(fbamp,*) "# Step 0"
  write(fmt,"('(',i3,'es18.8)')")2*ndf+3
  do bf = 1,nbasis
     write(fbasis,*) "basis"
     write(fbasis,*) "amp",real(C(bf)),dimag(C(bf))
     do i = 1,nparticles
        write(fbasis,*) x(i,bf),px(i,bf)
        vals(2*ncartdim*(i-1)+1) =  x(i,bf)
        vals(2*ncartdim*(i-1)+2) = px(i,bf)
        
        if (ncartdim .ge.2 ) then
           write(fbasis,*) y(i,bf),py(i,bf)
           vals(2*ncartdim*(i-1)+3) =  y(i,bf)
           vals(2*ncartdim*(i-1)+4) = py(i,bf)
           if (ncartdim .ge.3 ) then
              write(fbasis,*) z(i,bf),pz(i,bf)
              vals(2*ncartdim*(i-1)+5) =  z(i,bf)
              vals(2*ncartdim*(i-1)+6) = pz(i,bf)
           end if
        end if
     end do
     write(fbamp,fmt) 0.0,real(C(bf)),dimag(C(bf)),(vals(j),j=1,2*ndf)
  end do
  write(fbamp,*) 

  close(fbasis)
  close(fbamp)
end subroutine writebasisparticles



subroutine writebasisnormalmodes
  use arrays
  implicit none

  integer :: bf,i

  open(unit=fbasis,file="basis")
  write(fbasis,*) "nbasis",nbasis
  write(fbasis,*) "amplitudes"

  open(unit=fqp,file="qp")
  write(fqp,*) "#nbasis ",nbasis
  do bf = 1,nbasis
     write(fbasis,*) "basis"
     write(fbasis,*) "amp",real(C(bf)),dimag(C(bf))
     do i = 1,ndf
        if (displace) then
           write(fbasis,*) qb(i,bf)-qb0(i),pb(i,bf)-pb0(i)
           write(fqp,*) qb(i,bf)-qb0(i),pb(i,bf)-pb0(i)
        else
           write(fbasis,*) qb(i,bf),pb(i,bf)
           write(fqp,*) qb(i,bf),pb(i,bf)
        end if
     end do
  end do
  close(fbasis)
  close(fqp)
end subroutine writebasisnormalmodes
