program twostagedecay
  use basismodule
  use errors

  implicit none

  integer,dimension(:,:),allocatable :: Nediss     !have the Neons dissociated? save config at dissociation.
  integer,dimension(:),allocatable :: onediss      ! which dissociated first?
  integer,dimension(:),allocatable :: ndiss        ! number of neons that have dissociated for each bf.
  integer,dimension(:,:),allocatable :: dissorder  ! order of dissociation
  real(kind=8),dimension(:),allocatable :: probdis ! dissociation probability

  integer :: nneons,nbromine
  character(len=100) :: ampfile
  character(len=100),dimension(:),allocatable :: ampfiles
  integer :: nfiles
  integer :: ierr
  integer :: nbasis
  integer :: ndf
  integer :: ncartdim
  integer :: nparticles
  integer :: readconfigerr
  real(kind=8) :: gammaBr,gammaNe ! gammas

  integer,dimension(:),allocatable :: fdiss ! file unit for dissociation files

  integer :: famp = 10

  real(kind=8) :: rcut,rcut2
  real(kind=8) :: prob

  ! Variables to control partial trajectory
  real(kind=8) :: t0  ! Initial time offset
  logical      :: pad ! Pad out the dissociation rates for t < t0.
  real(kind=8) :: dt     ! time step to use
  integer      :: nsteps ! number of padding steps to write out.

  ! Weighting for simulation
  real(kind=8) ::simweight


  integer :: bf,i,j,file ! for loops
  integer :: fug=99
  integer :: fopt=11
  integer :: fdconfig = 12
  integer :: nconfigs
  logical :: firstdissevent ! does this config have the first dissociation event for this bf?
  logical :: writedissconfig ! write out configs when a neon dissociates?


  ! for reading in/output
  character(len=100) :: line
  character(len=50)  :: keyword
  interface
     function R2(bs,bf,ncartdim,Ne)
       use basismodule
       implicit none
       ! ---- interface ----
       type(basisset),intent(in) :: bs
       integer, intent(in)       :: bf ! which basis function
       integer, intent(in)       :: ncartdim
       integer, intent(in)       :: ne ! which neon
       real(kind=8) :: R2
     end function R2

     function sep2(bs,bf,ncartdim,i,j)
       use basismodule
       implicit none
       ! ---- interface ----
       type(basisset),intent(in) :: bs
       integer, intent(in)       :: bf ! which basis function
       integer, intent(in)       :: ncartdim
       integer, intent(in)       :: i,j ! which particles
       real(kind=8)              :: sep2
     end function sep2
  end interface


  ! ---- open output file ----
  open(unit=fug,file="decay.out")

!!!! Need to set number of basis functions etc.
!!!! allocate/create wavefn.

  nfiles = 0
  !  ampfile="amplitudes.out"
  ncartdim = 3
  nneons = 2
  nbromine = 2

  writedissconfig = .false. ! don't write dissociating configs
  nbasis = 50

  ! ---- cutoff radius ----
  rcut = 12.0d0
  rcut2 =rcut**2

  gammaBr=385.57
  gammaNe=10.1

  ! ---- defaults for padding ----
  pad = .false.
  t0 = 0
  dt = 1
  nsteps = 0

  ! weighting for this simulation
  simweight = 1.0d0

  ! ---- Try to open options file ----
  open(unit=fopt,file="decay.inpt",status="old",iostat = ierr)
  if ( ierr .ne. 0 ) then
     write(fug,*) "Failed to open option file decay.inpt" 
     write(fug,*) "Using default values"
  else
     ! ---- read options from file ----
     ! Read in options
     read (fopt,'(a100)',iostat=ierr) line
     do while (ierr .eq.0)
        read(line,*,iostat=ierr) keyword
        if (ierr.eq. 0) then
           if (keyword=="nbasis") then
              read(line,*) keyword,nbasis
           else if (keyword=="rcut") then
              read(line,*) keyword,rcut
              rcut2 = rcut**2
           else  if (keyword=="nneons") then
              read(line,*) keyword,nneons
           else  if (keyword=="nbromine") then
              read(line,*) keyword,nbromine
           else if (keyword=="gammaBr") then
              read(line,*) keyword,gammaBr
           else if (keyword=="gammaNe") then
              read(line,*) keyword,gammaNe    
           else if (keyword=="writedissconfig") then
              writedissconfig = .true.

           else if (keyword=="t0") then
              read(line,*) keyword,t0
           else if (keyword=="pad") then
              read(line,*) keyword
              pad = .true.
           else if (keyword=="dt") then
              read(line,*) keyword,dt
           else if (keyword=="simweight") then
              read(line,*) keyword,simweight
           else if (keyword=="multifile") then
              if (nfiles .ne. 0) then
                 write(*,*) "Trying to set number of files using multifile option, but it has already been set."
                 stop
              end if
              read(line,*,iostat = ierr) keyword,nfiles
              if (ierr .ne. 0 ) then
                 write(*,*) "Problem reading the number of files from decay.inpt"
                 stop
              end if

              ! Allocate array of file names
              allocate(ampfiles(nfiles))
              ! Read in file names
              do i = 1,nfiles
                 read(fopt,'(a100)') ampfiles(i)
              end do
           else if (keyword=="ampfile") then
              if (nfiles .ne. 0) then
                 write(*,*) "Trying to set number of files using multifile option, but it has already been set."
                 stop
              end if

              nfiles = 1
              allocate(ampfiles(nfiles))
              read(line,'(a100)') ampfiles(1)
           end if
        end if
        read (fopt,'(a100)',iostat=ierr) line
     end do
  end if

  ! Has one or more filenames been given? If not use amplitudes.out
  if (nfiles .eq. 0 ) then
     nfiles = 1
     allocate(ampfiles(nfiles))
     ampfiles(1) = "amplitudes.out"
  end if

  ! Calculate the total number of particles
  nparticles = nneons + nbromine
  !  and number of degrees
  ndf = nparticles*ncartdim  

  ! Calculate the number of steps for padding
  if(pad) then
     nsteps = t0/dt
  end if

  write(fug,*) "Summary"
  write(fug,*) 
  write(fug,*) "Number of Neons: ",nneons
  write(fug,*) "Number of basis functions:",nbasis
  write(fug,*) "Dissociation radius:",rcut
  write(fug,*) "Number of degrees of freedom:",ndf
  write(fug,*) "Weight",simweight
  write(fug,*) 
  write(fug,*) "Using ",nfiles," amplitude files:"
  do i = 1,nfiles
     write(fug,'("   ",i4,"  ",a100)') i,ampfiles(i)
  end do


  ! Create the wavefunction
  CALL createbasisset(wavefn,ndf,nbasis,zr=.true.)

  ! Set gammas and masses
  wavefn%gamma(1:nbromine*ncartdim) = gammaBr
  wavefn%gamma(nbromine*ncartdim+1:ndf) = gammaNe
  wavefn%mass(1:nbromine*ncartdim) = 79.904 ! Br mass
  wavefn%mass(nbromine*ncartdim+1:ndf) = 20.1797

  allocate(Nediss(nneons,nbasis))
  allocate(onediss(nbasis))
  Nediss(:,:) = -1
  onediss(:) = 0
  allocate(ndiss(nbasis))
  ndiss(:) = 0
  allocate(dissorder(nneons,nbasis))
  dissorder(:,:) = 0

  allocate(probdis(nneons))
  ! open output files
  allocate(fdiss(nneons))
  do i = 1,nneons
     fdiss(i) = i+40
     write(line,*) i
     open(fdiss(i),file="diss"//trim(adjustl(line)))
     if (i == 1) then
        write(fdiss(i),*) "# Probabiltiy of 1st dissociation"
     else if (i == 2) then
        write(fdiss(i),*) "# Probabiltiy of 2nd dissociation"
     else if (i == 3) then
        write(fdiss(i),*) "# Probabiltiy of 3rd dissociation"
     else 
        write(fdiss(i),*) "# Probabiltiy of ",trim(adjustl(line)),"th dissociation"
     end if
!     write(fdiss(i),"(16a,e18.8,a6)") "# Average over ",simweight," steps"
     write(fdiss(i),*) "# Average over ",simweight," steps"
     if (pad) then
        write(fdiss(i),*) "# Intitial time ",t0
     end if
  end do


  ! ---- pad dissociation files if required ----
  if(pad) then
     do i = 1,nneons
        do j = 0,nsteps-1
           write(fdiss(i),*)  j*dt, 0.0
        end do
     end do
  end if

  ! ---- Set dissociation prob to zero ----
  probdis(:) = 0.0d0

  do file = 1,nfiles
     ! ---- open amplitudes file ----
     open(unit=famp,file=ampfiles(file),status="old",iostat=ierr) 
     if (ierr .ne. 0) then 
        write(*,*) "Failed to open amplitude file ",ampfiles(file)
        stop
     end if
     write(fug,*) "Reading configurations from file ",file,ampfiles(file)
     ! Read in first configuration
     call readconfiguration(famp,wavefn,readconfigerr,fug)
     ! If this is not the first file, skip the first configuration as
     ! it should be the same as the last configuration in the previous file.
     if (file > 1) then
        call readconfiguration(famp,wavefn,readconfigerr,fug)
     end if
     write(fug,*) "read config err",readconfigerr
     nconfigs = 0
     do while (readconfigerr .eq. 0) 
        nconfigs = nconfigs+1
        write(fug,*) "Config ",nconfigs

        ! Calculate C
        if ( .not.wavefn%Cset) then
           CALL calcC(wavefn)
        end if

        do bf = 1,wavefn%nbasis
           firstdissevent = .false.
           do i = 1,nneons
              if (Nediss(i,bf) < 0) then ! has this neon in this basis fn dissociated?
                 if (R2(wavefn,bf,ncartdim,i) > rcut2) then

                    ndiss(bf) = ndiss(bf) + 1 ! count the number of dissociated Neonds
                    if (ndiss(bf) == 1)  then
                       firstdissevent = .true.
                    end if

                    write(fug,*) bf, i, R2(wavefn,bf,ncartdim,i)

                    ! Calculate probaility associated with this basis function
                    prob = real(dconjg(wavefn%C(bf))*wavefn%D(bf))

                    ! Calculate the probability of dissociation
                    probdis(ndiss(bf)) = probdis(ndiss(bf)) + prob

                    ! Record when this dissociation event occured.
                    Nediss(i,bf) = nconfigs

                    ! Record the order.
                    dissorder(ndiss(bf),bf) = i

                 end if
              end if
           end do

           ! Check for multiple dissociations
           if ( firstdissevent  .and. nneons > 1) then

              if (ndiss(bf) > 1) then
                 write(fug,*) "Multiple dissociation. N=",Ndiss(bf),"bf = ",bf,"prob = ", real(dconjg(wavefn%C(bf))*wavefn%D(bf))
                 ! calculte the Ne-Ne separation
                 write(fug,*) "Ne-Ne ",sep2(wavefn,bf,ncartdim,dissorder(1,bf),dissorder(2,bf))
              end if

              ! Write out dissociation configuration?
              if(writedissconfig) then
                 write(line,*) bf
                 open(fdconfig,file="dissconfig"//trim(adjustl(line)))
                 write(fdconfig,*) "BasisFunction",bf
                 write(fdconfig,*) "Time",wavefn%time
                 write(fdconfig,*) "Config",nconfigs
                 write(fdconfig,*) "Weight",real(dconjg(wavefn%C(bf))*wavefn%D(bf))
                 write(fdconfig,*) "DissociatedParticle", nbromine + dissorder(1,bf)
                 if (Ndiss(bf) > 1) then
                    write(fdconfig,*) "Multiple Dissociation"
                 end if
                 write(fdconfig,*) "PosMom"
                 do i = 1,ndf
                    write(fdconfig,*) wavefn%cs(bf)%q(i),wavefn%cs(bf)%p(i)
                 end do

                 close(fdconfig)

              end if

           end if
        end do

        ! Write out the dissociation
        do i = 1,nneons
           write(fdiss(i),*) wavefn%time+t0,probdis(i)
        end do

        call readconfiguration(famp,wavefn,readconfigerr,fug)
     end do

  end do
  do i = 1,nneons
     close(fdiss(i))
  end do

  close(fug)
end program twostagedecay


function R2(bs,bf,ncartdim,ne)
  use basismodule
  implicit none
  ! ---- interface ----
  type(basisset),intent(in) :: bs
  integer, intent(in)       :: bf ! which basis function
  integer, intent(in)       :: ncartdim
  integer, intent(in)       :: ne ! which neon
  real(kind=8) :: R2
  ! ---- local ----
  real(kind=8),dimension(:),allocatable :: c ! centre
  integer :: i

  allocate(c(ncartdim))

  do i = 1,ncartdim
     c(i) = 0.5*(bs%cs(bf)%q(i) + bs%cs(bf)%q(ncartdim+i))
  end do

  
  R2 = 0.0d0
  do i = 1,ncartdim
     ! 1+Ne was 2+ (Ne-1) but these are clearly the same. Originally just 2.
     R2 = R2 + (bs%cs(bf)%q((1+Ne)*ncartdim+i)-c(i))**2
  end do
end function R2

!!% ========================================================================
!!%  Function: sep2
!!%  Function to calculate the square of the separation of two particles
!!% ========================================================================
function sep2(bs,bf,ncartdim,i,j)
  use basismodule
  implicit none
  ! ---- interface ----
  type(basisset),intent(in) :: bs
  integer, intent(in)       :: bf ! which basis function
  integer, intent(in)       :: ncartdim
  integer, intent(in)       :: i,j ! which particles
  real(kind=8)              :: sep2
  ! ---- local ----
  integer :: d
  
  sep2 = 0.0d0
   do d = 1,ncartdim
    sep2 = sep2 + (bs%cs(bf)%q((i-1)*ncartdim+d) - bs%cs(bf)%q((j-1)*ncartdim+d))**2
  end do
  
end function sep2
