program brenergy
  use basismodule 
  use system
  use br2nehamiltonian
  use errors
  use histogram
  use morsesampling

  implicit none
  integer :: ierr
  integer :: i ! Loop index
  character(len=80) :: pottype
  integer :: filendf ! for checking the number of degrees of freedom
   
  character(len=101) :: line
  character(len=50)  :: keyword

  character ( len = 80 ) arg ! for command line options
  integer numarg             ! number of command line options
  integer iargc              ! For interpretation loop

  ! ---- open output file ----
  open(unit=fug,file="brenergy.out")

  inputfile="brenergy.inpt"  

  ! =============== Find out number command line options ===============
  numarg = iargc ( )
  i = 1
  do while (i .le. numarg)
     call getarg ( i, arg )
     
     ! ----- range ----
     if (arg .eq. "-h" .or. arg .eq. "--help" ) then
        call help
        stop
     else ! get name of input file to use.
        inputfile=arg
     end if
  end do
  
  
  !  ampfile="amplitudes.out"
  filesuffix=""
  ncartdim = 3
  nbromines = 2
  nneons = 1 ! Number of neons

  ! ---- cutoff radius ----
  rcut = 12.0d0
  rcut2 =rcut**2

  ! Set default gammas and masses - these may be chaned in input file.
  gammaBr=385.57
  gammaNe=10.15
  massBr = 79.904
  massNe = 20.1797


  histmin = -12.0d0
  histmax = -2.0d0
  nbins = 100
  brnehistmin = -1
  brnehistmax = 3
  brnenbins = 50
  timedep = .false. ! Should we calculate a separate histogram at each time?

  quantise = .false.
  vmin = -1
  vmax = 0

  tmin = -1d20 ! Effective set time interval to [-infinity, infinity]
  tmax = 1d20

  
  ! ---- Read input file ----
  open(unit=fopt,file=inputfile,iostat=ierr)
  if (ierr .ne. 0) then
     write(fug,*) "Failed to open input file, brenergy.inpt"
     stop
  end if
  ! Read in options
  read (fopt,'(a100)',iostat=ierr) line
  do while (ierr .eq.0)
     read(line,*,iostat=ierr) keyword
     if (ierr.eq. 0) then
        if (keyword=="nbasis") then
           read(line,*) keyword,nbasis
        else if (keyword=="rcut") then
           read(line,*) keyword,rcut
           rcut2 = rcut**2
        else if (keyword=="nbins") then
           read(line,*) keyword,nbins
        else if (keyword=="min") then
           read(line,*) keyword,histmin
        else if (keyword=="max") then
           read(line,*) keyword,histmax
        else if (keyword=="brnemin") then
           read(line,*) keyword,brnehistmin
        else if (keyword=="brnemax") then
           read(line,*) keyword,brnehistmax
        else if (keyword=="gammaBr") then
           read(line,*) keyword,gammaBr
        else if (keyword=="gammaNe") then
           read(line,*) keyword,gammaNe
        else if (keyword=="quantise") then
           quantise = .true.
        else if (keyword=="vrange") then
           read(line,*),keyword,vmin,vmax
        else if (keyword=="timedep") then
           timedep = .true.   
        else if (keyword=="nneons") then
           read(line,*,iostat=ierr),keyword,nneons
           if (ierr .ne.0) then
              write(fug,*) "Problem reading the number of neons from input file"
              write(fug,*) "Read: ",line
              stop
           end if
        else if (keyword=="ncartdim") then
           read(line,*,iostat=ierr),keyword,ncartdim
           if (ierr .ne.0) then
              write(fug,*) "Problem reading the number of cartesian dimensions from input file"
              write(fug,*) "Read: ",line
              stop
           end if
        else if (keyword=="ampfile") then
           if(.not.allocated(ampfiles)) then
              nfiles=1
              allocate(ampfiles(nfiles))
              read(fopt,*) ampfiles
           else
              write(fug,*) "Trying to set name for single amplitude file but it has already been set."
              stop
           end if

        else if (keyword=="multifile") then
           if (allocated(ampfiles)) then
              write(fug,*) "Trying to set number of files using multifile option, but it has already been set."
              stop
           end if
           read(line,*,iostat = ierr) keyword,nfiles
           if (ierr .ne. 0 ) then
              write(fug,*) "Problem reading the number of files from brenergy.inpt"
              stop
           end if
           ! Allocate array of file names
           allocate(ampfiles(nfiles))
           ! Read in file names
           do i = 1,nfiles
              read(fopt,'(a100)') ampfiles(i)
           end do

        else if (keyword=="tmin") then
           read(line,*,iostat=ierr) keyword,tmin
           if (ierr .ne. 0 ) then
              write(fug,*) "Error reading tmin from input file"
              stop
           end if
        else if (keyword=="suffix") then
           read(fopt,*) filesuffix
        else if (keyword=="tmax") then
           read(line,*,iostat=ierr) keyword,tmax
           if (ierr .ne. 0) then
              write(fug,*) "Error reading tmax from input file"
              stop
           end if
        end if
     end if
     read (fopt,'(a100)',iostat=ierr) line
  end do

  ! Has one or more filenames been given? If not use amplitudes.out
  if (.not. allocated(ampfiles)) then
     nfiles = 1
     allocate(ampfiles(nfiles))
     ampfiles(1) = "amplitudes.out"
  end if

  ! ---- Calculate the number of particles etc. ----
  nparticles = nneons + nbromines
  ndf = nparticles*ncartdim  ! number of degrees of freedom

  ! ---- Create the wavefunction ----
  CALL createbasisset(wavefn,ndf,nbasis,zr=.true.)
  
  ! ---- Read in the hamiltonian ----
  open(unit=fham,file="hamiltonian.inpt",status="old",iostat=ierr)
  if (ierr .ne. 0) then
     call fileerror(ierr,"hamiltonian.inpt",fham)
  end if

  read(fham,*) pottype
  if (.not.(pottype == "cartesian")) then
     write(fug,*) "hamiltonian.inpt does not a cartesian potential"
     stop
  end if
  read(fham,*) line,filendf
  if (filendf .ne. ndf) then
     write(fug,*) "Wrong number of degrees of freedom"
     write(fug,*) "Here ",ndf," in hamiltonian.inpt",filendf
     stop
  end if


  ! ---- set gammas ----
  wavefn%gamma(1:2*ncartdim) = gammaBr
  wavefn%mass(1:2*ncartdim) = massBr ! Br mass
  if (nneons > 0) then
     wavefn%gamma(2*ncartdim+1:ndf) = gammaNe
  wavefn%mass(2*ncartdim+1:ndf) = massNe
  end if

  ! ----- read hamiltonian ----
  call readhamcart(wavefn)

  ! ---- read morse hamiltonian if required ----
  if (quantise) then
     call readhamiltonian("morseham.inpt")
  end if

  
  if (nneons .eq. 1) then
     call calcbrengdist()
  else if (nneons .eq. 0) then
     call calcbrengdistnoneons()
  else
     write(fug,*) "Sorry, I do no know how to deal with ",nneons," neons"
  end if
  

end program brenergy


!!% ========================================================================
!!%  Subroutine: help
!!%  Describe the operation of the program
!!% ========================================================================

subroutine help(opt)
  implicit none
  character,intent(in),optional :: opt(:)
  write(*,*) "brenergy"
  write(*,*)
  stop
end subroutine help
