!!% ========================================================================
!!% Subroutine: calcbrengdist
!!% Author: Stewart Reed
!!% Date: 10/11/2009
!!% Description: This subroutine now contains the code used to calculate 
!!%              the Br2 Energy Distributions for systems that have 1 Ne.
!!% ========================================================================
subroutine calcbrengdist
  use basismodule 
  use system
  use br2nehamiltonian
  use histogram
  use morsesampling

  implicit none

  ! ---- local variables ----
  integer :: file,i,j,k,l ! Loop indices

  integer :: ierr ! file error
  integer :: readconfigerr
    
  type(histogram1D) :: modeenergyprofile
  type(histogram1D) :: br2modeenergyprofile,brNemodeenergyprofile
  
  real(kind=8) :: modeenergy
  real(kind=8),dimension(:),allocatable :: modeengexpect,pbonds ! mode energies, and momentas calculated by matrix inversion

  integer :: bf
  real(kind=8),dimension(:),allocatable :: prob ! probability of a particular basis funciton
  integer :: nconfigs  ! number of configurations read in.
  logical,dimension(:),allocatable :: dissociated  ! has a particular basis fn already dissociated?
  integer :: ndisthistime ! number of basis fns that have dissociated at a particular time.
  integer :: totaldis     ! Total number of basis functions that have dissosciated
  
  real(kind=8) :: rBrBr,rBrNe
  real(kind=8) dvlprob
  integer :: dvl

  type(histogram1D) :: br2quantisedenergy
  type(basisset) Br2Wfn
  real(kind=8) :: norm
  complex(kind=8) :: ovlpzeta,ovlpz

  ! Total Dissociation Probability
  real(kind=8) :: totprob
  real(kind=8) :: x
  integer :: ndx
  integer :: dof
  complex(kind=8) :: px

  character(len=20) :: modenumber
  
  ! Interface for the R2 routine
  interface
     function R2(bs,bf,ncartdim)
       use basismodule
       implicit none
       ! ---- interface ----
       type(basisset),intent(in) :: bs
       integer, intent(in)       :: bf ! which basis function
       integer, intent(in)       :: ncartdim
       real(kind=8) :: R2
     end function R2
  end interface


  ! ---- allocate array to say if a particular basis fn has dissociated ----
  allocate(dissociated(wavefn%nbasis))
  dissociated(:) = .false.
  ! ---- allocate an array for dissociation probabilities ----
  allocate(prob(wavefn%nbasis))
  ! ---- allocate array for bond energies ----
  if (nparticles == 3) then
     allocate(pbonds(3))
     allocate(modeengexpect(3))
  else if (nparticles == 4) then
     allocate(pbonds(6))
     allocate(modeengexpect(6))
  else
     write(fug,*) " I don't know how big to make pbonds array for ",nparticles," particles"
     stop
  end if


  ! ---- Create histogram ----
  !  call createhistogram(modeenergyprofile,histmin,histmax,nbins)
  call createhistogram(br2modeenergyprofile,histmin,histmax,nbins)
  call createhistogram(brNemodeenergyprofile,brnehistmin,brnehistmax,brnenbins)

  if (quantise) then
     call createhistogram(br2quantisedenergy,dble(vmin)-0.5d0,dble(vmax)+0.5d0,vmax-vmin+1)
     !     call createhistogram(brNequantisedenergy,dble(vmin)-0.5d0,dble(vmax)+0.5d0,vmax-vmin+1)
     call createbasisset(Br2Wfn,3,nbasis,zr=.true.)
     Br2Wfn%gamma(1) = gammaBr/2.0d0
     Br2Wfn%mass(1) = massBr/2.0d0
     Br2Wfn%gamma(2) = gammaBr*gammaNe/( gammaBr+gammaNe)
     Br2Wfn%mass(2) = massBr*massNe/(massBr+massNe)
     Br2Wfn%gamma(3) = Br2Wfn%gamma(2)
     Br2Wfn%mass(3) = Br2Wfn%mass(2)

  end if

  ! 
  nconfigs = 0
  totprob = 0.0d0

  write(fug,*) "Reading configurations"

  do file = 1,nfiles
     ! ---- open amplitudes file ----
     open(unit=famp,file=ampfiles(file),iostat=ierr,status="old") 
     if (ierr .ne. 0) then 
        write(fug,*) "Failed to open amplitude file ",ampfiles(file)
        stop
     end if
     write(fug,*) "File ",file,trim(ampfiles(file))
     ! Read in first configuration
     call readconfiguration(famp,wavefn,readconfigerr)
     if (readconfigerr .ne. 0) then
        write(fug,*) "read config err",readconfigerr
     end if

     ! Keep count of the total number of basis functions to have dissociated.
     totaldis = 0
     configloop: do while (readconfigerr .eq. 0 .and. totaldis < wavefn%nbasis .and. wavefn%time < tmax ) 
        nconfigs = nconfigs+1
        write(fug,*) "Config ",nconfigs

        ! If we haven't yet got to the minimum time, move onto next configuration.
        if ( wavefn%time < tmin) then
           call readconfiguration(famp,wavefn,readconfigerr)
           cycle configloop
        end if

        ! Calculate C
        if(.not.wavefn%cset) then
           CALL calcC(wavefn)
        end if

        ndisthistime = 0 ! Count how many have dissociated this time step
        bfloop: do bf = 1,wavefn%nbasis
           if (.not. dissociated(bf)) then           
              ! calculate the distance between Br2 and Ne
              if (R2(wavefn,bf,ncartdim) > rcut2) then
                 ! make a note that this basis function has been counted.
                 dissociated(bf) = .true.

                 ndisthistime = ndisthistime + 1
                 totaldis = totaldis + 1
                 !              modeenergy = brmodehexpect(wavefn,bf)*eu
                 prob(ndisthistime) = real(dconjg(wavefn%C(bf))*wavefn%D(bf))
                 totprob = totprob + prob(ndisthistime) 

                 !             call addtohistogram(modeenergyprofile,modeenergy,prob(ndisthistime))

                 call  modeshexpect2D(wavefn,bf,modeengexpect, pbonds)
                 modeengexpect(:) = modeengexpect(:)*eu
                 call addtohistogram(br2modeenergyprofile,modeengexpect(1),prob(ndisthistime))
                 call addtohistogram(brNemodeenergyprofile,modeengexpect(2),prob(ndisthistime))
                 call addtohistogram(brNemodeenergyprofile,modeengexpect(3),prob(ndisthistime))


                 if (quantise) then
                    ! Project this basis function on to internal co-ordinates.
                    rBrBr = sqrt(sum( (wavefn%cs(bf)%q(1:ncartdim) - wavefn%cs(bf)%q(ncartdim+1:2*ncartdim))**2))
                    Br2Wfn%cs(ndisthistime)%q(1) = rBrBr
                    Br2Wfn%cs(ndisthistime)%p(1) = pbonds(1)

                    rBrNe =  sqrt(sum( (wavefn%cs(bf)%q(1:ncartdim) - wavefn%cs(bf)%q(2*ncartdim+1:3*ncartdim))**2))
                    Br2Wfn%cs(ndisthistime)%q(2) = rBrNe
                    Br2Wfn%cs(ndisthistime)%p(2) = pbonds(2)

                    rBrNe =  sqrt(sum( (wavefn%cs(bf)%q(ncartdim+1:2*ncartdim) - wavefn%cs(bf)%q(2*ncartdim+1:3*ncartdim))**2))
                    Br2Wfn%cs(ndisthistime)%q(3) = rBrNe
                    Br2Wfn%cs(ndisthistime)%p(3) = pbonds(3)

                    if (rcut2 > 0.1) then ! assume they are all independent.
                       do dvl = vmin,vmax
                          ! Project Br-Br on to Morse wave function
                          Br2Wfn%C(ndisthistime) =  &
                               projectcsmorsedvib(1,Br2Wfn%cs(ndisthistime)%q(1),Br2Wfn%cs(ndisthistime)%p(1), dvl)
                          ! Add D*k Dk |<nu| zeta>?^2 Actually we assume that D*k = C*k
                          ! Check this assumption
                          if (abs(wavefn%C(bf)-wavefn%D(bf)) > 1e-3) then
                             write(fug,*) "**** bf ",bf,"C .ne. D config ",nconfigs," ***** " 
                          end if
                          !  Calculate probability
                          dvlprob = prob(ndisthistime)*real(conjg(Br2Wfn%C(ndisthistime))*Br2Wfn%C(ndisthistime))
                          ! Add to histogram                        
                          call addtohistogram(br2quantisedenergy,dble(dvl),dvlprob)                        
                       end do
                       !                 else ! if rcut2 == 0

                    end if

                 end if
                 write(fug,*) bf,R2(wavefn,bf,ncartdim),modeenergy,prob(ndisthistime),modeengexpect(1)

              end if
           end if
        end do bfloop

        if (quantise) then
           if (rcut2 < 0.1) then
              write(fug,*) "Doing double sum probability calculation"
              dof = 1  ! For plotting wave function
              if (ndisthistime > 0 ) then
                 if (ndisthistime .ne. wavefn%nbasis) then
                    write(fug,*) " ********* ndisthistime .ne. wavefn%nbasis **************";
                    write(fug,*) "rcut2 = ",rcut2
                    stop
                 end if

                 Br2Wfn%nbasis = ndisthistime
                 call calczfromqp(Br2Wfn)

                 ! Loop over requested vibrational levels.
                 do dvl = vmin,vmax

                    dvlprob = 0.0d0
                    dx = (0.0d0,0.0d0)
                    do i = 1,Br2Wfn%nbasis
                       ! Project each basis function onto
                       ! Actually, it is the conjg of C that we need,
                       Br2Wfn%C(i) = projectcsmorsedvib(1,Br2Wfn%cs(i)%q(1),Br2Wfn%cs(i)%p(1),dvl)

                       ! Now add the diagonal contribution.
                       dvlprob = dvlprob + real(conjg(wavefn%D(i))*wavefn%D(i) * Br2Wfn%C(i)*conjg(Br2Wfn%C(i)))

                    end do
                    do i = 1,Br2Wfn%nbasis
                       do j = i+1,Br2Wfn%nbasis
                          ! Calculate overlap
                          ovlpzeta = overlap1dof(Br2Wfn,i,j,2)*overlap1dof(Br2Wfn,i,j,3)
                          !                       ovlpzeta = overlap(Br2Wfn,i,j)
                          if (abs(ovlpzeta) > 1d-10) then
                             dvlprob = dvlprob + 2*real(conjg(wavefn%D(i))*wavefn%D(j)  &
                                  *ovlpzeta  & !overlap(wavefn,i,j)/ovlpzeta &
                                  *( Br2Wfn%C(i)*conjg(Br2Wfn%C(j))))
                          end if
                       end do
                    end do

                    ! Add to histogram                        
                    call addtohistogram(br2quantisedenergy,dble(dvl),dvlprob)  
                 end do

                 ! Now calculate D for Br2Wfn
                 !              write(fug,*) "Now calculate D for Br2Wfn"
                 !              call calcD(Br2Wfn)

                 ! Plot wave fn
                 write(fug,*) "Projecting wave function"

                 do dof = 1,3
                    write(modenumber,"(i4)") dof
                    outfile="wfnprojection-mode"//trim(adjustl(modenumber))//trim(filesuffix)
                    open(unit=fproject,file=outfile)
                    if (dof.eq. 1) then
                       xmin = 2.0
                       xmax = 4.0
                       ndx = 400
                       dx = (xmax-xmin)/dble(ndx)
                    else
                       xmin = 3.2
                       xmax = 5.2
                       ndx = 400
                       dx = (xmax-xmin)/dble(ndx)
                    end if

                    do i = 0,ndx
                       x = dble(i) * dx + xmin
                       
                       px = 0.0d0
                       do j = 1,nbasis
                          px = px + (abs(wavefn%D(j)* project_xz(x,Br2wfn,j,dof))) **2
                          do k = j+1,nbasis
                             
                             ! Calculate overlap of other modes
                             ovlpzeta = 1.0d0
                             do l = 1,3
                                if (l.eq.dof) cycle
                                ovlpzeta = ovlpzeta * overlap1dof(Br2Wfn,j,k,l)
                             end do

                          !                        write(998,'(10es18.8)') overlap(wavefn,j,k)/ overlap1dof(Br2Wfn,j,k,1),ovlpzeta
                             px =  px  &
                                  + 2*real(conjg(wavefn%D(j)*project_xz(x,Br2wfn,j,dof))*wavefn%D(k)&
                                  *project_xz(x,Br2wfn,k,dof) * ovlpzeta)
                          end do

                    end do
                    write(fproject,*) x,real(px),imag(px)
                 end do
                 close(fproject)
              end do
                 


              end if
           end if
           Br2Wfn%nbasis = Wavefn%nbasis
        end if

        call readconfiguration(famp,wavefn,readconfigerr)       
     end do configloop

     close(famp)
  end do
  !  call normalisehistogram(modeenergyprofile)
  !  call writehistogram(modeenergyprofile,"Br2ModeEnergy","Distribution of Br2 mode energies")

  call normalisehistogram(br2modeenergyprofile)
  outfile="Br2ModeEnergyMatrix"//trim(filesuffix)
  call writehistogram(br2modeenergyprofile,outfile,"Distribution of Br2 mode energies by matrix inversion")

  call normalisehistogram(BrNemodeenergyprofile)
  call writehistogram(brNemodeenergyprofile,"BrNeModeEnergy"//trim(filesuffix),"Distribution of Br-Ne mode energies")

  if (quantise) then
     ! Write out unnormalised distribution
     outfile="Br2ModeEnergyQuantised-notnormalised"//trim(filesuffix)
     call writehistogram(br2quantisedenergy,outfile,"Distribution of quantised Br2 mode energies")

     ! Now normalise
     call normalisehistogram(br2quantisedenergy)
     br2quantisedenergy%v(:) = br2quantisedenergy%v(:)*totprob
     write(fug,*) "Total Probability",totprob
     outfile="Br2ModeEnergyQuantised"//trim(filesuffix)
     call writehistogram(br2quantisedenergy,outfile,"Distribution of quantised Br2 mode energies")
  end if


end subroutine calcbrengdist


!!% ========================================================================
!!% Funtion: R2
!!% Author: Stewwart Reed
!!% Date: 
!!% Description: Calculate the square separation of the Neon and the centre
!!%              of mass of the Br2 molecule
!!% Arguments: bs - a basis set
!!%            bf - which basis function to calculate separation for              
!!%            ncartdim - Number of cartesian dimensions of the system
!!% ========================================================================
function R2(bs,bf,ncartdim)
  use basismodule
  implicit none
  ! ---- interface ----
  type(basisset),intent(in) :: bs
  integer, intent(in)       :: bf ! which basis function
  integer, intent(in)       :: ncartdim
  real(kind=8) :: R2
  ! ---- local ----
  real(kind=8),dimension(:),allocatable :: c ! centre
  integer :: i

  allocate(c(ncartdim))

  do i = 1,ncartdim
     c(i) = 0.5*(bs%cs(bf)%q(i) + bs%cs(bf)%q(ncartdim+i))
  end do

  R2 = 0.0d0
  do i = 1,ncartdim
     R2 = R2 + (bs%cs(bf)%q(2*ncartdim+i)-c(i))**2
  end do
end function R2
