subroutine calcprob

  use eamodule

  implicit none

  complex(kind=8) :: C,Cmode
  integer :: bf,i
  integer :: superid


  ! Set the amplitude of this basis function
  C = (1.0,0.0)
  
  superid = 0
  ! Loop over the degrees of freedom
  !do bf = 1,ninitstates
  bf = 1
  superid = 0
  do i=1,ndf

     if (harmmethod(i) .eq. 3  ) then
        
        call harmuseinitial(zinit(i,bf),vstate(i),Cmode)
        
     else if (harmmethod(i) .eq.4) then
        call harmuseinitial(zinit(i,bf)-zoffset(i),vstate(i),Cmode)
        
     else if (harmmethod(i) .eq.7) then
        superid = superid + 1
        call harmuseinitialsuper(zinit(i,bf),superid,Cmode)
        
     else if (harmmethod(i) .eq.8) then
        superid = superid + 1
        call harmuseinitialsuper(zinit(i,bf)-zoffset(i),superid,Cmode)
     end if
     
     C = C*Cmode
  end do

  !end do

  ! ---- Finally, the probablity ----
  prob = real(dconjg(C)*C)
  write(fug,*) "Camp",C, dconjg(C)*C,prob


end subroutine calcprob
! ===============================================
!                harmuseinitial
!  subroutine to use basis functions from the 
!  intial states and project it on to state 
!  |n> of the harmonic oscillator. 
!
!  if the harmonic mode is offset then 
!   "unoffset" the initial basis function for 
!  projection. We assume that basis functions 
!  in the intial states are already offset.
! ===============================================
 subroutine harmuseinitial(z,n,ampC)
   
   implicit none

   complex(kind=8),intent(in) :: z
   integer, intent(in) :: n
  ! amplitude i.e. <z|Phi_n>
   complex(kind=8),intent(out) :: ampC 

   integer :: if ! for factorial loop
   real(kind=8) :: factn ! n factorial

   ! Now we should calculate the amplitude for this mode/fn
   select case(n)
   case(0)
      
      ampC = dexp(-0.5*(cdabs(z)**2))
   case(1)
      ampC = dconjg(z)*dexp(-0.5*(cdabs(z)**2))
      
   case default
      ! Calculate n factorial
      factn=1.0
      do if = 2,n
         factn = factn*if
      end do

      ! Now the projection
      ampC = dconjg(z)**(n)*dexp(-0.5*(cdabs(z)**2))/dsqrt(factn)
      
  end select

end subroutine harmuseinitial


! ===============================================
!                harmuseinitialsuper
!  subroutine to use basis functions from the 
!  intial states and project it on to state 
!  |n> of the harmonic oscillator. 
!
!  if the harmonic mode is offset then 
!   "unoffset" the initial basis function for 
!  projection. We assume that basis functions 
!  in the intial states are already offset.
! ===============================================
 subroutine harmuseinitialsuper(z,n,ampC)
   
   use eamodule, ONLY: superphi,nsuperbasis
   implicit none

   complex(kind=8),intent(out) :: z
   integer, intent(in) :: n ! which superposition
  ! amplitude i.e. <z|Phi_n>
   complex(kind=8),intent(out) :: ampC 
   
   integer :: if ! for factorial loop
   real(kind=8) :: factn ! n factorial

   integer :: j ! for loop

   ! Now we should calculate the amplitude for this mode/fn
   ampC = superphi(1,n) * dexp(-0.5*(cdabs(z)**2))
   if (nsuperbasis .eq. 1) return
      

   ampC =  ampC + superphi(2,n) * dconjg(z)*dexp(-0.5*(cdabs(z)**2))
   if (nsuperbasis .eq. 2) return
   
   do j = 3,nsuperbasis
      ! Calculate n factorial
      factn=1.0
      do if = 2,n
         factn = factn*if
      end do
      
      ! Now the projection
      ampC =  ampC + superphi(j,n) *dconjg(z)**(n)*dexp(-0.5*(cdabs(z)**2))/dsqrt(factn)
      
   end do
  
   
 end subroutine harmuseinitialsuper
  
