! **************************************************
!                  energyaverage
! 
!                   S.K. Reed
!              Fri. 8th February 2008
!
!
! **************************************************

! ========== eamodule ==============================
!  A simple module to hold all the variables and
!  subroutines
! ================================================== 
module eamodule

  ! ---- File units ----
  integer :: fopt=10 ! for control file
  integer :: fis=11  ! For initial states
  integer :: fengin = 12 ! For reading in energy

  integer :: fug=21 ! General output 
  integer :: fengout=22 ! Energy output

  ! ---- File names ----
  character(len=100) :: dirstem
  character(len=100) :: energyfile
  character(len=100) :: statesfile
  

  ! ---- Some numbers ----
  real(kind=8),parameter ::pi     = 3.1415926535897932385d0

  ! ---- details of the system ----
  integer :: ndf ! number of degrees of freedom
  integer :: nsteps ! number of steps to read in

  ! ---- details of the basis type ----
  integer ::  basismethod

  ! -- gammas --
  real(kind=8),dimension(:),allocatable :: gamma
  real(kind=8),dimension(:),allocatable :: s2gammarec !1/sqrt(2*gamma)
  real(kind=8),dimension(:),allocatable ::shgamma !sqrt(gamma/2)
  real(kind=8),dimension(:),allocatable ::sgamma ! sqrt(gamma)

  ! -- initial state(s)  --
  ! number,z(ndf,ninitstates) and widths (cmprss) 
  integer :: ninitstates
  complex(kind=8),dimension(:,:), allocatable :: zinit
   ! -- For Harmonic oscillator --
  ! vibrational state
  integer,dimension(:),allocatable :: harmmethod
  integer,dimension(:),allocatable :: vstate
  real(kind=8),dimension(:),allocatable :: harmComp    ! sampling compression
  ! offset of of origin for sampling. I.e. produce a distribution about this pt.
  complex(kind=8),dimension(:),allocatable :: zoffset 
  
  ! superposition of harmonic oscillator wave functions
  integer :: nsupermodes
  integer :: nsuperbasis
  real(kind=8), dimension(:,:), allocatable :: superphi
  
  ! ---- details of runs performed ----
  integer :: nruns

  ! ---- For averaging ----
  ! Averagetype
  integer :: atype
  ! Mean energy
  real(kind=8), dimension(:,:), allocatable :: meanenergy 
  ! Variance, for averaging whole system runs.
  real(kind=8), dimension(:,:), allocatable :: varenergy

  ! probability for this run, and sum probs so far.
  real(kind=8) :: prob
  real(kind=8) :: sumprob

contains

   ! ========== readbasistype =========================
  !  Subroutine to readin details of basis type.
  !  From simulation program but only handles 
  !  harmonic oscillator basis functions.
  ! ================================================== 
  subroutine readinputs
    use errors
    implicit none
    CHARACTER(len=50) :: line, basistype
    integer :: ierr
    integer :: i,s
    ! ---- Set some defaults ----
    atype = 0
    nsteps = 1
    ndf = 0
    
    dirstem = ""
    energyfile = "energy.out"
    statesfile = "states.inpt"

    basismethod = 0

    ! ---- Open input file ----
    open(fopt,file="energyaverage.inpt",status='old',iostat=ierr)
    if(ierr .ne. 0 ) then
       call help
       call fileerror(ierr,"energyaverage.inpt",fug)
       stop
    end if
        
    ! ---- read options from file ----
    read(fopt,*,iostat=ierr) line
    do while (ierr == 0)   

       if (line(1:1) == "#" ) then
          ! ---- This is a comment ----

       else if (line == "atype" ) then
          ! ----Average type ----
          backspace(fopt) 
          read(fopt,*,iostat=ierr) line,line
          if(ierr.ne.0) call filereaderror(ierr,"energyaverage.inpt","Error reading number of time steps",fug)
          if (trim(line) == "mean" ) then
             atype = 1
             write(fug,*) " "
          else if (trim(line) .eq. "weighted" ) then
             atype = 2
             write(fug,*) "Performing a weighted mean"
          else 
             write(fug,*) "Unkown averaging method",line
             stop
          end if

       else if (trim(line).eq."dirstem") then
          backspace(fopt)
          read(fopt,*) line,dirstem
          write(fug,*) "Directory stem: ",dirstem

       else if (trim(line).eq."energyfile") then
          backspace(fopt)
          read(fopt,*) line,energyfile

       else if (line == "nruns" ) then
          ! ---- Number of runs ----
          backspace(fopt) 
          read(fopt,*,iostat=ierr) line,nruns
          if(ierr.ne.0) call filereaderror(ierr,"energyaverage.inpt","Error reading number of runs",fug)
          write(fug,*) "Averaging over ",nruns," runs"

       else if (line == "nsteps" ) then
          ! ---- Number of steps ----
          backspace(fopt) 
          read(fopt,*,iostat=ierr) line,nsteps
          if(ierr.ne.0) call filereaderror(ierr,"energyaverage.inpt","Error reading number of time steps",fug)
          write(fug,*) "Each run contains ",nsteps," steps"
          
       else if  (line == "ndf" ) then
          ! ---- number of degrees of freedom ----
          backspace(fopt) 
          read(fopt,*,iostat=ierr) line,ndf
          if(ierr.ne.0) call filereaderror(ierr,"energyaverage.inpt","Error reading number of degrees of freedom",fug)
          write(fug,*) ndf," degrees of freedom"

           ! ---- gammas ----
         else if (line == "gamma") then
            write(fug,*) "Reading gamma"
            if(ndf .eq. 0 ) then
               write(fug,*) &
                    " Need to specify ndf before specifying gamma"
               stop
            else
               allocate(gamma(ndf),s2gammarec(ndf),shgamma(ndf),sgamma(ndf))
               do i = 1,ndf
                  read(fopt,*,iostat=ierr) s,gamma(i)
                  if(ierr.ne.0) then
                     write(fug,*) "state: ",i
                     call filereaderror(ierr,"run.inpt","Error reading gamma",fug)
                  end if
               end do

               s2gammarec(:) = 1/dsqrt(2.0*gamma(:)) 
               shgamma(:)    = dsqrt(gamma(:)/2.0)
               sgamma(:) = dsqrt(gamma(:))
            end if

       else if  (line == "basis" ) then
          ! ---- basis type ----
          backspace(fopt) 
          read(fopt,*,iostat=ierr) line,basistype
          if(ierr.ne.0) call filereaderror(ierr,"energyaverage.inpt","Error readingbasis type",fug)
          if(ndf .eq. 0 ) then
             write(fug,*) &
                  " Need to specify ndf before specifying basis type"
             stop
          else
             call readbasistype(basistype)
          end if
       else
          write(fug,*) "Unkown option",line,"."
       end if

       read(fopt,*,iostat=ierr) line
    end do 

    ! ---- Close the input file ----
    close(fopt)

    ! ---- Check options ----
    if (atype .eq. 1) then
       
    else if ( atype .eq. 2) then
       if (basismethod .eq. 0) then
          write(*,*) "You must specify a basis type to calculate a weighted mean"
          stop
       end if
       if(.not. allocated(gamma)) then
          write(*,*) "You must specify gammas to calculate a weighted mean"
          stop
       end if
    end if
  end subroutine readinputs


  ! ========== readbasistype =========================
  !  Subroutine to readin details of basis type.
  !  From simulation program but only handles 
  !  harmonic oscillator basis functions.
  ! ==================================================
  subroutine readbasistype(basistype)
    use errors

    implicit none
    character(*), intent(in) :: basistype
    integer :: ierr
    integer :: nbf ! number of basis functions
    integer :: i,j
    integer :: superid,nhb
    CHARACTER(len=50) :: line
    
    real(kind=8) :: rzoffset,izoffset ! for reading in origin offset

    basismethod = 0

    if (basistype == "harmosc") then
       basismethod = 4
       ! Do some initialisation
       allocate(harmmethod(ndf))
       allocate(vstate(ndf))
       allocate(harmComp(ndf))

       harmComp(:) = 1.0d0
       nsupermodes = 0

       read(fopt,*,iostat=ierr) line
       do while (line .ne. "vibstates" .and. ierr.eq.0)
          backspace(fopt)
          if( line.eq."zmax") then
             ! Don't need this here
             read(fopt,*,iostat=ierr) line
          else if ( line .eq. "probscale") then
             ! Don't need this here
             read(fopt,*,iostat=ierr) line

          else if  ( line .eq. "maxtries") then
             ! Don't need this here
              read(fopt,*,iostat=ierr) line


          else if (line .eq. "super") then
             ! ---- allow modes to be in a super position ----
             !      of harmonic oscillator states 
             read(fopt,*) line,nsupermodes,nsuperbasis
             allocate(superphi(nsuperbasis,nsupermodes))
             superphi(:,:) = 0.0d0

          else
             write(fug,*) "Error: Unrecognised option in harmonic basis:",line
             stop
          end if
          read(fopt,*,iostat=ierr) line
       end do

       ! Now read in the vibrational states of each mode
       ! 0 - put at origin
       ! 1,2 harmonic oscillator at origin and offset
       ! 3,4 as 1,2 but using basis functions from initial states.
       ! 5,6 as 1,2 but where the state is a superposition of harmonic states
       ! 7,8 as 3,4 but where the state is a superposition of harmonic states 

       superid = 0 ! For superpositions.

       do i = 1,ndf

          read(fopt,*,iostat=ierr) harmmethod(i) 

          if (harmmethod(i) .eq. 1.or. harmmethod(i) .eq. 3) then
             backspace(fopt) 
             read(fopt,*,iostat=ierr) harmmethod(i),vstate(i),harmComp(i)

             if(ierr .ne. 0) then
                write(fug,*) "Error reading in state and compression for ", &
                     i," degree of freedom."
                stop
             end if

             ! Check to see if vstate is < 0
             !if (vstate(i) < 0  .and. vstate(i) > -100) then
          else if (harmmethod(i) .eq. 2.or. harmmethod(i) .eq. 4 ) then
             ! Only allocate the array if it is needed.
             if(.not.allocated(zoffset)) then
                allocate(zoffset(ndf))
                zoffset(:) = (0.0,0.0)
             end if
             backspace(fopt)
             read(fopt,*,iostat=ierr) harmmethod(i),vstate(i),harmComp(i),rzoffset,izoffset
             if(ierr .ne. 0) then
                write(fug,*) "Error reading in state, compression and offset  ", &
                     i," degree of freedom."
                stop
             end if
             zoffset(i) =  rzoffset+(0,1)*izoffset

             ! set vibrational state to be a superposition of harmonic oscillator modes
          else if (harmmethod(i) .eq. 7 ) then
             if (nsupermodes .eq. 0 ) then
                write(fug,*) "Can't readin a wave function superposition as the option super has not been set."
                stop
             end if

             superid = superid + 1
             if (superid .gt. nsupermodes ) then
                write(fug,*) "Too many wavefunctions superpositions. Change the super statement at top of basis specification."
                stop
             end if

             backspace(fopt)
             read(fopt,*,iostat=ierr) harmmethod(i), nhb
             backspace(fopt)
             read(fopt,*,iostat=ierr) harmmethod(i), nhb,(superphi(j,superid),j=1,nhb)
          else if (harmmethod(i) .eq. 8 ) then
             ! Only allocate the array if it is needed.
             if(.not.allocated(zoffset)) then
                allocate(zoffset(ndf))
                zoffset(:) = (0.0,0.0)
             end if

             if (nsupermodes .eq. 0 ) then
                write(fug,*) "Can't readin a wave function superposition as the option super has not been set."
                stop
             end if

             superid = superid + 1
             if (superid .gt. nsupermodes ) then
                write(fug,*) "Too many wavefunctions superpositions. Change the super statement at top of basis specification."
                stop
             end if

             backspace(fopt)
             read(fopt,*,iostat=ierr) harmmethod(i),rzoffset,izoffset, nhb
             backspace(fopt)
             read(fopt,*,iostat=ierr) harmmethod(i),rzoffset,izoffset, nhb,(superphi(j,superid),j=1,nhb)


          else if (harmmethod(i) .ne. 0) then
             write(fug,*) "Unknown harmonic basis set method ",harmmethod(i)
             write(fug,*) "degree of freedom number ",i
             stop
          end if
          
       end do
       write(fug,*) "Read basis method information"
    else
       write(fug,*) " I don't know how to handle basis type", basistype
    end if


  end subroutine readbasistype

  subroutine help
    implicit none

    write (*,*) "      energy average"
    write(*,*) 
    write(*,*) " Requires an input file called energyaverage.inpt"
    write(*,*) " Reads in energy.out files from sequentially named"
    write(*,*) " directories. Weighted means also require states.inpt"
    write(*,*) " files."
    write(*,*) " This energyaverage.inpt file should contain the following."
    write(*,*)
    write(*,*)  " atype type = type may be either mean or weighted"
    write(*,*)  " dirstem Run = first part of directory name."
    write(*,*)  "nruns  46 = Number of different runs e.g. Run1 -> Run46"
    write(*,*)  "nsteps 10000 = number of entries in energy.out"
    write(*,*)  "ndf 9 = number of degrees of freedom. "
    write(*,*)   "       Note: the energy files should contain 2*ndf+3 columns"
    write(*,*)
    write(*,*)   "gamma = a list of gammas as per main ccs code. To be followed by ndf lines n gamma."
    write(*,*)   "basis harmosc = harmonic basis. Needed for the weighted mean. Please see man ccs code manual for details"
    

  end subroutine help
end module eamodule




program energyaverage
  use eamodule
  use errors

  implicit none

  integer :: ierr
  character(len=20) :: runnum
  character(len=150) :: fullname
  character(len=50) :: stuff
  character(len=20) :: fmt

  real(kind=8),dimension(:),allocatable :: values
  real(kind=8) :: tzero,tone,dt,thistzero,thistone

  integer :: run,s,i ! for loops

  ! ---- Open out file ----
  open(fug,file="energyaverage.out",iostat=ierr)
  if(ierr .ne. 0 ) then
     write(*,*) "No such file"
     call fileerror(ierr,"energyaverage.out",0) ! stderr?
  end if

  ! ---- read inputs to find out what we are going to do ----
  call readinputs

  ! ---- set initial probability sum to zero ----
  sumprob = 0.0d0

  ! ---- Allocate memory ----
  allocate(meanenergy(2*ndf+1,nsteps))
  if (.not.allocated(meanenergy)) then
     write(fug,*) "Problem allocating memory. Try using fewer steps"
  end if
  
  meanenergy(:,:) = 0.0d0
  allocate(values(2*ndf+3)) ! For reading in energies
  
 
  write(fug,*) "Entering main loop"
  readruns: do run = 1,nruns
     ! work out file name
     
     write(fug,*) "Run ",run
     write(runnum,"(i10)") run
     !     modenumber="1"
       
     fullname = trim(dirstem) // trim(ADJUSTL(runnum)) // "/" // energyfile
     open(fengin,file=fullname,status="old",iostat=ierr)
     
     
!     if (atype .eq. 1) then
        
!     else
     if (atype .eq. 2) then
        
        fullname = trim(dirstem) // trim(ADJUSTL(runnum)) // "/" // statesfile
        call readinitstates(fullname)
       
        call calcprob

        write(fug,*) "Probability ",prob
        sumprob = sumprob+prob
     end if

     readenergies: do s = 1,nsteps
        read (fengin,*) (values(i),i=1,2*ndf+3)

        ! ---- Work out time/check step ----
        if (run .eq. 1 .and. s .eq. 1) then
           tzero = values(1)
        else  if (run .eq. 1 .and. s .eq. 2) then
           tone = values(1)          
           dt = tone-tzero
           write(fug,*) "dt = " ,dt
        else if (run.gt. 1 .and. s .eq. 1) then
           thistzero = values(1)
        else  if (run .gt. 1 .and. s .eq. 2) then
           thistone = values(1)
           ! Check that it is the same as the first
           if ( dabs(dt - ( thistone-thistzero)) .gt. 1e-5 ) then
              write(fug,*) "The time step in run ",run," is different"
              write(fug,*) dt, " =/= " ,thistone-thistzero
              stop
           end if
        end if

        ! ---- Do the averaging ----
        if (atype .eq. 1) then
           meanenergy(1,s) =  meanenergy(1,s) + values(2)
           if (ndf .gt. 0) then
              meanenergy(2:2*ndf+1,s) =meanenergy(2:2*ndf+1,s)+ values(4:2*ndf+3)
           end if
        else
           meanenergy(1,s) =  meanenergy(1,s) + prob*values(2)
           if (ndf .gt. 0) then
              meanenergy(2:2*ndf+1,s) =meanenergy(2:2*ndf+1,s)+prob*values(4:2*ndf+3)
           end if
        end if

     end do readenergies
  end do readruns

  
  if (atype .eq. 1) then
     sumprob = dble(nruns)
  end if

  write(fug,*) "Total Q = sum(prob) = ",sumprob
    
  ! ---- Now  write energies ----
  open (fengout,file="meanenergy.out")
  if (atype .eq. 1 ) then
     write(fengout,*) "# Mean energy calculated over ",nruns," runs"
  else if (atype .eq. 2 ) then
     write(fengout,*) "# Weighted mean energy calculated over ",nruns," runs"
  end if

  write(fmt,'("(",i4,"es18.8)")') 2*ndf+2
  
  writeenergies: do s = 1,nsteps
     write(fengout,fmt ) (s-1)*dt,(meanenergy(i,s)/sumprob,i=1,2*ndf+1)
  end do writeenergies
end program energyaverage
