! **************************************************
!                    initstates
!
!                  Stewart K. Reed
!                   5th June 2007
!
!  Description:
!      This file contains routines to read in /
!      setup initial states.
! **************************************************

subroutine readinitstates(filename)
  use eamodule
  use errors

  implicit none
  character(*),intent(in) :: filename
  character(50) ::line
  real(kind=8) :: q,p ! Initial position and momentum
  integer :: i,j ! For loops
  integer :: ierr ! File I/O errors

  ! Open the file
  open(fis,file=filename,status='old',iostat=ierr)
  if(ierr .ne. 0 ) then
     call fileerror(ierr,filename,fug)
  end if

  ! How many states are given in the file?
  read(fis,*,iostat=ierr) line
  if  (line == "nstates" ) then
     backspace(fis) 
     read(fis,*,iostat=ierr) line,ninitstates
     if(ierr.ne.0) call filereaderror(ierr,filename,"Error reading number of states",fug)
  else
     write(fug,*) "Failed to read the number of states in ",trim(filename)
     stop
  end if


  ! Now allocate the arrays
!  if(allocated(zinit)) then
!     write(fug,*) "Ahhhh....zinit is already allocated in readinitstates!"
!     stop
!  end if
  if (.not. allocated(zinit) )then
     allocate(zinit(ndf,ninitstates))
  end if
     zinit(:,:) = (0.0,0.0)

  do i = 1, ninitstates
     read(fis,*,iostat=ierr) line
     if ( line .ne. "state".or.ierr .ne. 0) then
        write(fug,*) "Error reading state keyword ",i," in ",trim(filename)
        write(fug,*) "Actually read:",line
        stop
     end if
     do j = 1,ndf
        read(fis,*,iostat=ierr) q,p
        if ( ierr .ne. 0) then
           write(fug,*) "Error reading position/momentum ",j," state", &
                i," in ",trim(filename)
           stop
        end if
        ! z = sqrt(gamma(j)/2)*q+ i*p/sqrt(2*gamma(j))
        zinit(j,i) = q*shgamma(j)+(0,1)*p*s2gammarec(j)
     end do
  end do

  ! Close the file
  close(fis)
  
  ! Summarise
  write(fug,*)
  write(fug,*) "Initial States"
  write(fug,*) "--------------"
  do i = 1,ninitstates
     write(fug,*) "  State",i
     do j = 1,ndf
        write(fug,*) "         z",j,zinit(j,i)
     end do
  end do

end subroutine readinitstates
