#!/usr/bin/perl
# 

#use strict;
use warnings; 

sub help
{

    print <<END_OF_TEXT;
       average
    23rd January 2006
      S.K. Reed

Last modified: 29th April 2009.

A wee script to average density profiles. The density profiles each
 need to contain the line
# Average over n steps 
where n is the number of steps overwhich the profile was created. 


Usage:
    densityav [ -o OUTFILE ] [options] INPUTFILES
  where:
    -o OUTFILE specifies an output file
    -e         requests the standard deviation to be determined. 
               Doesnot include 1/sqrt(N).
    -s         Calculate the weighted difference in profiles.
               Assumes the first in the list is the later profile.
	       I.e. calculates (N1  * rho1 - N2 * rho2) /(N1-N2)
	       where rho1 is the density profile given first, and 
	       rho2 the profile second in the list. 
	       Errors might be (probably are?) rubbish!
    -w         Calculate a weighted sum. The weights are given by 
               the number of steps. This may be a real number.
    -max m     Max xrange to consider. I.e. ignore x values greater
               than m.

Based on a program for averaging histograms.

For this program to work it is necessary to add a line to the header of every density profile:
# Average over x steps
where x is the number of steps over which the average was calculated. 

Should also work for rdfs.

END_OF_TEXT
    
}

# ============= Variables ==================
my $outfilename;  # Name of file to write averaged histogram to
my $OUTFILE;      # File handle
my $infilename;   # Name of file to read from
my $INFILE;       # File handle
my $numdatafiles; # How histograms are we averaging over?
my $numcomments=0;# How many lines of comments in first data file
my @comments;     # Comments from the first file
my $i;            # for loops

my $numbins=0;    # Number of bins in the histogram
my $bin;          # index for loop
my $N;            # Number of data in each file
my $numsteps;      # Total number of data averaged over
my $totalnumsteps;

my (@z,@rho);    # Arrays for the z position and density.
my @sigmarho;    # The standard deviation.
my $line;         # For reading in lines from a file
my @words;        # For splitting lines
my $numcols;       # How many columns in the density profile
my ($scale1,$scale2); # Scale factors for calculating averages.

my $calcerrors;
my $diff; # Calculate the difference in density profiles
          # errors are probably rubbish


my $weightedsum; # calculate a weighted sum rather than 
                 # the mean.
my $limitx;      # Limit x values to be less than
my $maxx;        # maxx

my $havenumberofsteps;  # Get rid of the requirement for the number of steps comment line.
                        # Defaults to 1. 

# ---- Check to see how many arguments have been give ----
my $numargs = @ARGV;
if ($numargs <2 ){
    &help;
    exit;
}

# ---- Set a couple of defaults ----
$calcerrors = 0; # Don't calculate errors by default
$outfilename = "STDOUT";  # write average density to the STDOUT
$weightedsum = 0;

# ---- Check command line arguments for options ----
while ($ARGV[0] =~ /^-/){
  
  # Get name for output file
  if ($ARGV[0] eq "-o" ){ 
    $_ = shift(@ARGV);
    $outfilename = shift(@ARGV);
    
    # Print standard deviations
  }elsif ($ARGV[0] eq "-e") {
    $_ = shift(@ARGV);
    $calcerrors = 1;
    
  }elsif ($ARGV[0] eq "-s") {
    $_ = shift(@ARGV);
    $diff = 1;   
  }elsif ($ARGV[0] eq "-w") {
    $_ = shift(@ARGV);
    $weightedsum = 1;
  }elsif ($ARGV[0] eq "-max") {
    $_ = shift(@ARGV);
    $limitx = 1;
    $maxx = shift(@ARGV);
   
    # Print the help message
  }elsif ($ARGV[0] eq "-h" || $ARGV[0] eq "-help" || $ARGV[0] eq "--help") {
    &help;
    exit;
	
    # Some other option which we do not recognise.
  }else{
    &help;
    print "\n";
    print "I do not know what to do with $ARGV[0]\n";
    exit;
  }
  
}

# How many data files are there?
$numdatafiles = @ARGV;

# Get name of first data file
$infilename = shift(@ARGV);

#Open first file
if (!open($INFILE, $infilename)) {
    die "Dying! Can't open ", $infilename;
}


$totalnumsteps = 0;
$havenumberofsteps = 0; 
#Read in first file 
while ($line = <$INFILE>){
  # Read in comments at the top.
  if ($line =~ /^#/){
    # Count the number of comments
    $numcomments++;
    chomp($line);
    $comments[$numcomments] = $line;
    # Over how many steps was this density distribution averaged?
    if ($line =~ /Average over/){
      @words  = split(/\s+/, $line);
      $numsteps = $words[3];
      $totalnumsteps = $numsteps;
      $havenumberofsteps = 1;
    }
  }else{
    if ($limitx == 0 || $z[$numbins] < $maxx){
      # Read in data and store in an array
      $numbins++;
      
      @words = split(/\s+/, $line);
      shift(@words) if ($words[0] eq "");
      
      # How many columns are there in the file
      if ($numbins == 1) {
	$numcols = @words;
	print " There are $numcols columns of data in your file\n";
      }
      
      # Store the z position in an array
      $z[$numbins] = $words[0];
      
      foreach $i ( 1..$numcols-1){
	$sigmarho[$numbins][$i] = 0;
	
	$rho[$numbins][$i] = $words[$i];
      }
    }
  }
}# While reading in first data file

# Haas the number of steps been read in? If not assume it to be 1.
if ($havenumberofsteps == 0) {
	$totalnumsteps = 1;
}

# Read the other histograms
foreach $infilename (@ARGV){
    
  #Open first file
  if (!open($INFILE, $infilename)) {
    die "Dying! Can't open ", $infilename;  
  }       
  
  $bin = 0;

  # Is there a line specifying the number of steps?
  $havenumberofsteps = 0; 	
  
  while ($line = <$INFILE>){
    # Read in comments at the top.
    if ($line =~ /^#/){
      if ($line =~ /Average over/){
	@words  = split(/\s+/, $line);
	$numsteps = $words[3];
	$numsteps = -1*$numsteps if ($diff);
	$havenumberofsteps = 1;
      }
    }else{
      if ($limitx == 0 || $z[$numbins] < $maxx){
	# Read in data and store in an array
	$bin++;
	@words = split(/\s+/, $line);
	shift(@words) if ( $words[0] eq "");
	
	die "z values not equal ($z[$bin],  $words[0] )\n " if ($z[$bin]  != $words[0]);
	#Check for the number of columns
	if ($bin == 1){
	  die "Dying: There is a different numbers of species in $infilename" if (@words != $numcols);
	}
	# read in the density from each columns
	if ($weightedsum == 1) { # If we just want a weighted sum then set these scale values to 1.
	  $scale1 = 1;
	  $scale2 = 1;
	}else{
	  $scale1 = $totalnumsteps/($totalnumsteps+$numsteps);
	  $scale2 = $numsteps/($totalnumsteps+$numsteps);
	}
	foreach $i ( 1..$numcols-1){
	  $sigmarho[$bin][$i] = 
	    $scale1*( $sigmarho[$bin][$i] +
		      ( $words[$i]-$rho[$bin][$i])*( $words[$i]-$rho[$bin][$i])*$scale2);
	  
	  $rho[$bin][$i] =  $rho[$bin][$i]*$scale1   
	    + $words[$i] * $scale2;
	}
      
      }
    }
  }
  
  # Has the number of steps been read in? If not assume it to be 1.
   if ($havenumberofsteps == 0) {
	  $totalnumsteps = $totalnumsteps + 1;
   }else{
      $totalnumsteps = $totalnumsteps + $numsteps;
   }
}

# To be useful, I suppose we should write out an averaged histogram!

if ($outfilename eq "STDOUT") {
    $OUTFILE = "STDOUT";
}else{
    if (!open($OUTFILE,"> $outfilename")) {
	die "Dying! Can't open ", $outfilename," to write to\n";
    }
}

# File header
if (!$diff){
    print $OUTFILE "# Average of $numdatafiles histograms\n";
}else{
    print $OUTFILE "# difference between $numdatafiles density profiles\n";
}
if ($calcerrors){
    print $OUTFILE "# Includes standard deviation of each bin.\n";
}
foreach $i (1..$numcomments){
  if ($comments[$i] =~ /Average over/){
    print $OUTFILE "# Average over $totalnumsteps steps\n";
  }else{
    print  $OUTFILE "$comments[$i]\n";
  }
}

# Now write out the actually density profiles
foreach $bin (1..$numbins){
  print $OUTFILE "$z[$bin]\t";
  foreach $i ( 1..$numcols-1) {
      if ($calcerrors) {
	  print $OUTFILE "$rho[$bin][$i] ",sqrt($sigmarho[$bin][$i])," \t";
      }else{
	  print $OUTFILE "$rho[$bin][$i]\t";
      }
  }
  print $OUTFILE "\n";
}




