!!% ************************************************************************
!!%  Program: datahistogram
!!%  Author:  Stewart Reed
!!%  Date:    11th June 2009
!!%
!!%  Description:
!!%   A simple program which makes use of the histogram module to create
!!%   a histogram from a column of data.
!!%
!!%   Currently, it will only calculate the histogram for a single column 
!!%   of data. It would however be very easy to extend it to do two columns:
!!%   one for the position and the other for the weight.
!!% ************************************************************************

program datahistogram
  use histogram
  use errors
  implicit none

  ! ---- local variables ----
  character(len=100) :: datafile      ! name of file containing data
  character(len=100) :: histogramfile ! name of file to write data to
  integer :: fdata = 10               ! File units for the data file
  integer :: ierr                     ! for file errors
  integer :: col  ! column containing the data to be made into a histogram
  character(len=100) :: line          ! For reading in stuff
  character(len=200) :: comment       ! comment for histogram file

  ! Histogram details
  real(kind=8) :: min,max
  integer :: nbins
  type(histogram1D) :: hist           ! The histogram
  real(kind=8) :: x

  integer :: j


  ! ---- Get name and open data file ----
  write(*,*) "Please enter the name of the file containing the data"
  read(*,*) datafile
  open(unit=fdata,file=datafile,iostat=ierr)
  if (ierr.ne.0) then
     call  fileerror(ierr,datafile)
     stop
  else
     write(*,*) "Opened the data file ",trim(datafile)
  end if

  ! ---- determine which column contains the data ----
  write(*,*) "Which column of data do you want to create a histogram from?"
  read(*,*) col
  write(*,*) "Using column ",col

  ! ---- Get details of the desired histogram ----
  write(*,*) "Please enter the minimum histogram value"
  read(*,*) min
  write(*,*) " the maximum value"
  read(*,*) max
  write(*,*) " and the number of bins required"
  read(*,*) nbins
  write(*,*) "What should the file containing the histogram be called?"
  read(*,*) histogramfile
  
  ! create histogram
  call createhistogram(hist,min,max,nbins)


  ! Now read in the data
  ierr = 0
  read(fdata,*,iostat=ierr)  line

  do while(ierr == 0) 
     if (line(1:1) == "#") then
        ! We have a comment
     else
        backspace(fdata)
        read(fdata,*,iostat=ierr) ( line,j = 1,col-1),x
        if (ierr .ne. 0) then
           call filereaderror(ierr,"reading data","Failed to read data from the requested column")
        else
           ! Add to the histogram 
           call addtohistogram(hist,x,1.0d0)
        end if
     end if
     
     ! Try to read the next line
     read(fdata,*,iostat=ierr)  line
  end do

  ! Normalise the histogram
  call normalisehistogram(hist)
  
  ! Write it out
  write (*,*) "Writing histogram to ",histogramfile
  write(line,*) col
  comment = "Histogram from column "//trim(adjustl(line))//" of "//trim(datafile)
  call writehistogram(hist,histogramfile,comment)
  write(*,*) "Done. Good bye."
end program datahistogram
