!  errors.f90
!  28th November 2005
!  S. Reed 
!
! A simple subroutine to deal with errors in opening files.
! Gives an error message and stops the program

module errors

contains

  subroutine fileerror(ierr,filename,fu)
    implicit none
    ! Get the error number and filename from the calling subroutine
    integer, intent(in)      :: ierr
    character(*), intent(in) :: filename
    integer, intent(in),optional :: fu
   
    integer :: fug ! file unit for writing message to.
    if (present(fu)) then
       fug = fu
    else
       fug = 6
    end if

    ! Give a specific message if we know what the error number means
    ! if not, give a message including the error number and stop.


    ! File already exists but was required to be new
    if (ierr == 10) then
       write(fug,*) "Fatal error: The file ",trim(filename)," file already exists."

       ! File doesn't exist but should
    else if (ierr == 29) then
       write(fug,*) "Fatal error: The file ",trim(filename)," does not exist so I can't open it!" 

       ! Another error whose meaning we don't know
    else 
       write(fug,*) "Fatal error, ",ierr,": Problem opening the file ",trim(filename)
    end if

    stop

  end subroutine fileerror





  subroutine filereaderror(ierr,errlocation,message,fu)
    implicit none
    integer, intent(in) :: ierr
    character(*),intent(in) :: errlocation
    character(*), intent(in) :: message
    integer, intent(in),optional :: fu
   
    integer :: fug ! file unit for writing message to.
    if (present(fu)) then
       fug = fu
    else
       fug = 6
    end if

    if (ierr == -1) then
       write(fug,*) "Fatal error: Reached the end of file whilst reading from ",&
	trim(errlocation),"."
       stop
    else     
       write(fug,*) "Fatal error number ",ierr," in ",trim(errlocation)
    end if

    write(fug,*) "  ",trim(message)
    stop
  end subroutine filereaderror

end module errors
