
module stuff

  integer :: ndf
  integer :: nstates

  ! The states
  real(kind=8),dimension(:,:),allocatable :: statesq,statesp
  character(len=100) :: statesfile
  integer :: fis=10 ! fie unit

  
  character(len=100) :: stem
  integer :: offsetn
  integer :: fnstate=20

  ! The reordered states
  integer :: reordermode ! which state to reorder
  integer,dimension(:,:),allocatable :: orders
  integer  :: norders
  
  contains
    subroutine readstates
      
      implicit none
      integer ::ierr
      character(len=50) :: line ! For reading text from file

      integer :: bf,s ! basis function, state for loops

      ! Open the file
      open(fis,file=statesfile,status='old',iostat=ierr)
      if(ierr .ne. 0 ) then
         write(*,*) "Failed to open states file, ",statesfile
         stop
      end if

      ! find out how many states are in the file
      read(fis,*) line,nstates

      ! allocate states array
      allocate(statesq(ndf,nstates))
      allocate(statesp(ndf,nstates))

      ! Read in the initial states
      do s = 1,nstates
          read(fis,*) line
         do bf = 1,ndf
            read(fis,*)statesq(bf,s),statesp(bf,s)
         end do
      end do

      close(fis)
    end subroutine readstates

    subroutine createrandomlist(n,order)
      implicit none

      integer,intent(in) :: n
      integer,dimension(n),intent(out) :: order

      integer :: i,j,rc,choice
      real(kind=8) :: r ! random number

      logical,dimension(n) :: chosen
      chosen(:) = .false.

      do i = 1,n
         call random_number(r)
         r = r*dble(n+1-i)
         
         rc = floor(r)+1

         
         choice = 0
         j = 0
         do while(choice .lt. rc .and. j .lt. n)
            j = j+1
            if (.not. chosen(j)) choice = choice + 1
         end do

         order(i) = j
         chosen(j) = .true.
!debug         write(*,*) n+1-i, rc,choice
      end do
  
 !debug     write(*,*) order(:)
    end subroutine createrandomlist


    !
    ! A subroutine to compare the norder order to those before it.
    ! returns 0 if different to all otherwise returns an integer
    ! corresponding to which order it is equal to.
    !
    function compareorders(no)
      implicit none
      integer, intent(in) :: no
      integer :: compareorders

      integer :: o,s ! order and state for loops

      compareorders = 0

      orderlist: do o = 1,no-1
         s = 1
         do while (orders(s,o) .eq. orders(s,no) .and. s .le. nstates)
            s = 1+1
         end  do
         if (s .eq. nstates+1) then
            compareorders = o
            exit orderlist
         end if
      end do orderlist
         
    end function compareorders

    
end module stuff


program recombinestates
  
  use stuff
  
  implicit none

  integer :: i,s,bf
  integer :: differentorder

  character(len=15) :: statenum

  character(len=100) :: filename


  write(*,*) "Enter states file"
  read(*,"(a100)") statesfile

  ndf = 9
  write(*,*) "ndf = ",ndf

  reordermode = 9
  write(*,*) "Reordering states in mode ",reordermode
  call readstates()

  write(*,*) "How many different orders do you want?"
  read(*,*) norders

  
  stem="pstate"
  offsetn=0
  ! allocate an array to hold the different orders
  allocate(orders(nstates,norders))

  ! We have the first order
!  do i = 1,nstates
!     orders(i,1) = i
!  end  do


  do i = 1,norders
     differentorder = 1
     do while (differentorder .ne. 0) 
        write(*,*) " Generating order ",i
        
        call createrandomlist(nstates,orders(:,i))
        if (i > 1 ) then
           differentorder = compareorders(i)
        else
           differentorder = 0
        end if
        
     end do

     write(*,*) "Order ",i
     write(*,*) orders(:,i)

     !write out new states file
     write(statenum,*) i+offsetn
     filename = trim(stem) //adjustl(statenum)
     write(*,*) filename
     open(fnstate,file=filename)
     
     write(fnstate,*) "nstates",nstates
     do s = 1,nstates
        write(fnstate,*) "state",s
        do bf = 1,ndf
           if (bf .eq. reordermode) then
              write(fnstate,*) statesq(bf,orders(s,i)),statesp(bf,orders(s,i))
           else
              write(fnstate,*) statesq(bf,s),statesp(bf,s)
           end if
        end do
     end do
     close(fnstate)
  end do



end program recombinestates
