module data

  implicit none

  ! a switch to determine how the program works
  integer :: trajmode 
  logical :: randomiselog

  character(len=100) :: dirstem
  character(len=100) :: ampfilenames
  character(len=100) :: basisfile
  character(len=100) :: controlfilename

  ! which columns are the positions and momenta in
  integer :: qcol,pcol 
  integer :: firstcol

  ! number of modes
  integer :: nmodes

  ! number of configs -> number of basis functions
  integer :: nbasis

  ! file units
  integer :: fcont=10
  integer :: foutput=11
  integer :: fmodesoffset=50
  integer,dimension(:),allocatable :: fmodes
  
  ! -- gammas --
  real(kind=8),dimension(:),allocatable :: gamma

  ! -- rsigma -- 
  ! width of distribution for perturbation
  real(kind=8),dimension(:),allocatable :: rsigma
  

  real(kind=8),parameter ::pi     = 3.1415926535897932385d0
contains
  subroutine readinputs
    implicit none

    character(len=100) :: stuff  ! 
    integer :: i,s                 ! for loops
    integer :: ierr 
    
    real(kind=8) :: rsscale ! change the randomise distribution widths

    ! set some defaults
    trajmode = 1
    nmodes = 0
    nbasis = 0
    dirstem = ""
    ampfilenames = "amplitudes.out"
    basisfile = "trainbasisstates"
    qcol = 4
    pcol = 5
    firstcol = 4

    randomiselog = .false.
    rsscale = 1.0d0

    read(fcont,*,iostat=ierr) stuff

    ! Check the options in the control file
    do while (ierr .eq.0)
        if (trim(stuff) .eq. "trajmode") then
          backspace(fcont)
          read(fcont,*) stuff,stuff
          if (trim(stuff) .eq. "separate" ) then
             trajmode = 1
          else if (trim(stuff) .eq. "single") then
             trajmode = 2
          else
             write (*,*) "Trajectory mode not recognised", stuff
             stop
          end if

       else if (trim(stuff) .eq. "nmodes") then
          backspace(fcont)
          read(fcont,*) stuff,nmodes

       else if (trim(stuff) .eq. "steps") then
          backspace(fcont)
          read(fcont,*) stuff,nbasis

       else if (trim(stuff).eq."dirstem") then
          backspace(fcont)
          read(fcont,*) stuff,dirstem
          
       else if (trim(stuff).eq."ampfile") then
          backspace(fcont)
          read(fcont,*) stuff,ampfilenames

         else if (trim(stuff).eq."basisfile") then
          backspace(fcont)
          read(fcont,*) stuff, basisfile
          
       else if (trim(stuff) .eq. "qcol") then
          backspace(fcont)
          read(fcont,*) stuff,qcol

       else if (trim(stuff) .eq. "pcol") then
          backspace(fcont)
          read(fcont,*) stuff,pcol

       else if (trim(stuff).eq."firstcol") then
          backspace(fcont)
          read(fcont,*) stuff,firstcol

       else if (trim(stuff).eq."randomise") then
          ! ---- randomise zs? ----
          randomiselog = .true.
          write(*,*) "Will randomise (q,p)"
          allocate(rsigma(nmodes))
          if(allocated(gamma)) then
             rsigma(:) = 1/gamma(:)
          else
             rsigma(:) = 1.0d0
          end if

       else if (trim(stuff).eq."rsscale") then
          backspace(fcont)
          read(fcont,*) stuff,rsscale

          ! ---- gammas ----
       else if (stuff == "gamma") then
          write(*,*) "Reading gamma"
          if(nmodes .eq. 0 ) then
             write(*,*) &
                  " Need to specify ndf before specifying gamma"
             stop
          else
             allocate(gamma(nmodes))
             do i = 1,nmodes
                read(fcont,*,iostat=ierr) s,gamma(i)
                if(ierr.ne.0) then
                   write(*,*) "state: ",i
                   write(*,*) "Error reading gamma",i
                end if
             end do
          end if
          
       end if
       read(fcont,*,iostat=ierr) stuff
           
    end do
       
    ! scale the distribution widths for the randomisation
    if(allocated(rsigma)) then
       rsigma(:) = rsscale*rsigma(:)
    end if

    ! test to see if the number of modes and number of 
    ! degrees of freedom have been set
    if (nmodes .eq. 0) then
       write(*,*) "You need to specify the number of modes"       
       stop
    else if (nbasis .eq.0) then
       write(*,*) "You need to specify the number of steps"       
       stop
    end if


  end subroutine readinputs
  
  subroutine help
    
    implicit none
    
    write (*,*) "The control file,",controlfilename," must contain at least"
    write(*,*) "  nmodes <n>"
    write(*,*) "  nsteps <ns>"
    write(*,*) "and possibly "
    write(*,*) "  trajmode <mode>  can be either separate or single. Separate corresponds to"
    write(*,*) "                  each mode having a separate file in its own sequentially"
    write(*,*) "                  numbered directory."
    write(*,*) "                  Single describes the case when all modes are in one file."
    write(*,*) 
    write(*,*) "Other entries may be:"
    write(*,*) "  dirstem <stem>  Default: current directory."
    write(*,*) "  ampfile <name>  Name of file containing qs and ps. Default: amplitudes.out"
    write(*,*) "  qcol <c>      Column containing q for separate mode. Default: 4."
    write(*,*) "  qcol <c>      Column containing p for separate mode. Default: 5."
    write(*,*) "  firstcol <c>  Column of first q for single mode. Default: 4."
<<<<<<< .mine
    write(*,*) "  randomise     add a random delta q and delta p to each state"
    write(*,*) "                perturbations are drawn from a circular gaussian"
    write(*,*) "                probability distribution for z"
    write(*,*) "  rscale  s     Scale the widths of the aforementioned distributions."
    write(*,*) "                Tthe widths are s/gamma(i)."    
=======
    write(*,*) "  randomise     add a random delta q and delta p to each state"
    write(*,*) "                perturbations are drawn from a circular gaussian"
    write(*,*) "                probability distribution for z"
    write(*,*) "  rscale  s     Scale the widths of the aforementioned distributions."
    write(*,*) "                The widths are s/gamma(i)."    
>>>>>>> .r23
    
  end subroutine help
  
  
  ! ******************** separatemode *****************
  !  create trajectory from several runs each with 1 
  !  basis function in 1 mode.
  ! ***************************************************
  subroutine separatemodes
    
    integer :: i,j,ns  ! for loops
    integer :: ierr   ! for errors
    
    character(len=15)   :: modenumber
    character(len=150) :: fullname
    
    real,dimension(:),allocatable :: values
    integer :: dimvalues ! size of values
    
    character(len=100) :: stuff  ! 
    
    complex(kind=8) :: dz
    real(kind=8) :: dq,dp

    !assign file units for modes
    allocate(fmodes(nmodes))
    do i = 1,nmodes
       fmodes(i) = fmodesoffset+i
       
       write(modenumber,*) i
       !     modenumber="1"
       
       fullname = trim(dirstem) // trim(ADJUSTL(modenumber)) // "/" // ampfilenames
       open(fmodes(i),file=fullname,status="old",iostat=ierr)
       
       if (ierr.ne.0) then
          write(*,*) "There was a problem opening file ",trim(fullname)
          stop
       else
          write(*,*) "Opened ",trim(fullname)
       end if
       read(fmodes(i),*) stuff
       
    end do
    
    
    ! Create an array to read numbers in from the files
    if (pcol > qcol) then
       dimvalues = pcol
    else
       dimvalues = qcol
       
    end if
    allocate(values(dimvalues))
    
    ! Open output file
    open(foutput,file=basisfile)
    write(foutput,*) "nstates",nbasis
    
    !Now we want to read in the basis functions
    numstates: do ns = 1,nbasis
       write(foutput,*) "state ",ns
       nummodes: do i = 1,nmodes
          ! Read in two comment lines
          ! Only need to read in one as the other is blank
          ! it is automatically skipped
          read(fmodes(i),*) stuff 
          
          !	 read(fmodes(i),*) values(1),values(2),values(3),values(4),values(5)
          !(values(j),j=1,dimvalues)
          
          read(fmodes(i),*,iostat=ierr) (values(j),j=1,dimvalues)
          if(ierr .ne. 0) then
             write(*,*) "Error reading mode ",i, " step ",ns
             write(*,*) "Error number ", ierr
             stop
          end if
         
          ! Change the positions slightly?
          if(randomiselog) then
             dz = deltaz(rsigma(i))
             dq = dsqrt(2.0d0/gamma(i))*real(dz)
             dp = dsqrt(2.0d0*gamma(i))*dimag(dz)
             values(qcol)= values(qcol) + dq
             values(pcol)= values(pcol) + dp
          end if


          write(foutput,*) values(qcol),values(pcol)
       end do nummodes
    end do numstates
    deallocate(values)
  end subroutine separatemodes
  

  ! ******************** singlemode *******************
  !  create trajectory from a single run with 1 basis
  !  function but lots of modes
  ! ***************************************************
  subroutine singlemode
    implicit none
    
    
    integer :: i,j,ns  ! for loops
    integer :: ierr   ! for errors

    character(len=150) :: fullname
        
    real,dimension(:),allocatable :: values
    integer :: dimvalues ! size of values
    
    character(len=100) :: stuff  ! 

    integer :: fm  ! input file

    complex(kind=8) :: dz
    real(kind=8) :: dq,dp

    write(*,*) "All modes in one trajectory"
    ! set file unit and file name
    fm = fmodesoffset+1
    if (trim(dirstem) .ne. "" ) then
       fullname = trim(dirstem) // "/" // ampfilenames
    else
       fullname= ampfilenames
    end if
	
    ! Open file
    open(fm,file=fullname,status="old",iostat=ierr)
    ! Check to see if we were successful
    if (ierr.ne.0) then
       write(*,*) "There was a problem opening file ",trim(fullname)
       stop
    else
       write(*,*) "Opened ",trim(fullname)
    end if
    ! read first line
    read(fm,*) stuff
    
    ! Allocate an array for reading in data
    dimvalues = firstcol -1 + 2*nmodes
    allocate(values(dimvalues))
    
    ! Open output file
    open(foutput,file=basisfile)
    write(foutput,*) "nstates",nbasis

    !Now we want to read in the basis functions
    do ns = 1,nbasis
       write(foutput,*) "state ",ns

       ! Read in two comment lines
       ! Only need to read in one as the other is blank
       ! it is automatically skipped
       read(fm,*) stuff 
          
          !	 read(fm,*) values(1),values(2),values(3),values(4),values(5)
          !(values(j),j=1,dimvalues)
          
       read(fm,*,iostat=ierr) (values(j),j=1,dimvalues)
       if(ierr .ne. 0) then
          write(*,*) "Error reading mode ",i, " step ",ns
          write(*,*) "Error number ", ierr
          stop
       end if
       
        ! Change the positions slightly?
       if(randomiselog) then
          do i = 1,nmodes
             dz = deltaz(rsigma(i))
             dq = dsqrt(2.0d0/gamma(i))*real(dz)
             dp = dsqrt(2.0d0*gamma(i))*dimag(dz)
             write(*,*) "dz = ",dz
             write(*,*) "dq,dp = ",dq,dp
             values(firstcol+i*2-2)= values(firstcol+i*2-2) + dq
             values(firstcol+i*2-1)= values(firstcol+i*2-1) + dp
          end do
       end if

       ! write out positions of each mode on separate lines
       do i = 0,nmodes-1
          write(foutput,*) values(firstcol+i*2),values(firstcol+i*2+1)
       end do
    end do
    
    deallocate(values)
    
  end subroutine singlemode

  
<<<<<<< .mine

  
  ! ******************** deltaz ***********************
  !  Calculate a random displacement for the states.
  !  Do it in terms of z as then the probability 
  !  distribution can have rotational symmetry.
  ! ***************************************************
  function deltaz(s)
    implicit none
    real(kind=8),intent(in) :: s
    complex(kind=8) :: deltaz
    real(kind=8) :: p,r,theta

    real(kind=8),external :: ZBQLU01
    ! Choose theta
    !call random_number(theta)
    theta = ZBQLU01()
    theta = theta*pi*2.0d0

    ! Choose r
    p = ZBQLU01()
!    call random_number(p)

    r = s*dsqrt(-1*dlog(pi*s**2*p))
    
    write(*,*) "theta,p", theta,p,pi*s**2*p,r    
    deltaz = r*cos(theta) + r*(0,1)*sin(theta)
    
  end function deltaz

=======

  
  ! ******************** deltaz ***********************
  !  Calculate a random displacement for the states.
  !  Do it in terms of z as then the probability 
  !  distribution can have rotational symmetry.
  ! ***************************************************
  function deltaz(s)
    implicit none
    real(kind=8),intent(in) :: s
    complex(kind=8) :: deltaz
    real(kind=8) :: p,r,theta

    ! Choose theta
    call random_number(theta)
    theta = theta*pi*2.0d0

    ! Choose r
    call random_number(p)

    r = s*dsqrt(-1*dlog(pi*s**2*p))
    
    write(*,*) "theta,p", theta,p,pi*s**2*p,r    
    deltaz = r*cos(theta) + r*(0,1)*sin(theta)
    
  end function deltaz

>>>>>>> .r23
end module data



! ******************** main program *****************
!  Basically just calls routines from elsewhere.
! ***************************************************
program trainbasis

  use data

  implicit none

  integer :: ierr ! for file errors


  ! Set name of input file
  controlfilename = "trains.inpt"

  ! try to open the input file
  open(fcont,file=controlfilename,status="old",iostat=ierr)
  if (ierr .eq. 29 ) then
     write(*,*) "Input file does not exist"
     call help
     stop
  else if ( ierr .ne. 0) then
     write(*,*) "Problem opening input file. Error number ",ierr
     call help
     stop
  end if

  ! Read data in from input file
  call readinputs
<<<<<<< .mine

  if(randomiselog) then
     call ZBQLINI(0)
     CALL RANDOM_SEED( )  
  end if

=======

  if(randomiselog) then
     CALL RANDOM_SEED( )  
  end if

>>>>>>> .r23
  ! do something useful
  select case (trajmode)
  case (1)
     call separatemodes
  case (2)
     call singlemode 
  end select
  
  write(*,*) "Bye bye"

end program trainbasis



