#!/usr/bin/perl

#           extractbasisfn
#              S. Reed
#          26th October 2007
#
# A wee script to extract individual basis functions from pos.out files


use strict;
use warnings; 



sub help
{

    print <<END_OF_TEXT;
        extractbasisfn

Assuming that basis functions are on individual lines and that each 
set begins with a line containing the word step then this wee script
will extract all the nth basis functions.

Usage:
   extractbasisfn filename bf

where bf is the basis function to extract. The program does not know 
the total number of basis functions so can not do any checking to see
if bf is less than this. Choosing bf greater than should result in 
simply lowering the temporal frequency of the points but this should
be checked if that is the desired beaviour.
END_OF_TEXT
    
}


# ============= Variables ==================

my $bf; # Which basis function
my $nbf; # How many basis functions are there
my $filename; # Name of pos.out file
my $INFILE; # File handle for pos.out file
my $i; # For loop
my $line; # For reading in a line from the pos.out file

my $numargs = @ARGV;
if ($numargs != 2 ){
print "$numargs\n";
    &help;
    exit;
}

# Get name for pos file
$filename = shift(@ARGV);
#Open pos.out file
if (!open($INFILE, $filename)) {
    die "Dying! Can't open ", $filename;
}


#total number of basis fns
#$nbf = shift(@ARGV);

# which basis fns
$bf = shift(@ARGV);

print "# Extracting basis fn number $bf\n";

while ($line = <$INFILE>){
  if ($line =~ /Step/){
    if ($bf == 1) {
      $line = <$INFILE>;
      print $line;
    }else{
      foreach $i  (1..$bf){
	$line = <$INFILE>;
      } 
      print $line;
    }
  }
}
